/***************************************************

	CodominantMarkerData.h
	
	Handles autosomal codominant genetic markers 
	
	Use different classes for dominant, haploid, sex-linked or haplodiploid markers
	
****************************************************/

#ifndef CodominantMarkerDatah
#define CodominantMarkerDatah = 1


#include "EvolutionaryGeneticsData.h"
#include "TextDatum.h"
#include "TextDataList.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "Randomness.h"
#include "Stringclass.h"
#include "StringList.h"
#include "GenotypeFreqDistribution.h"
#include "GenotypeFreqMultiAlleleDist.h"
#include "GeneFrequencyDistribution.h"
#include "GeneFrequencyDistMultiAllele.h"
#include "GeneContainers.h"
#include "ANOVAGenetics.h"
#include "GeneralLinearModel.h"
#include "aVector.h"
#include "aMatrix.h"
#include "MatrixList.h"
#include "PartitionedMatrix.h"
#include <fstream.h>
#include <iostream.h>
#include <math.h>
#include "InlineTemplates.h"
#include "MemoryManager.h"



	class CodominantMarkerData: public virtual EvolutionaryGeneticsData{
		public:
			long nlociCodom_;//omits skipped loci
			int ploidy_, alleleLen_;
			aList<int> allCodomLoci_,locusNumCodom_, skippedLocusNumCodom_;
					//these hold places in (=indices of) EvolutionaryGeneticsData::traitName_[]
			aList<int> variableLocusNumCodom_;//holds places in (=indices of) locusNumCodom_[]
			ObjList<aList<int> > codomIndivs_;//[pop][indiv]; holds places in rawTraits_
			ObjList<ObjList<ObjList<TextDataList> > > rawCodomGtypes_;//[pop][indiv][locus=trait][ploidy] - includes skipped loci
					//list of pops, each with list of individual genotype strings, sorted within individuals so that e.g., BA->AB
			ObjList<StringList> allelesPresent_;//loci correspond to order of locusNumCodom_[]
					//list of loci, each with list of all alleles encountered; skipped loci omitted
			ObjList<ObjList<aList<long double> > > alleleFreq_;//loci correspond to order of locusNumCodom_[]
					//list of pops, each with list of loci, each with list of allele freqs
			ObjList<aList<long double> > alleleFreqGlobal_;//loci correspond to order of locusNumCodom_[]
					//list of loci, each with list of allele freqs.  	UNWEIGHTED average over populations (use function GetFreq() for weighted average)
			ObjList<aList<int> > twoN_;	//locus order follows values in locusNumCodom_
					//list of pops, each with list of sample sizes for each locus
			ObjList<ObjList<ObjList<aList<int> > > > genotypeCount_;//[pop][loc][allele1][allele2]:loci correspond to order of locusNumCodom_[]; alleles in order of allelesPresent_	
					//list of pops, each with list of loci, each with list of frequencies below|on the diagonal;RETURNS RAW COUNTS
			ObjList<ObjList<aList<int> > > genotypeCountGlobal_;//[loc][allele1][allele2]:loci correspond to order of locusNumCodom_[]; alleles in order of allelesPresent_	
					//list of loci, each with list of frequencies below|on the diagonal; unweighted average over pops;RETURNS RAW COUNTS
			ObjList<ObjList<ObjList<aList<long double> > > > genotypeFreq_;//[pop][loc][allele1][allele2]:loci correspond to order of locusNumCodom_[]; alleles in order of allelesPresent_	
					//list of pops, each with list of loci, each with list of frequencies below|on the diagonal;RETURNS RELATIVE FREQUENCIES
			ObjList<ObjList<aList<long double> > > genotypeFreqGlobal_;//[loc][allele1][allele2]:loci correspond to order of locusNumCodom_[]; alleles in order of allelesPresent_	
					//list of loci, each with list of frequencies below|on the diagonal; unweighted average over pops;RETURNS RELATIVE FREQUENCIES
			ObjList<ObjList<aList<long double> > > HetObsByAllele_;//loci correspond to order of locusNumCodom_[]
					//list of pops, each with list of loci, each with list of allele freqs
			ObjList<aList<long double> > HetObsByLocus_;//loci correspond to order of locusNumCodom_[]
					//list of pops, each with list of observed heterozygosities for each locus 
			aList<long double> diseqD_;//for each pop;//loci correspond to order of locusNumCodom_[]
			ObjList<ObjList<aList<long double> > > locusD_;	//pairwise Ds within each pop
			int indivsCodomCurrent_;//boolean
			int varLociCurrent_;//boolean
			int allelesCurrent_;//boolean
			int alleleFreqCurrent_;//boolean
			int genotypeFreqCurrent_;//boolean
			int HetObsCurrent_;//boolean
			int DCurrent_;//boolean
					//list of pops, each with list of gametic disequilibria for each locus pair
		public:
		CodominantMarkerData(void):  EvolutionaryGeneticsData(), nlociCodom_(0), ploidy_(2),alleleLen_(1), indivsCodomCurrent_(0),
							varLociCurrent_(0), allelesCurrent_(0), alleleFreqCurrent_(0), HetObsCurrent_(0),DCurrent_(0) {}
		CodominantMarkerData(aString& s): EvolutionaryGeneticsData(s), nlociCodom_(0), ploidy_(2),alleleLen_(1), indivsCodomCurrent_(0),
							varLociCurrent_(0), allelesCurrent_(0), alleleFreqCurrent_(0), HetObsCurrent_(0),DCurrent_(0) {}
virtual	~CodominantMarkerData(void){
			Clear();
			}

		CodominantMarkerData(CodominantMarkerData& parentData);//copy constructor

		CodominantMarkerData(CodominantMarkerData& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	virtual EvolutionaryGeneticsData* Duplicate(void){//use for virtual copying of derived classes
			return NEW CodominantMarkerData(*this);}


	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(CodominantMarkerData& pgd);
	void  		Initialize(CodominantMarkerData& parentData, aList<int>& popSubsetNums);//makes a subset of pops

	virtual int alleleLen(void){return alleleLen_;}
	virtual int alleleID(int loc, TextDatum& alleleVal){//#of this allele in locus list
						if(!allelesCurrent_) {CheckForNewAlleles();}
						return allelesPresent_[loc].Loc(&alleleVal);}//location of the allele
	virtual int alleles(int loc){
						if(!allelesCurrent_) {CheckForNewAlleles();}
						return allelesPresent_[loc].len();}//number of alleles
	void		Calculate2LocusDisequilibrium(void);//using unskipped loci
	void		CalculateAlleleFreqs (void);//using unskipped loci
	ObjList<aList<long double> >* 
				CalculateAlleleFreqsByPopSet(aList<int>& popSet, aList<long double>& twoNList);
	void		CalculateGenotypeFreqs (void);//using unskipped loci
	void		CalculateObservedHeterozygosity (void);//using unskipped loci
	void		CalculateTwoN(void);//using unskipped loci
	void 		CheckForNewAlleles (void);//using unskipped loci
	virtual void		Clear(void);//clears all information
	void 		CountScoredIndividuals(void);
	int			codomIndividuals(int pop);
	int			codomIndividuals(int pop, int loc);
	void		CreateBootstrapData(CodominantMarkerData& origData, Ran3& ran);//individuals resampled
	void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void 		CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran);
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void 		CreateRandomDataLocFirst(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void 		CullLoci (aList<int>& lociToCull);
	void 		CullLoci (StringList& lociToCull);
	void 		CullLocus (aString& locusToCull);
	void 		CullLocus (char* locusToCull);
	void 		CullLocus (int locusToCull);
	void 		CullMonomorphicLoci (void);
	virtual void 		
				CullToOneGroup (EvolutionaryGeneticsData& parentData, int groupNum);//makes a subset data structure from the specified group
	void 		ExpungeLoci (StringList& lociToCull);//transfer to skipped
	void 		ExpungeLoci (aList<int>& lociToCull);
	void 		ExpungeLocus (aString& locusToCull);
	void 		ExpungeLocus (char* locusToCull);
	void	FitAllozymeCline(aString& dataFileName);
	TextDataList& 	GenotypicVal(int pop, int indiv, int trait);//using unskipped loci
	TextDatum& 	GenotypicVal(int pop, int indiv, int trait, int allele);//using unskipped loci
	int 		GenotypeSize(void){return alleleLen_ * ploidy_;}
	void 	GenotypeTable(ObjList<MatrixList>& gtypeTableList);//stored pops within loci; square matrices of genotypes
	long double 	GetExpectedHeterozygosity (int pop, int loc, int allele);//using unskipped loci
	long double 	GetExpectedHeterozygosity (int pop, int loc);//using unskipped loci
	long double 	GetExpectedHeterozygosity (int pop);//returns mean over unskipped loci
	long double 	GetFreq (int loc, int allele);//pooling populations, using unskipped loci:WEIGHTED AVERAGE USED -- DIFFERENT FROM alleleFreqGlobal_ info
	long double 	GetUnweightedFreq (int loc, int allele){
							if(!alleleFreqCurrent_){CalculateAlleleFreqs();}
							return alleleFreqGlobal_[loc][allele];}
	long double 	GetFreq (int pop, int loc, int allele);//using unskipped loci
	virtual void 	GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList);
							//uploads data to pass to genetic analysis classes
	virtual void	GetGenome(GeneContainerCore& gc, int pop, int indiv, bool setID=true);
						//uploads this indiv's data to MultiHaplotypeContainer<int>
	int			 	GetGenotypeCount (int pop, int loc, int allele1, int allele2){//using unskipped loci
						if(allele2<allele1) SWAP(allele1,allele2);
						if(!genotypeFreqCurrent_){CalculateGenotypeFreqs();}
						return genotypeCount_[pop][loc][allele1][allele2];
						}
	int			 	GetGenotypeCount (int loc, int allele1, int allele2){//using unskipped loci
						if(allele2<allele1) SWAP(allele1,allele2);//unweighted average over pops
						if(!genotypeFreqCurrent_){CalculateGenotypeFreqs();}
						return genotypeCountGlobal_[loc][allele1][allele2];
						}
	long double 	GetGenotypeFreq (int pop, int loc, int allele1, int allele2){//using unskipped loci
						if(allele2<allele1) SWAP(allele1,allele2);
						if(!genotypeFreqCurrent_){CalculateGenotypeFreqs();}
						return genotypeFreq_[pop][loc][allele1][allele2];
						}
	long double 	GetGenotypeFreq (int loc, int allele1, int allele2){//using unskipped loci
						if(allele2<allele1) SWAP(allele1,allele2);//unweighted average over pops
						if(!genotypeFreqCurrent_){CalculateGenotypeFreqs();}
						return genotypeFreqGlobal_[loc][allele1][allele2];
						}
	long double 	GetFreqMostCommonAllele (int loc);//using unskipped loci
	long double 	GetFreqMostCommonAllele (int pop, int loc);//using unskipped loci
//	aString* 	GetGenotypeString(int pop, int indiv);//drops code & returns new string; must be deleted elsewhere
//	aString* 	GetGenotypeString(int pop, int indiv, int loc);//returns new string; must be deleted elsewhere
//	aString* 	GetGenotypeString(int pop, int indiv, int loc, int chrom);//returns new string; must be deleted elsewhere
	long 		GetIndividualsSampled (int pop, int loc);
	void		GetLocusNames(StringList& names){
					TraitNamesFromSublist(names,locusNumCodom_);}
	long 		GetLocusSampleSize (int loc);//pooling populations
	long 		GetLocusSampleSize (int pop, int loc);//using unskipped loci
	int	 	GetLocusSampleSize (int pop, int indiv, int loc);//returns ploidy//using unskipped loci
	long double	GetMeanAllelesPerLocus (int pop);//using unskipped loci
	int	 	GetNumAlleles (int loc);//gives total in data set//using unskipped loci
	int	 	GetNumAlleles (int pop, int loc);//gives extant alleles in pop//using unskipped loci
	long double 	GetObservedHeterozygosity (int pop, int loc, int allele);//using unskipped loci
	long double 	GetObservedHeterozygosity (int pop, int loc);//using unskipped loci
	long double 	GetObservedHeterozygosity (int pop);//returns mean over loci//using unskipped loci
	long double 	GetPercentPolymorphicLoci (int pop, long double criterion = (long double)100.0);//using unskipped loci
	aString*		GetTraitString(int pop, int indiv, int loc, int allele);//loc indexed as in locusNumCodom_ 
	long double	GetVarJackAllelesPerLocus (int pop);//returns jackknife variance//using unskipped loci
	long double 	GetVarJackExpectedHeterozygosity (int pop);//returns jackknife variance//using unskipped loci
	long double 	GetVarJackObservedHeterozygosity (int pop);//returns jackknife variance//using unskipped loci
	long double 	GetVarJackPercentPolyLoci (int pop, long double criterion = (long double)100.0);//using unskipped loci
	virtual bool 	individualIsUnscored(int pop, int indiv, int loc);
	virtual int 	individuals(int pop){return EvolutionaryGeneticsData::individuals(pop);}
	int		IsPolymorphic(int loc);//boolean//using unskipped loci
	int		IsPolymorphic(int loc, long double criterion);//boolean//using unskipped loci
	int		IsPolymorphic(int pop, int loc);//boolean//using unskipped loci
	int		IsPolymorphic(int pop, int loc, long double criterion);//boolean//using unskipped loci
	virtual int 	loci(void){return nlociCodom_;}// unskipped loci
	virtual int 	lociSkipped(void){return skippedLocusNumCodom_.len();}
	virtual aString& locusName(int loc){ return traitName_[locusNumCodom_[loc]];}
	virtual long 	maxall(void);//maximum alleles per locus in data set
//	virtual void 	ParseGenotypicData(aList<int>& codominantLoci, int startingLocus=0);
	virtual void 	ParseGenotypicData(int startingLocus=0);
							//convert from raw trait data
							//startingLocus = the first trait that contains genotypes
	virtual int 	ploidy(void){return ploidy_;}
	virtual int 	Ploidy(int /*pop*/, int /*indiv*/, int /*loc*/){return ploidy_;}
	virtual void 		PrintAlleleFreqs (ostream& outFile, int ncols, char colSpacer = ' ');//outfile assumed open
	void  		PrintAlleleFreqsByPopList (ostream& outFile, 
										ObjList<ObjList<aList<long double> > >& popFreqList, 
										ObjList<aList<long double> >& popTwoNList, 
										int ncols);
	virtual void	 	PrintArlequinFile(aString& outFileName);
	virtual void 		PrintArlequinFile (ostream& outFile);
	virtual void	 	PrintGelJockFile(aString& outFileName);
	virtual void 		PrintGelJockFile (ostream& outFile);
	void 		PrintHeterozygosity (ostream& outFile, const int hetType, int ncols);//outfile assumed open
	virtual void 		PrintLoci (ostream& outFile, int lociPerLine);//outfile assumed open//using unskipped loci
	virtual void		PrintSASFormatData(aString& outFileName, char colSpacer = '\t');//output file assumed closed
	virtual void		PrintSASFormatData(ostream& outFile, char colSpacer = '\t');//outfile assumed open
				//missing data represented by '.', columns separated by spaces
	virtual void 		PrintVariabilityScores(aString& outFileName, char sepChar='\0');
	virtual void 		PrintVariabilityScores(ostream& outFile, char sepChar='\0');
	virtual int 		ReadArlequinDataFile (aString* dataFileName);//input file assumed closed
	virtual int 		ReadGelJockDataFile (aString* dataFileName);//input file assumed closed
	virtual void		ResetFlags(void){
							EvolutionaryGeneticsData::ResetFlags();//just in case the base class adds any
							CodominantMarkerData::ResetGeneticFlags();}
	virtual void		ResetGeneticFlags(void){
							indivsCodomCurrent_=varLociCurrent_=allelesCurrent_=alleleFreqCurrent_=HetObsCurrent_=DCurrent_=0;}
	virtual void 		RestoreLoci (aList<int>& lociToRestore);
	virtual void 		RestoreLoci (StringList& lociToRestore);
	virtual void 		RestoreLocus (aString& locusToRestore);
	virtual void 		RestoreLocus (char* locusToRestore);
	virtual void 		RestoreLocus (int locusToRestore);
	virtual void 		SnipArlequinComment(aString& arqStr);
	virtual void		SetAlleleLength(int allelelength){
							alleleLen_=allelelength;}
	virtual void 		SetGenome(ObjList<PopulationGeneContainer<int> >& pgcList);
							//records all data, passed along from genetic analysis classes
							//empties old data and creates new data structures
	virtual void 		SetGenome(GeneContainerCore& gc, int pop, int indiv);
							//records data from MultiHaplotypeContainer<int>
							//assumes blank data records already created in EvolutionaryGeneticsData
	void				SetLoci(aList<int>& lociToInclude);//constrains the diploid data set to these loci only
							//skipped loci status unchanged (use CullLoci to skip more; RestoreLoci to skip fewer)
	virtual int 		traits(void){return locusNumCodom_.len();}	
	virtual int 		traits(int /*traitType*/){return CodominantMarkerData::traits();}	
	virtual void		UpdateVariableLoci(void);//using unskipped loci
	virtual aString&	VariableLocusName(int loc){
							return traitName_[locusNumCodom_[variableLocusNumCodom_[loc]]];}
	CodominantMarkerData& operator=(CodominantMarkerData& pgd);
	};//end class CodominantMarkerData
		
		
#endif

