/*****************************************


	 EvolutionaryGeneticsData.h

	This is a base class to handle general features of all types of genetic data that can be
	analyzed in evolutionary genetics.  Data are classified by population, individual and trait.
	
	Derived classes are used to handle traits of different types, including codominant markers,
	dominant markers, sex-linked codominant markers, sex-linked dominant markers, haploid markers
	such as Y-chromosomes and cytoplasmic elements, quantitative traits, and DNA or protein sequences.   
	Each derived class should handle analysis of its own trait type, and combined analyses over 
	different trait types need special classes as well.
	
	Data here are read in from the data file and parsed as text.  Derived classes convert the text data
	into appropriate data formats for analysis.


******************************************/

#ifndef EvolutionaryGeneticsDatah
#define EvolutionaryGeneticsDatah = 1


#include "aError.h"
#include "TextDatum.h"
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "aVector.h"
#include "aMatrix.h"
#include "StringList.h"
#include "TextDataList.h"
#include <fstream.h>
#include <iostream.h>
#include <math.h>
#include "InlineTemplates.h"
#include "MemoryManager.h"
#include "Randomness.h"
#include "PopulationIntf.h"
#include "GeneContainers.h"



const char endChr = '\n';
//const char endChr = '\r';

	class EvolutionaryGeneticsData{
		public://hetTypes
			const static int observed=0;
			const static int expected=1;
		public:
			aString title_;	//title_ of the data set from the first data line
			long npop_, ngroups_, ntraits_;
			StatList<int> popSize_;
			StringList popName_, groupName_, localityName_;//grouped by last word in title; locality name determined by first word
			ObjList<aList<int> > groupHierarchy_, popsByLocality_;	
					//each group gets its own list of populations, by number
			aMatrix<long double> localityCoordinates_;//stores as many dimensions as needed; determined by what is in (parentheses) in the population line
			StringList traitName_;//traitName_[0] is always the indvidual's id number
			aList<int> traitNum_, skippedTraitNum_;//keep these sorted
			ObjList<ObjList<TextDataList> > rawTraits_;
					//as read from data file, parsed into array of [pop][indiv][trait]
					//traits are stored in form of TextDatum to handle missing data
			StringList missing_;//characters|strings interpreted as missing data; the first will be used for printout; default is "?"
		public:
		EvolutionaryGeneticsData(void): npop_(0), ngroups_(0), ntraits_(0) {}
		EvolutionaryGeneticsData(aString& s): title_(s), npop_(0), ngroups_(0), ntraits_(0) {}
virtual	~EvolutionaryGeneticsData(void){
			Clear();
			}

		EvolutionaryGeneticsData(EvolutionaryGeneticsData& sourceData);//copy constructor

		EvolutionaryGeneticsData(EvolutionaryGeneticsData& sourceData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	
	virtual EvolutionaryGeneticsData* Duplicate(void){//use for virtual copying of derived classes
			return NEW EvolutionaryGeneticsData(*this);}
			
			
	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(EvolutionaryGeneticsData& egd);
	void		Initialize(EvolutionaryGeneticsData& egd, aList<int>& popSubsetNums);

	virtual void		Clear(void);//clears all information
	void		CreateBootstrapData(EvolutionaryGeneticsData& origData, Ran3& ran);//individuals resampled
	void		CreateBootstrapDataConstPop(EvolutionaryGeneticsData& origData, Ran3& ran);//individuals resampled
	void 		CullTraits (aList<int>& traitsToCull);
	void 		CullTraits (StringList& traitsToCull);
	void 		CullTrait (aString& traitToCull);
	void 		CullTrait (char* traitToCull);
	void 		CullTrait (int traitToCull);
	EvolutionaryGeneticsData* 		
				CullToOneGroup (int groupNum);//makes a subset data structure from the specified group
	virtual void		
				CullToOneGroup (EvolutionaryGeneticsData& parentData, int groupNum);//makes a subset data structure from the specified group
	virtual void	GetGenome(GeneContainerCore& /*gc*/, int /*pop*/, int /*indiv*/, bool /*setID=true*/){
						//refills container class at appropriate level
						aError("Error: base class EvolutionaryGeneticData has no genome");}
	int		GetIndivNumInDataSequence(int pop, int indiv);
				//returns a surrogate ID number, when numerical ID is needed: 0-indexed
	long 	GetIndividualsSampled (void){ return popSize_.sum();}
	long 	GetIndividualsSampled (int pop){return popSize_[pop];}
	long 	GetIndividualsSampled (int pop, int trait);
	int	 	GetLocality (int pop);
	long 		GetTraitSampleSize (int trait);//pooling populations
	long 		GetTraitSampleSize (int pop, int trait);
	virtual void 		GroupMembership (void);
	virtual int 	 	GroupMembership (int pop);
	virtual int 		individuals(int pop){return rawTraits_[pop].len();}
	virtual int 		individuals(int pop, int trait);//handles missing data
	virtual bool		isSkipped(int traitnum){
								return skippedTraitNum_.Includes(traitnum);}
	virtual bool		isSkipped(aString& traitname){
								if(! traitName_.Includes(&traitname)) return false;
								int tr = traitName_.Loc(&traitname);
								return isSkipped(tr);
								}
	int 		localities(void){return popsByLocality_.len();}
	void 		LocalityMembership (void);
	int	 		LocalityMembership (int pop);
	virtual int 		popSize(int pop){ return popSize_[pop];}
	virtual void 		PrintTraits (ostream& outFile, int traitsPerLine);//outfile assumed open
	virtual void 		PrintPopKey (ostream& outFile, char colSpacer = '\t');//outfile assumed open
	virtual void		PrintSASFormatData(aString& outFileName,char colSpacer = '\t');//output file assumed closed
	virtual void		PrintSASFormatData(ostream& outFile,char colSpacer = '\t');//outfile assumed open
				//missing data represented by '.', columns separated by spaces
	virtual void		ResetFlags(void){}//meaningful in derived classes
	virtual void		ResetGeneticFlags(void){}//meaningful in derived classes
	virtual void 		RestoreTraits (aList<int>& traitsToRestore);
	virtual void 		RestoreTraits (StringList& traitsToRestore);
	virtual void 		RestoreTrait (aString& traitToRestore);
	virtual void 		RestoreTrait (char* traitToRestore);
	virtual void 		RestoreTrait (int traitToRestore);
	virtual void	SetGenome(GeneContainerCore& /*gc*/, int /*pop*/, int /*indiv*/){//initializes dataset using appropriate level of container class
						aError("Error: base class EvolutionaryGeneticData has no genome");}
	void 		SetMissingDataString(aString& missing);
	aString& 	Trait(int pop, int indiv, int trait);
	virtual int	traitNum(aString& trName){
						if(traitName_.Includes(&trName)){
							return traitName_.Loc(&trName);}
						return -1;//error code
						}
	virtual int traits(void){return traitNum_.len();}	
	virtual int traits(int /*traitType*/){return traitNum_.len();}	
			//use for printing; skips traits appropriately;
			//returns 'missing character' for indivs without data
	virtual void	TraitNamesFromSublist(StringList& names, aList<int>& itemNumbers);//use to get trait names
//	virtual long 	traits(void){return traitNum_.len();}
	void 		UpdateMissingDataFlags(StringList& useForMissing);
	EvolutionaryGeneticsData& operator=(EvolutionaryGeneticsData& pgd);
	};//end class EvolutionaryGeneticsData
		
		
#endif

