/***************************************************

	MultiformatMarkerData.h
	
	Handles genetic markers of varying inheritance patterns in the same individuals
	Quantitative trait data supported, but not yet explicitly handled
	
****************************************************/

#ifndef MultiformatMarkerDatah
#define MultiformatMarkerDatah = 1


#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "EvolutionaryGeneticsData.h"
#include "SexLinkedData.h"
#include "HaploidMarkerData.h"
#include "CodominantMarkerData.h"
#include "DominantMarkerData2.h"
#include "SexLinkedCodomMarkerData.h"
#include "CytoplasmicFemaleMarkerData.h"
#include "CytoplasmicMaleMarkerData.h"
#include "SexLinkedDomMarkerData2.h"
#include "GeneContainers.h"
#include "TextDatum.h"
#include "TextDataList.h"
#include "StatList.h"
#include "GenotypeFreqDistribution.h"
#include "GenotypeFreqDistDom.h"
#include "GenotypeFreqDistSexLinkDom.h"
#include "GenotypeFreqMultiAlleleDist.h"
#include "GeneFrequencyDistribution.h"
#include "GeneFrequencyDistMultiAllele.h"
#include "ANOVAGenetics.h"
#include "GeneralLinearModel.h"
#include "aVector.h"
#include "aMatrix.h"
#include "MatrixList.h"
#include "PartitionedMatrix.h"
#include <fstream.h>
#include <iostream.h>
#include <math.h>
#include "InlineTemplates.h"
#include "MemoryManager.h"

//#ifndef DEBUG_SORTTRAITS
//#define DEBUG_SORTTRAITS=1
//#endif


//20081202:  updated CreateRandomData to adjust for gamete (default) or zygote migration
//20081202:  updated CreateRandomData to allow scaling for the number of demes in the total (not sampled) population
				//default=infinite (unscaled)
//20081208:  Bug fix: Modified the maxall() function to look over all inheritance types
//20081209:  Bug fix: Modified the CreateRandomData functions to properly fill in missing data for CYfemaleOnly and CYmaleOnly data
//20081210:  Bug fix: SortTraitsInteractively() bad index in SLcodom trait conflict resolution loop


enum traitOrder {ID=0,sex=1,hap=2,codom=3,dom=4,SLcodom=5,SLdom=6,CYfemale=7,CYmale=8,CYfemaleOnly=9,CYmaleOnly=10};

	class MultiformatMarkerData: public virtual HaploidMarkerData,
								 public virtual CodominantMarkerData,
								 public virtual DominantMarkerData2,
								 public virtual SexLinkedCodomMarkerData,
								 public virtual SexLinkedDomMarkerData2,
								 public virtual CytoplasmicFemaleMarkerData,
								 public virtual CytoplasmicMaleMarkerData,
								 public virtual SexLinkedData,
								 public virtual EvolutionaryGeneticsData
								 {
		public:
			static const int numGeneticMarkerTypes_=9;
			static const int numTraitTypes_=11;//id, sex, genetic markers
		public:
			aList<int> traitTypesIncluded_;//T or F: use to cross-reference traits into appropriate subclasses; skipped traits included
					//0=unspecified,1=sex,2=haploid nuclear,3=codominant,4=dominant,5=sex-linked codominant,6=sex-linked dominant,7=cytoplasmic female,8=cytoplasmic male
			aList<int> traitType_;//[index by trait]: use to cross-reference traits into appropriate subclasses; skipped traits included
					//0=unspecified,1=sex,2=haploid,3=codominant,4=dominant,5=sex-linked codominant,6=sex-linked dominant,7=cyto female,8=cyto male
			bool traitTypeCurrent_;//
			TextDatum dummytextDatum_;
			TextDataList dummytextDataList_;
		public:
		MultiformatMarkerData(void):traitTypeCurrent_(false){
							for(int t=0;t<numTraitTypes_;++t){traitTypesIncluded_.Append(false);}
							}
		MultiformatMarkerData(aString& s):traitTypeCurrent_(false){ 
							EvolutionaryGeneticsData::Initialize(s);
							for(int t=0;t<numTraitTypes_;++t){traitTypesIncluded_.Append(false);}
							}
virtual	~MultiformatMarkerData(void){
			Clear();
			}

		MultiformatMarkerData(MultiformatMarkerData& parentData);//copy constructor
		MultiformatMarkerData(HaploidMarkerData& parentData);//copy constructor
		MultiformatMarkerData(CodominantMarkerData& parentData);//copy constructor
		MultiformatMarkerData(DominantMarkerData2& parentData);//copy constructor
		MultiformatMarkerData(SexLinkedCodomMarkerData& parentData);//copy constructor
		MultiformatMarkerData(SexLinkedDomMarkerData2& parentData);//copy constructor
		MultiformatMarkerData(CytoplasmicFemaleMarkerData& parentData);//copy constructor
		MultiformatMarkerData(CytoplasmicMaleMarkerData& parentData);//copy constructor

		MultiformatMarkerData(MultiformatMarkerData& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	virtual EvolutionaryGeneticsData* Duplicate(void){//use for virtual copying of derived classes
			return NEW MultiformatMarkerData(*this);}


	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(MultiformatMarkerData& pgd);
	void  		Initialize(MultiformatMarkerData& parentData, aList<int>& popSubsetNums);//makes a subset of pops

	virtual void	AddTraits(int traitType, aList<int>& traitNumbers);//adds traits 
	int			alleleLen(void){//all are set the same
					return HaploidMarkerData::alleleLen_;}
	virtual int alleleID(int loc, TextDatum& alleleVal);//#of this allele in locus list  -- loci ordered H,C,D,SLC,SLD,CYF,CYM
	virtual int alleles(int loc);//number of alleles  -- loci ordered H,C,D,SLC,SLD,CYF,CYM
//	void		Calculate2LocusDisequilibrium(void);//using unskipped loci
	void		CalculateAlleleFreqs (void);//using unskipped loci
	ObjList<ObjList<aList<long double> > >*		////[trtype][pop][loc][allele]
				CalculateMarkerFreqsByPopSet(aList<int>& popSet, 
									ObjList<aList<long double> >& sampleSizeListbyTypeAndLoc, bool splitCytotypeBySex);
					//combines CYfemaleOnly with CYfemale, and combines CYmaleOnly with CYmale; CYfemaleOnly and CYmaleOnly left empty
	void		CalculateObservedHeterozygosity (void);//using unskipped loci
	void		CalculateTwoN(void);//using unskipped loci
	void 		CheckForNewAlleles (void);//using unskipped loci
	virtual void		Clear(void);//clears all information
	bool		codominantDataIncluded(void){return (bool) traitTypesIncluded_[codom];}
	bool		codominantDataUnskipped(void){return ((bool) traitTypesIncluded_[codom]) && nlociCodom_>0;}
	bool		codominantSexLinkedDataIncluded(void){return (bool) traitTypesIncluded_[SLcodom];}
	bool		codominantSexLinkedDataUnskipped(void){return ((bool) traitTypesIncluded_[SLcodom]) && nlociSLCodom_>0;}
	void 		ConvertCodominantData(MultiformatMarkerData& mfData);//never use self-reference mfData=*this
	void		CreateBootstrapData(MultiformatMarkerData& origData, Ran3& ran);//individuals resampled
	virtual void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,
							long double sexratio=half,
							int numTrueDemes=0,//<2 -> infinite, >1 is scaled
							bool zygoteMigration=false);
	virtual void 		CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<ObjList<aList<long double> > >& pBar, //[datatype][loc][allele]
							ObjList<ObjList<aList<int> > >& numIndivsPerPopLocDataType,//[pop][datatype][loc]
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,
							long double sexratio=half,
							int numTrueDemes=0,//<2 -> infinite, >1 is scaled
							bool zygoteMigration=false);
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,
							long double sexratio=half,
							int numTrueDemesWithinSpecies=0,//<2 -> infinite, >1 is scaled
							int numTruDemesBtweenSpecies=2,//<2 -> infinite, >1 is scaled
							bool zygoteMigration=false);
/*	virtual void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,
							aList<int>& lociOnlyInFemales, aList<int>& lociOnlyInMales);
	virtual void 		CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<ObjList<aList<long double> > >& pBar, //[datatype][loc][allele]
							ObjList<ObjList<aList<int> > >& numIndivsPerPopLocDataType,//[pop][datatype][loc]
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,
							aList<int>& lociOnlyInFemales, aList<int>& lociOnlyInMales);
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,
							aList<int>& lociOnlyInFemales, aList<int>& lociOnlyInMales);
//	void 		CreateRandomData(const long double Fspt,//differentation among species
//							const long double Flt, //differentiation among localities
//							const long double Fis,
//							const long double pBar, //grand mean
//							const int numSpeciesSampled,
//							const int numLocalitiesSampled,
//							const int numIndivsPerPop,
//							StatList<int>& numLociSampledByDataType,
//							Ran3List& ran);
//	void 		CreateRandomDataLocFirst(const long double Fspt,//differentation among species
//							const long double Flt, //differentiation among localities
//							const long double Fis,
//							const long double pBar, //grand mean
//							const int numSpeciesSampled,
//							const int numLocalitiesSampled,
//							const int numIndivsPerPop,
//							StatList<int>& numLociSampledByDataType,
//							Ran3List& ran);
*/
	virtual void 		CullLoci (aList<int>& lociToCull);
	virtual void 		CullLoci (StringList& lociToCull);
	virtual void 		CullLocus (aString& locusToCull);
	virtual void 		CullLocus (char* locusToCull);
	virtual void 		CullLocus (int locusToCull);
	virtual void 		CullMonomorphicLoci (void);
	virtual void 		
				CullToOneGroup (EvolutionaryGeneticsData& parentData, int groupNum);//makes a subset data structure from the specified group
	void 		CullToOneSex (MultiformatMarkerData& parentData, genderSet sexVal);//makes a subset data structure from the specified group
	void 		CullToTheseSexes (MultiformatMarkerData& parentData, aList<int>& sexValList);//makes a subset data structure from the specified group
	bool 		cytoplasmicFemaleMarkersIncluded(void){
				 	return (bool) traitTypesIncluded_[CYfemale] || traitTypesIncluded_[CYfemaleOnly];}//includes female-specific sex markers
	bool 		cytoplasmicFemaleMarkersUnskipped(void){
					return (bool) (traitTypesIncluded_[CYfemale] || traitTypesIncluded_[CYfemaleOnly]) && nlociCYfemale_>0;}//includes female-specific sex markers
	bool 		cytoplasmicMaleMarkersIncluded(void){ 
					return (bool) traitTypesIncluded_[CYmale] || traitTypesIncluded_[CYmaleOnly];}//includes male-specific sex markers
	bool 		cytoplasmicMaleMarkersUnskipped(void){ 
					return (bool) (traitTypesIncluded_[CYmale] || traitTypesIncluded_[CYmaleOnly]) && nlociCYmale_>0;}//includes male-specific sex markers
	virtual void		DetermineFemaleOnlyTraits(void);
	virtual void		DetermineMaleOnlyTraits(void);
	bool		dominantDataIncluded(void){return (bool) traitTypesIncluded_[dom];}
	bool		dominantDataUnskipped(void){return ((bool) traitTypesIncluded_[dom]) && nlociDom_>0;}
	bool		dominantSexLinkedDataIncluded(void){return (bool) traitTypesIncluded_[SLdom];}
	bool		dominantSexLinkedDataUnskipped(void){return ((bool) traitTypesIncluded_[SLdom]) && nlociSLDom_>0;}
	void	FitAllozymeCline(aString& dataFileName);
	TextDataList& 	GenotypicVal(int pop, int trait, int loc);//using unskipped loci
	TextDatum& 	GenotypicVal(int pop, int indiv, int trait, int allele);//using unskipped loci
	int 		GenotypeSize(int loc);//loc is the trait number
	void 	GenotypeTable(ObjList<MatrixList>& gtypeTableList);//stored pops within loci; square matrices of genotypes
	long double 	GetExpectedHeterozygosity (int pop, int loc, int allele);//using unskipped loci
	long double 	GetExpectedHeterozygosity (int pop, int loc);//using unskipped loci
	long double 	GetExpectedHeterozygosity (int pop);//returns mean over unskipped loci
	long double 	GetFreq (int loc, int allele);//pooling populations, using unskipped loci
	long double 	GetFreq (int pop, int loc, int allele);//using unskipped loci
	long double 	GetFreqMostCommonAllele (int pop, int loc);//using unskipped loci
	virtual void 	GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList);
							//uploads data to pass to genetic analysis classes
	virtual void 	GetGenome(GeneContainerCore& gc, int pop, int indiv, bool setID=true);
							//uploads this indiv's data to MultiInheritanceModeContainer<int>
	long 		GetIndividualsSampled (int pop, int loc);
	long 		GetIndividualsSampled (int traitType, int pop, int loc);
	long 		GetLocusSampleSize (int loc);//pooling populations
	long 		GetLocusSampleSize (int pop, int loc);//using unskipped loci
	int	 	GetLocusSampleSize (int pop, int indiv, int loc);//returns ploidy//using unskipped loci
//	long double	GetMeanAllelesPerLocus (int pop);//using unskipped loci
	int	 	GetNumAlleles (int loc);//gives total in data set//using unskipped loci
	int	 	GetNumAlleles (int pop, int loc);//gives extant alleles in pop//using unskipped loci
	long double 	GetObservedHeterozygosity (int pop, int loc, int allele);//using unskipped loci
	long double 	GetObservedHeterozygosity (int pop, int loc);//using unskipped loci
	long double 	GetObservedHeterozygosity (int pop);//returns mean over loci//using unskipped loci
	long double 	GetPercentPolymorphicLoci (int pop, double criterion = 100.0);//using unskipped loci
//	aString* 	GetGenotypeString(int pop, int indiv);//drops code & returns new string; must be deleted elsewhere
	aString*	GetTraitName(int traitType, int trNumWithinType);//ignores skipped traits					
	int			GetTraitNum(int traitType, int trNumWithinType);//ignores skipped traits					
	aString* 	GetTraitString(int pop, int indiv, int loc, int allele);//loc=item in total list; returns new string; must be deleted elsewhere
//	aString* 	GetGenotypeString(int pop, int indiv, int loc, int chrom);//returns new string; must be deleted elsewhere
//	long double	GetVarJackAllelesPerLocus (int pop);//returns jackknife variance//using unskipped loci
//	long double 	GetVarJackExpectedHeterozygosity (int pop);//returns jackknife variance//using unskipped loci
//	long double 	GetVarJackObservedHeterozygosity (int pop);//returns jackknife variance//using unskipped loci
//	long double 	GetVarJackPercentPolyLoci (int pop, double criterion = 100.0);//using unskipped loci
	bool			haploidDataIncluded(void){return (bool) traitTypesIncluded_[2];}
	bool			haploidDataUnskipped(void){return ((bool) traitTypesIncluded_[2]) && nlociHap_>0;}
	virtual int 	individuals(int pop){return EvolutionaryGeneticsData::individuals(pop);}
	virtual int 	individuals(int pop, int loc);//overrides base class
	int		IsPolymorphic(int pop, int loc);//boolean//using unskipped loci
	int		IsPolymorphic(int pop, int loc, double criterion);//boolean//using unskipped loci
	virtual int 	loci(void);// unskipped loci
	virtual int 	loci(int traitType);// unskipped loci
	virtual int 	lociSkipped(void);
	virtual aString& locusName(int loc);// -- loci ordered H,C,D,SLC,SLD,CYF,CYM
	virtual int		markerTypes(void){return (int) numTraitTypes_;}
	virtual long 	maxall(void);//maximum alleles per locus in data set

	virtual void 	MergeDataSets(MultiformatMarkerData& dataToAppend);//sorts by population name and ID number
	virtual void 	ParseGenotypicData(void);
//	virtual void 	ParseGenotypicData(aList<int>& /*codominantLoci*/, int /*startingLocus=1*/){ParseGenotypicData();}
	virtual void 	ParseGenotypicData(int /*startingLocus=1*/){ParseGenotypicData();}
							//convert from raw trait data
							//startingLocus = the first trait that contains genotypes
	virtual int 	ploidy(void){//ploidy is the same among data types except haploids
						return CodominantMarkerData::ploidy_;}
	virtual int 	ploidy(int pop, int indiv){
						if(dominantSexLinkedDataIncluded() || codominantSexLinkedDataIncluded()){
								if((! codominantDataIncluded()) && (! dominantDataIncluded())){
										//treat as haplodiploid system
										if(heterogametic(pop,indiv)){return HaploidMarkerData::ploidy();}}}
							else{
								if((! codominantDataIncluded()) && (! dominantDataIncluded()) && haploidDataIncluded()){
									//treat as haplopid
									return HaploidMarkerData::ploidy();}}
						//treat as diploid
						return ploidy();
						}
	virtual int 	Ploidy(int loc);
	virtual int 	Ploidy(int pop, int indiv, int loc);
	long double 	PloidyEffective(void){
						if(sexLinkedDataIncluded()){return SexLinkedData::PloidyEffective();}
						return ploidy();}
	void 		PrintAlleleFreqs (ostream& outFile, int ncols, char colSpacer = ' ');//outfile assumed open
//	void  		PrintAlleleFreqsByPopList (ostream& outFile, 
//										ObjList<ObjList<aList<double> > >& popFreqList, 
//										ObjList<aList<double> >& popTwoNList, 
//										int ncols);
	virtual void	 	PrintArlequinFile(aString& outFileName);
	virtual void 		PrintArlequinFile (ostream& outFile);
	virtual void	 	PrintGelJockFile(aString& outFileName);
	virtual void 		PrintGelJockFile (ostream& outFile);
	void 		PrintHeterozygosity (ostream& outFile, const int hetType, int ncols);//outfile assumed open
//	virtual void 		PrintLoci (ostream& outFile, int lociPerLine);//outfile assumed open//using unskipped loci
	virtual void		PrintSASFormatData(aString& outFileName, char colSpacer = '\t');//output file assumed closed
	virtual void		PrintSASFormatData(ostream& outFile, char colSpacer = '\t');//outfile assumed open
				//missing data represented by '.', columns separated by spaces
	virtual void 		PrintVariabilityScores(aString& outFileName, char sepChar='\0');
	virtual void 		PrintVariabilityScores(ostream& outFile, char sepChar='\0');
	virtual int 		ReadArlequinDataFile (aString* dataFileName);//input file assumed closed
	virtual int 		ReadGelJockDataFile (aString* dataFileName);//input file assumed closed
	virtual int 		ReadGelJockmtDNADataFile (aString* dataFileName){//input file assumed closed
							return HaploidMarkerData::ReadHaploidDataFile(*dataFileName);}
	virtual void		ReclassifyTraits(int oldTraitType, int newTraitType);
	virtual void		RemoveTraits(int traitType, aList<int>& traitNumbers);//removes traits 
	virtual void		ResetFlags(void){
								HaploidMarkerData::ResetFlags();
								CodominantMarkerData::ResetFlags();
								DominantMarkerData2::ResetFlags();
								SexLinkedCodomMarkerData::ResetFlags();
								SexLinkedDomMarkerData2::ResetFlags();
								CytoplasmicFemaleMarkerData::ResetFlags();
								CytoplasmicMaleMarkerData::ResetFlags();
								traitTypeCurrent_=0;}
	virtual void		ResetGeneticFlags(void){//skips sex-linked and trait-type flags
								HaploidMarkerData::ResetGeneticFlags();
								CodominantMarkerData::ResetGeneticFlags();
								DominantMarkerData2::ResetGeneticFlags();
								SexLinkedCodomMarkerData::ResetGeneticFlags();
								SexLinkedDomMarkerData2::ResetGeneticFlags();
								CytoplasmicFemaleMarkerData::ResetGeneticFlags();
								CytoplasmicMaleMarkerData::ResetGeneticFlags();
								}
	virtual void 		RestoreLoci (aList<int>& lociToRestore);
	virtual void 		RestoreLoci (StringList& lociToRestore);
	virtual void 		RestoreLocus (aString& locusToRestore);
	virtual void 		RestoreLocus (char* locusToRestore);
	virtual void 		RestoreLocus (int locusToRestore);
	virtual void		SetAlleleLength(int allelelength){
							HaploidMarkerData::alleleLen_=allelelength;
							CodominantMarkerData::alleleLen_=allelelength;
							DominantMarkerData2::alleleLen_=allelelength;
							SexLinkedCodomMarkerData::alleleLen_=allelelength;
							SexLinkedDomMarkerData2::alleleLen_=allelelength;
							CytoplasmicFemaleMarkerData::alleleLen_=allelelength;
							CytoplasmicMaleMarkerData::alleleLen_=allelelength;
							}
	virtual void 		SetGenome(ObjList<PopulationGeneContainer<int> >& pgcList);
							//records all data, passed along from genetic analysis classes
							//empties old data and creates new data structures
	virtual void 		SetGenome(GeneContainerCore& gc, int pop, int indiv);
							//records this indiv's data from MultiInheritanceModeContainer<int>
							//assumes empty data structure already created in EvolutionaryGeneticData
	virtual void		SetPloidy(int ploidyLevel){
							CodominantMarkerData::ploidy_=ploidyLevel;
							DominantMarkerData2::ploidy_=ploidyLevel;
							SexLinkedCodomMarkerData::ploidy_=ploidyLevel;
							SexLinkedDomMarkerData2::ploidy_=ploidyLevel;
							}
	virtual bool		sexLinkedDataIncluded(void){
								return (codominantSexLinkedDataIncluded() || dominantSexLinkedDataIncluded() ||
										cytoplasmicFemaleMarkersIncluded() || cytoplasmicMaleMarkersIncluded() ||
										femaleLimitedTraitsIncluded() || maleLimitedTraitsIncluded());}
	virtual void 		SnipArlequinComment(aString& arqStr);
	virtual void		SortTraits(int traitType, aList<int>& traitList);
	virtual void 		SortTraitsInteractively(ObjList<aList<int> >& presortedTraits);//confirms previous classification with user
	virtual void 		TryToSortTraits(void);//look at data and classify

	virtual int			traitIncluded(int traitNumGlobal){return traitNum_.Includes(traitNumGlobal);}
	virtual int 		traits(void){return EvolutionaryGeneticsData::traits();}	
	virtual int 		traits(int traitType);	
	virtual int			GetTraitType(int traitNumGlobal){ return traitType_[traitNumGlobal];}
	virtual void		UpdateVariableLoci(void);//using unskipped loci
	virtual aString&	VariableLocusName(int loc){
							return traitName_[locusNumCodom_[variableLocusNumCodom_[loc]]];}
	MultiformatMarkerData& operator=(MultiformatMarkerData& pgd);
	};//end class 
		
		
#endif

