/***************************************************

	SexLinkedCodomMarkerData.h
	
	Handles codominant genetic markers with sex linkage or haplodiploidy
	Assumes that one of the traits is 'sex', labeled M or F for each individual
	
	Use different classes for dominant, haploid, or autosomal markers
	
****************************************************/

#ifndef SexLinkedCodomMarkerDatah
#define SexLinkedCodomMarkerDatah = 1


#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "EvolutionaryGeneticsData.h"
#include "SexLinkedData.h"
#include "TextDatum.h"
#include "TextDataList.h"
#include "StatList.h"
#include "GenotypeFreqDistribution.h"
#include "GenotypeFreqMultiAlleleDist.h"
#include "GeneFrequencyDistribution.h"
#include "GeneFrequencyDistMultiAllele.h"
#include "ANOVAGenetics.h"
#include "GeneralLinearModel.h"
#include "aVector.h"
#include "aMatrix.h"
#include "MatrixList.h"
#include "PartitionedMatrix.h"
#include <fstream.h>
#include <iostream.h>
#include <math.h>
#include "InlineTemplates.h"
#include "MemoryManager.h"



	class SexLinkedCodomMarkerData: public virtual SexLinkedData,
									public virtual EvolutionaryGeneticsData{
		public:
			long nlociSLCodom_;//omits skipped loci
			int ploidy_, alleleLen_;//ploidy_ holds homogametic if sexes differ
			aList<int> allSLCodomLoci_,locusNumSLCodom_, skippedLocusNumSLCodom_;
					//these hold places in (=indices of) EvolutionaryGeneticsData::traitName_[]
			aList<int> variableLocusNumSLCodom_;//holds places in (=indices of) locusNumSLCodom_[]
			ObjList<ObjList<ObjList<TextDataList> > > rawSLCodomGtypes_;//[pop][indiv][locus=trait][ploidy] - includes skipped loci
					//list of pops, each with list of individual genotype strings, sorted within individuals so that e.g., BA->AB
			ObjList<StringList> allelesPresentSLCodom_;//loci correspond to order of locusNumSLCodom_[]
					//list of loci, each with list of all alleles encountered; skipped loci omitted
			ObjList<ObjList<aList<long double> > > alleleFreqSLCodom_;//loci correspond to order of locusNumSLCodom_[]
			ObjList<aList<long double> > alleleFreqSLCodomGlobal_;//loci correspond to order of locusNumSLCodom_[]
					//UNWEIGHTED MEAN OVER POPS -- use function GetFreq() for weighted mean
			ObjList<ObjList<aList<long double> > > alleleFreqHeterogametic_,alleleFreqHomogametic_;//separately for each sex
					//list of pops, each with list of loci, each with list of allele freqs
			ObjList<aList<long double> > alleleFreqHeterogameticGlobal_,alleleFreqHomogameticGlobal_;//separately for each sex
					//list of loci, each with list of allele freqs -- UNWEIGHTED MEAN OVER POPS 
			ObjList<ObjList<ObjList<aList<int> > > > genotypeCountHomogametic_;//for heterogametic, use alleleFreqHeterogametic_
					//[pop][loc][allele1][allele2] list of pops, each with list of loci, each with list of allele freqs;RETURNS RAW COUNTS
			ObjList<ObjList<aList<int> > > genotypeCountHomogameticGlobal_;//for heterogametic, use alleleFreqHeterogameticGlobal_
					//[loc][allele1][allele2] -- UNWEIGHTED MEAN OVER POPS; RETURNS RAW COUNTS
			ObjList<ObjList<aList<int> > > genotypeCountHeterogametic_;//for heterogametic, use alleleFreqHeterogametic_
					//[pop][loc][allele] list of pops, each with list of loci, each with list of allele freqs;RETURNS RAW COUNTS
			ObjList<aList<int> > genotypeCountHeterogameticGlobal_;//for heterogametic, use alleleFreqHeterogameticGlobal_
					//[loc][allele] -- UNWEIGHTED MEAN OVER POPS; RETURNS RAW COUNTS
			ObjList<ObjList<ObjList<aList<long double> > > > genotypeFreqHomogametic_;//for heterogametic, use alleleFreqHeterogametic_
					//[pop][loc][allele1][allele2] list of pops, each with list of loci, each with list of allele freqs;RETURNS RELATIVE FREQUENCIES
			ObjList<ObjList<aList<long double> > > genotypeFreqHomogameticGlobal_;//for heterogametic, use alleleFreqHeterogameticGlobal_
					//[loc][allele1][allele2] -- UNWEIGHTED MEAN OVER POPS; RETURNS RELATIVE FREQUENCIES
			ObjList<aList<int> > twoNSLCodom_;	//locus order follows values in locusNumSLCodom_
					//actuallly holds # allkele copies sampled, 2N homogametic + 1N heterogametic
			ObjList<aList<int> > NheterogameticPopLoc_,twoNhomogameticPopLoc_;	//[pop][loc]for each sex
			ObjList<ObjList<aList<long double> > > HetObsByAlleleSL_;//loci correspond to order of locusNumSLCodom_[]	
					//list of pops, each with list of loci, each with list of observed heterozygosities
			ObjList<aList<long double> > HetObsByLocusSL_;//loci correspond to order of locusNumSLCodom_[]
					//list of pops, each with list of observed heterozygosities for each locus 
			aList<long double> diseqSLCodomD_;//for each pop;//loci correspond to order of locusNumSLCodom_[]
			ObjList<ObjList<aList<long double> > > locusSLCodomD_;	//pairwise Ds within each pop
			int varLociCurrentSLCodom_;//boolean
			int allelesCurrentSLCodom_;//boolean
			int alleleFreqCurrentSLCodom_;//boolean
			int genotypeFreqCurrentSLCodom_;//boolean
			int HetObsCurrentSL_;//boolean
			int DCurrentSLCodom_;//boolean
					//list of pops, each with list of gametic disequilibria for each locus pair
		public:
		SexLinkedCodomMarkerData(void):  EvolutionaryGeneticsData(), nlociSLCodom_(0), ploidy_(2), alleleLen_(1),
							varLociCurrentSLCodom_(0), allelesCurrentSLCodom_(0), 
							alleleFreqCurrentSLCodom_(0),genotypeFreqCurrentSLCodom_(0), HetObsCurrentSL_(0),DCurrentSLCodom_(0) {}
		SexLinkedCodomMarkerData(aString& s): EvolutionaryGeneticsData(s), nlociSLCodom_(0), ploidy_(2),alleleLen_(1), 
							varLociCurrentSLCodom_(0), allelesCurrentSLCodom_(0), 
							alleleFreqCurrentSLCodom_(0), genotypeFreqCurrentSLCodom_(0), HetObsCurrentSL_(0),DCurrentSLCodom_(0) {}
virtual	~SexLinkedCodomMarkerData(void){
			Clear();
			}

		SexLinkedCodomMarkerData(SexLinkedCodomMarkerData& parentData);//copy constructor

		SexLinkedCodomMarkerData(SexLinkedCodomMarkerData& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	virtual EvolutionaryGeneticsData* Duplicate(void){//use for virtual copying of derived classes
			return NEW SexLinkedCodomMarkerData(*this);}


	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(SexLinkedCodomMarkerData& pgd);
	void  		Initialize(SexLinkedCodomMarkerData& parentData, aList<int>& popSubsetNums);//makes a subset of pops

	virtual int alleleLen(void){return alleleLen_;}
	virtual int alleleID(int loc, TextDatum& alleleVal){//#of this allele in locus list
						if(!allelesCurrentSLCodom_) {CheckForNewAlleles();}
						return allelesPresentSLCodom_[loc].Loc(&alleleVal);}//location of the allele
	virtual int alleles(int loc){
						if(!allelesCurrentSLCodom_) {CheckForNewAlleles();}
						return allelesPresentSLCodom_[loc].len();}//number of alleles
//	void		Calculate2LocusDisequilibrium(void);//using unskipped loci
	void		CalculateAlleleFreqs (void);//using unskipped loci
	ObjList<aList<long double> >* 
				CalculateAlleleFreqsByPopSet(aList<int>& popSet, aList<long double>& twoNList);
	void		CalculateGenotypeFreqs(void);
	void		CalculateObservedHeterozygosity (void);//using unskipped loci; only calculated for homogametic sex
	void		CalculateTwoN(void);//using unskipped loci
	void 		CheckForNewAlleles (void);//using unskipped loci
	virtual void		Clear(void);//clears all information
	void		CreateBootstrapData(SexLinkedCodomMarkerData& origData, Ran3& ran);//individuals resampled
	void 		CreatePerfectData(const int numPopsSampled,//p=0.5, Hardy-Weinberg proportions, no differences among populations, sex ratio = 1:1
							const int numIndivsPerPop,//choose a multiple of 4
							const int numLociSampled,
							Ran3& ran);
	void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void 		CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran);
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void 		CreateRandomDataLocFirst(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void 		CullLoci (aList<int>& lociToCull);
	void 		CullLoci (StringList& lociToCull);
	void 		CullLocus (aString& locusToCull);
	void 		CullLocus (char* locusToCull);
	void 		CullLocus (int locusToCull);
	void 		CullMonomorphicLoci (void);
	virtual void 		
				CullToOneGroup (EvolutionaryGeneticsData& parentData, int groupNum);//makes a subset data structure from the specified group
	void 		CullToOneSex (SexLinkedCodomMarkerData& parentData, genderSet sexVal);//makes a subset data structure from the specified group
	void 		CullToTheseSexes (SexLinkedCodomMarkerData& parentData, aList<int>& sexValList);//makes a subset data structure from the specified group
	void 		ExpungeLoci (StringList& lociToCull);//transfer to skipped
	void 		ExpungeLoci (aList<int>& lociToCull);
	void 		ExpungeLocus (aString& locusToCull);
	void 		ExpungeLocus (char* locusToCull);
	void	FitAllozymeCline(aString& dataFileName);
	TextDataList& 	GenotypicVal(int pop, int indiv, int trait);//using unskipped loci
									//use locusNumSLCodom_[loc] for 'trait', to get this for the right sex-linked locus
	TextDatum& 	GenotypicVal(int pop, int indiv, int trait, int allele);//using unskipped loci
	int 		GenotypeSize(void){return alleleLen_ * ploidy_;}
	void 	GenotypeTable(ObjList<MatrixList>& gtypeTableList);//stored pops within loci; square matrices of genotypes
	long double 	GetExpectedHeterozygosity (int pop, int loc, int allele);//using unskipped loci; calculated from allele freqs of both sexes
	long double 	GetExpectedHeterozygosity (int pop, int loc);//using unskipped loci; calculated from allele freqs of both sexes
	long double 	GetExpectedHeterozygosity (int pop);//returns mean over unskipped loci; calculated from allele freqs of both sexes
	long double 	GetFreq (int loc, int allele);//pooling populations, using unskipped loci -- USES WEIGHTED AVERAGE -- use alleleFreqGlobal_ for unewighted average
	long double 	GetUnweightedFreq (int loc, int allele){
							if(!allelesCurrentSLCodom_){ CalculateAlleleFreqs();}
							return alleleFreqSLCodomGlobal_[loc][allele];}
	long double 	GetFreq (int pop, int loc, int allele);//using unskipped loci
	long double 	GetFreqHeterogametic (int loc, int allele);//pooling populations, using unskipped loci -- USES WEIGHTED AVERAGE -- use alleleFreqGlobal_ for unewighted average
	long double 	GetUnweightedFreqHeterogametic (int loc, int allele){
							if(!allelesCurrentSLCodom_){ CalculateAlleleFreqs();}
							return alleleFreqHeterogameticGlobal_[loc][allele];}
	long double 	GetFreqHeterogametic (int pop, int loc, int allele);//using unskipped loci
	long double 	GetFreqHomogametic (int loc, int allele);//pooling populations, using unskipped loci -- USES WEIGHTED AVERAGE -- use alleleFreqGlobal_ for unewighted average
	long double 	GetUnweightedFreqHomogametic (int loc, int allele){
							if(!allelesCurrentSLCodom_){ CalculateAlleleFreqs();}
							return alleleFreqHomogameticGlobal_[loc][allele];}
	long double 	GetFreqHomogametic (int pop, int loc, int allele);//using unskipped loci
	long double 	GetFreqMostCommonAllele (int pop, int loc);//using unskipped loci
	long double 	GetFreqMostCommonAllele (int loc);//using unskipped loci
	virtual void 	GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList);
							//uploads data to pass to genetic analysis classes
	virtual void 	GetGenome(GeneContainerCore& gc, int pop, int indiv, bool setID=true);
						//uploads this indiv's data to MultiHaplotypeContainer<int>
	int				GetGenotypeCountHomogametic(int pop, int loc, int allele1, int allele2){
						if(!genotypeFreqCurrentSLCodom_){CalculateGenotypeFreqs();}
						if(allele1>allele2){SWAP(allele1,allele2);}
						return genotypeCountHomogametic_[pop][loc][allele1][allele2];}
	int				GetGenotypeCountHomogametic(int loc, int allele1, int allele2){
						if(!genotypeFreqCurrentSLCodom_){CalculateGenotypeFreqs();}
						if(allele1>allele2){SWAP(allele1,allele2);}
						return genotypeCountHomogameticGlobal_[loc][allele1][allele2];}
	int				GetGenotypeCountHeterogametic(int pop, int loc, int allele){
						if(!genotypeFreqCurrentSLCodom_){CalculateGenotypeFreqs();}
						return genotypeCountHeterogametic_[pop][loc][allele];}
	int				GetGenotypeCountHeterogametic(int loc, int allele){
						if(!genotypeFreqCurrentSLCodom_){CalculateGenotypeFreqs();}
						return genotypeCountHeterogameticGlobal_[loc][allele];}
	long double		GetGenotypeFreqHomogametic(int pop, int loc, int allele1, int allele2){
						if(!genotypeFreqCurrentSLCodom_){CalculateGenotypeFreqs();}
						if(allele1>allele2){SWAP(allele1,allele2);}
						return genotypeFreqHomogametic_[pop][loc][allele1][allele2];}
	long double		GetGenotypeFreqHomogametic(int loc, int allele1, int allele2){
						if(!genotypeFreqCurrentSLCodom_){CalculateGenotypeFreqs();}
						if(allele1>allele2){SWAP(allele1,allele2);}
						return genotypeFreqHomogameticGlobal_[loc][allele1][allele2];}
	long double		GetGenotypeFreqHeterogametic(int pop, int loc, int allele1){
						if(!alleleFreqCurrentSLCodom_){CalculateAlleleFreqs();}//same as allele freqs
						return alleleFreqHeterogametic_[pop][loc][allele1];}
	long double		GetGenotypeFreqHeterogametic(int loc, int allele1){
						if(!alleleFreqCurrentSLCodom_){CalculateAlleleFreqs();}//same as allele freqs
						return alleleFreqHeterogameticGlobal_[loc][allele1];}
//	aString* 	GetGenotypeString(int pop, int indiv);//drops code & returns new string; must be deleted elsewhere
//	aString* 	GetGenotypeString(int pop, int indiv, int loc);//returns new string; must be deleted elsewhere
//	aString* 	GetGenotypeString(int pop, int indiv, int loc, int chrom);//returns new string; must be deleted elsewhere
	aString* 	GetTraitString(int pop, int indiv, int loc, int allele);//loc is item in locusNumSLcodom_
	long 		GetIndividualsSampled (int pop, int loc);
	void		GetLocusNames(StringList& names){
					TraitNamesFromSublist(names,locusNumSLCodom_);}
	long 		GetLocusSampleSize (int loc);//pooling populations
	long 		GetLocusSampleSize (int pop, int loc);//using unskipped loci
	int	 	GetLocusSampleSize (int pop, int indiv, int loc);//returns ploidy//using unskipped loci
	long 		GetLocusSampleSizeHeterogametic (int loc);//pooling populations
	long 		GetLocusSampleSizeHeterogametic (int pop, int loc);//using unskipped loci
	int	 	GetLocusSampleSizeHeterogametic (int pop, int indiv, int loc);//returns ploidy//using unskipped loci
	long 		GetLocusSampleSizeHomogametic (int loc);//pooling populations
	long 		GetLocusSampleSizeHomogametic (int pop, int loc);//using unskipped loci
	int	 	GetLocusSampleSizeHomogametic (int pop, int indiv, int loc);//returns ploidy//using unskipped loci
	long double	GetMeanAllelesPerLocus (int pop);//using unskipped loci
	int	 	GetNumAlleles (int loc);//gives total in data set//using unskipped loci
	int	 	GetNumAlleles (int pop, int loc);//gives extant alleles in pop//using unskipped loci
	long double 	GetObservedHeterozygosity (int pop, int loc, int allele);//using unskipped loci; only calculated for homogametic sex
	long double 	GetObservedHeterozygosity (int pop, int loc);//using unskipped loci; only calculated for homogametic sex
	long double 	GetObservedHeterozygosity (int pop);//returns mean over loci; using unskipped loci; only calculated for homogametic sex
	long double 	GetPercentPolymorphicLoci (int pop, long double criterion = (long double)100.0);//using unskipped loci
	long double	GetVarJackAllelesPerLocus (int pop);//returns jackknife variance//using unskipped loci
	long double 	GetVarJackExpectedHeterozygosity (int pop);//returns jackknife variance//using unskipped loci
	long double 	GetVarJackObservedHeterozygosity (int pop);//returns jackknife variance//using unskipped loci
	long double 	GetVarJackPercentPolyLoci (int pop, long double criterion = (long double)100.0);//using unskipped loci
	virtual int 	individuals(int pop){return EvolutionaryGeneticsData::individuals(pop);}
	virtual int 	individuals(int pop, int loc);//overrides base class
	int		IsPolymorphic(int pop, int loc);//boolean//using unskipped loci
	int		IsPolymorphic(int pop, int loc, long double criterion);//boolean//using unskipped loci
	int		IsPolymorphic(int loc);//boolean//using unskipped loci
	int		IsPolymorphic(int loc, long double criterion);//boolean//using unskipped loci
	virtual int 	loci(void){return nlociSLCodom_;}// unskipped loci
	virtual int 	lociSkipped(void){return skippedLocusNumSLCodom_.len();}
	virtual aString& locusName(int loc){ return traitName_[locusNumSLCodom_[loc]];}
	virtual long 	maxall(void);//maximum alleles per locus in data set
//	virtual void 	ParseGenotypicData(aList<int>& codominantSLLoci, int startingLocus=1);
	virtual void 	ParseGenotypicData(int startingLocus=1);
							//convert from raw trait data
							//startingLocus = the first trait that contains genotypes
	virtual int 	Ploidy(int pop, int indiv, int /*loc*/);
	virtual int 	ploidy(void){return ploidy_;}
	virtual int 	ploidy(int pop, int indiv){return Ploidy(pop,indiv,0);}
	virtual void 	PrintAlleleFreqs (ostream& outFile, int ncols, char colSpacer = ' ');//outfile assumed open
	virtual void 	PrintAlleleFreqsBySex (ostream& outFile, int ncols, char colSpacer = ' ');//outfile assumed open
	void  		PrintAlleleFreqsByPopList (ostream& outFile, 
										ObjList<ObjList<aList<double> > >& popFreqList, 
										ObjList<aList<double> >& popTwoNList, 
										int ncols);
	virtual void	 	PrintArlequinFile(aString& outFileName);
	virtual void 		PrintArlequinFile (ostream& outFile);
	virtual void	 	PrintGelJockFile(aString& outFileName);
	virtual void 		PrintGelJockFile (ostream& outFile);
	virtual void 	PrintGenotypeCounts (ostream& outFile, int ncols, char colSpacer = ' ');//outfile assumed open
	void 		PrintHeterozygosity (ostream& outFile, const int hetType, int ncols);//outfile assumed open
	virtual void 		PrintLoci (ostream& outFile, int lociPerLine);//outfile assumed open//using unskipped loci
	virtual void		PrintSASFormatData(aString& outFileName, char colSpacer = '\t');//output file assumed closed
	virtual void		PrintSASFormatData(ostream& outFile, char colSpacer = '\t');//outfile assumed open
				//missing data represented by '.', columns separated by spaces
	virtual void 		PrintVariabilityScores(aString& outFileName, char sepChar='\0');
	virtual void 		PrintVariabilityScores(ostream& outFile, char sepChar='\0');
	virtual int 		ReadArlequinDataFile (aString* dataFileName);//input file assumed closed
	virtual int 		ReadGelJockDataFile (aString* dataFileName);//input file assumed closed
	virtual void		ResetFlags(void){
								SexLinkedData::ResetFlags();//just in case the base class adds any
								SexLinkedCodomMarkerData::ResetGeneticFlags();}
	virtual void		ResetGeneticFlags(void){//skips sex-linkage flags
								varLociCurrentSLCodom_=allelesCurrentSLCodom_=alleleFreqCurrentSLCodom_=HetObsCurrentSL_=DCurrentSLCodom_=0;}
	virtual void 		RestoreLoci (aList<int>& lociToRestore);
	virtual void 		RestoreLoci (StringList& lociToRestore);
	virtual void 		RestoreLocus (aString& locusToRestore);
	virtual void 		RestoreLocus (char* locusToRestore);
	virtual void 		RestoreLocus (int locusToRestore);
	virtual void		SetAlleleLength(int allelelength){
							alleleLen_=allelelength;}
	virtual void 		SetGenome(ObjList<PopulationGeneContainer<int> >& pgcList);
							//records all data, passed along from genetic analysis classes
							//empties old data and creates new data structures
	virtual void 		SetGenome(GeneContainerCore& gc, int pop, int indiv);
							//records data from MultiHaplotypeContainer<int>
							//assumes blank data records already created in EvolutionaryGeneticsData
	void				SetLoci(aList<int>& lociToInclude);//constrains the sex-linked codominant data set to these loci only
							//skipped loci status unchanged (use CullLoci to skip more; RestoreLoci to skip fewer)
	virtual void 		SnipArlequinComment(aString& arqStr);
	virtual int 		traits(void){return locusNumSLCodom_.len();}	
	virtual int 		traits(int /*traitType*/){return SexLinkedCodomMarkerData::traits();}	
	virtual void		UpdateVariableLoci(void);//using unskipped loci
	virtual aString&	VariableLocusName(int loc){
							return traitName_[locusNumSLCodom_[variableLocusNumSLCodom_[loc]]];}
	SexLinkedCodomMarkerData& operator=(SexLinkedCodomMarkerData& pgd);
	};//end class SexLinkedCodomMarkerData
		
		
#endif

