/***************************************************

	SexLinkedData.h
	
	Base class for handling sex linkage or haplodiploidy
	Assumes that one of the traits is 'sex', labeled M or F for each individual
	
	Use derived classes for dominant or codominant marker types
	
****************************************************/

#ifndef SexLinkedDatah
#define SexLinkedDatah = 1


#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "PopulationIntf.h"
#include "EvolutionaryGeneticsData.h"
#include "TextDatum.h"
#include "TextDataList.h"
#include "StatList.h"
#include "GenotypeFreqDistribution.h"
#include "GenotypeFreqMultiAlleleDist.h"
#include "GeneFrequencyDistribution.h"
#include "GeneFrequencyDistMultiAllele.h"
//#include "ANOVAGenetics.h"
#include "GeneralLinearModel.h"
#include "aVector.h"
#include "aMatrix.h"
#include "MatrixList.h"
#include "PartitionedMatrix.h"
#include <fstream.h>
#include <iostream.h>
#include <math.h>
#include "InlineTemplates.h"
#include "MemoryManager.h"



	class SexLinkedData: public virtual EvolutionaryGeneticsData{
		public://hetTypes
			const static char M='M';
			const static char F='F';
		public:
			char heterogameticSex_;//M or F
			int sexTrait_;//the trait number in rawTraits_ that specifies sex of each individual; usually trait[0]=ID && trait[1]=sex
			long double primarySexRatio_;// =F/(F+M) females first
			ObjList<aList<int> > heterogameticIndivs_, homogameticIndivs_;//[pop][indiv]; holds places in rawTraits_
			int Nheterogametic_, Nhomogametic_;
			bool sortedBySex_;
		public:
		SexLinkedData(void):  EvolutionaryGeneticsData(), sexTrait_(1),
							primarySexRatio_((long double)0.5), heterogameticSex_(M),Nheterogametic_(0),Nhomogametic_(0),sortedBySex_(0){}
		SexLinkedData(aString& s): EvolutionaryGeneticsData(s), sexTrait_(1),
							primarySexRatio_((long double)0.5), heterogameticSex_(M),Nheterogametic_(0),Nhomogametic_(0),sortedBySex_(0){}
virtual	~SexLinkedData(void){
			Clear();
			}

		SexLinkedData(SexLinkedData& parentData){//copy constructor
				Initialize(parentData);}

		SexLinkedData(SexLinkedData& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	virtual EvolutionaryGeneticsData* Duplicate(void){//use for virtual copying of derived classes
			return NEW SexLinkedData(*this);}


	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(SexLinkedData& pgd);
	void  		Initialize(SexLinkedData& parentData, aList<int>& popSubsetNums);//makes a subset of pops

	virtual void		Clear(void){
							//clears all information
							heterogameticIndivs_.Clear();
							homogameticIndivs_.Clear();
							EvolutionaryGeneticsData::Clear();
							}
	void		CreateBootstrapData(SexLinkedData& origData, Ran3& ran);//individuals resampled
	virtual void 		
				CullToOneGroup (EvolutionaryGeneticsData& parentData, int groupNum);//makes a subset data structure from the specified group
	void 		CullToOneSex (SexLinkedData& parentData, genderSet sexVal);//makes a subset data structure from the specified group
	void 		CullToTheseSexes (SexLinkedData& parentData, aList<int>& sexValList);//makes a subset data structure from the specified group
	virtual int		females(void){
						int f;
						if(heterogameticSex_=='F') {f=Nheterogametic_;} else {f=Nhomogametic_;}
						return f;}
	virtual int		females(int pop){
						int f;
						if(heterogameticSex_=='F') {f=heterogameticIndivs_[pop].len();} else {f=homogameticIndivs_[pop].len();}
						return f;}
	virtual void 	GetGenome(GeneContainerCore& gc, int pop, int indiv, bool setID=true);
						//uploads data to MultiInheritanceModeContainer<int>[locus]
	bool 			isMale(int pop, int indiv){//default sex is female by convention
						TextDatum& sex = rawTraits_[pop][indiv][sexTrait_];
						if(sex == M || sex == 'm'){ return true;}
						return false;}
	bool 			isFemale(int pop, int indiv){
						return ! isMale(pop,indiv);}
	char 			heterogameticSex(void){return heterogameticSex_;}
	virtual bool 	heterogametic(int pop, int indiv);
	virtual bool 	homogametic(int pop, int indiv){return (! heterogametic(pop,indiv));}
	virtual int		males(void){
						int m;
						if(heterogameticSex_=='M') {m=Nheterogametic_;} else {m=Nhomogametic_;}
						return m;}
	virtual int		males(int pop){
						int m;
						if(heterogameticSex_=='M') {m=heterogameticIndivs_[pop].len();} else {m=homogameticIndivs_[pop].len();}
						return m;}
	virtual int 	ploidy(void){return 2;}//default for autosomal ploidy
	virtual long double PloidyEffective(void);
	virtual double	primarySexRatio(void){// = F/(F+M) -- females first
							return primarySexRatio_;}
	virtual int		SampleSizeHeterogametic(void){if(!sortedBySex_)SortBySex();return Nheterogametic_;}
	virtual int		SampleSizeHomogametic(void){if(!sortedBySex_)SortBySex();return Nhomogametic_;}
	virtual int		SampleSizeHeterogametic(int pop){if(!sortedBySex_)SortBySex();return heterogameticIndivs_[pop].len();}
	virtual int		SampleSizeHomogametic(int pop){if(!sortedBySex_)SortBySex();return homogameticIndivs_[pop].len();}
	virtual void	SetHeterogametic(char hetChar);
	virtual void 	SetGenome(GeneContainerCore& gc, int pop, int indiv);
						//records sex data from MultiInheritanceModeContainer<int>
						//assumes blank data records already created in EvolutionaryGeneticsData
	virtual void	SetPrimarySexRatio(long double sr){// = F/(F+M) -- females first
							primarySexRatio_=sr;}
	virtual int		SetSexTrait(void);//the trait that contains sex information
							//returns 0 if successful, error code 1 otherwise
	int				sexTrait(void){return sexTrait_;}
	virtual void	SortBySex(void);//classifies into homogametic and heterogametic
	SexLinkedData& operator=(SexLinkedData& pgd);
	};//end class SexLinkedData
		
		
#endif

