/*******************************************************
*
*	GridCell
*
*
*	This class defines habitat types and keeps a list of populations
*	of different species.  It keeps a list of neighboring GridCells, which may
*	vary depending on the LandScape structure.  It's remaining functions simply 
*	provide communication between the LandScape and its Populations.

*******************************************************/
#ifndef GridCellh
#define GridCellh = 1



#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "PopulationIntf.h"
#include "SelectionIntf.h"
#include "RandomList.h"
#include "MemoryManager.h"

class Individual;
class GridCell;
class LandScape;
class Population;


class GridCell{

  protected:
	ObjList<Population> pops_;
	CoordinateType coords_;
	int habitat_;
		/*a list of adjacent cells, coded for use in different grid arrangements*/
	ObjList<GridCell> neighbors_;
	LandScape* region_;
	
  public:
  	GridCell(LandScape* r, CoordinateType coords, int habitat):
  		region_(r), 
  		coords_(coords), habitat_(habitat){}
  	GridCell(LandScape* r, CoordinateType coords):
  		region_(r), 
  		coords_(coords),habitat_(0){}
  	
  	GridCell(void):region_(0){}
  		
	GridCell(GridCell& g):/*neighbors & pops not copied*/
  		region_(g.region_), 
  		coords_(g.coords_), habitat_(g.habitat_){}
  	
	GridCell(GridCell* g):/*neighbors & pops not copied*/
  		region_(g->region_), 
  		coords_(g->coords_), habitat_(g->habitat_){}
  	
virtual ~GridCell(void);

virtual GridCell* 		Duplicate(LandScape* r, CoordinateType coords, int habitat);
virtual GridCell* 		Duplicate(LandScape* r, CoordinateType coords);
virtual GridCell* 		Duplicate(void);//virtual copy constructor
  	
  	void				Append(Population* p);
  	CoordinateType&		coords(void) {return coords_;}
virtual long double 	gridScale(void);
virtual int 			habitat(void) {return habitat_;}
virtual void			habitat(int h) {habitat_=h;}/*habitats can change*/
virtual long			immigrants(int species);//number of immigrants
virtual long double		intrinsicSexRatio(int species);
virtual long double		moveRate(int species, genderSet sex, int age);
virtual migrationModelSet migrationModel(void);
virtual long double 	MigrationProb(int species, int ageClass, int currentHabitat, int potentialHabitat);
//virtual void 			NaturalSelection(int species, Ran3List& ran, selectionSourceSet sSource);
virtual long			neighbors(void){return neighbors_.len();}
virtual GridCell*		neighbor(int direction);//can return null pointer
virtual Population*		neighbor(int species, int direction);//can return null pointer
virtual aList<int>*		NeighboringHabitats(void);/*delete this list elsewhere*/
  	      Population& 	pop(int species);
  		int 			species(void) {return pops_.len();}
  	
virtual void			SetNeighbor(GridCell* newNeighbor);
virtual void			SetNeighbor(GridCell* newNeighbor, long direction);
		long 			time(void);//returns time from Landscape
virtual void			DropNeighbor(GridCell* oldNeighbor);
  		LandScape*	region(void);
  	
		Population&	operator[] (int p);
friend  int			operator== (GridCell& a, GridCell& b){
  							return (a.coords( )==b.coords( ));}
friend  int			operator != (GridCell& a, GridCell& b){
  							return !(a==b);}
  					

virtual void 			AdultDispersal(int species, Ran3List& ran, aList<long double>& dispersalInfo);
virtual long			adults(int species);
virtual void			AgeOneStep(int species, int maxage);
virtual void			AssignMates (int species, Ran3List& ran, aList<long double>& reproductionInfo);
virtual long			babies(int species);
virtual void			BuryTheDead(int species);
virtual long			carryingCapacity(int species);
virtual void			ClearOffspringLists(int species);
virtual long			dead(int species);
virtual void 			Dispersal(int species, Ran3List& ran, aList<long double>& dispersalInfo);
virtual int				extinct(int species);
virtual void 			Extract (int species, Individual* indiv);
virtual long double		Fecundity(int species, int age);
virtual void 			GameteDispersal (int species, Ran3List& ran, aList<long double>& dispersalInfo);
virtual void			GetAgeDistribution(int species, aList<int>& ageDist);
virtual void			ImmatureDispersal(int species, Ran3List& ran,aList<long double>& dispersalInfo);
virtual void			InitializeANewGeneration(int species);
virtual void 			Install (int species, Individual* indiv);
virtual void			IPopulation (int species, CoordinateType loc, long k, Ran3List& ran);
virtual int				IsAllOneSex(int species);
virtual int				NoInconsistentSettings(int species);
virtual void			PrintAgeDistribution(ostream& outFile, int species, int maxage);
virtual void			PrintPopHeading(ostream& outFile, int species);
virtual void			Recruit(int species, Ran3List& ran);
virtual void			Recruit(int species, long nthBaby);
virtual void			ReplaceTheDead (int species, Ran3List& ran);
virtual void			Reproduce (int species, long& idCount, Ran3List& ran, aList<long double>& specialInfo);
virtual void			ReproduceBySyngamy (int species, long& idCount, Ran3List& ran, aList<long double>& specialInfo);
virtual void			SetID (int species, int IDnum);
virtual void			SetReproductiveStatus(int species);
virtual long			size(int species);
virtual void 			SomeAdultsDie (int species, aList<long double>& fSurvivalSchedule,
										aList<long double>& mSurvivalSchedule, Ran3List& ran);
virtual void 			SomeImmaturesDie (int species, aList<long double>& fSurvivalSchedule,
										aList<long double>& mSurvivalSchedule, Ran3List& ran);
virtual void 			SomeGametesDie (int species, aList<long double>& fSurvivalSchedule,
										aList<long double>& mSurvivalSchedule, Ran3List& ran);
							  /*returns an error code if settings are inconsistent*/

							
							
/* THESE ARE FUNCTIONS TO BE CALLED ONLY BY DERIVED GENETIC CLASSES	*/
/* They return nonsense, false, or zero as appropriate			*/			
/*virtual double	AlleleFrequency(int, int, int){
							return (-2.0);}
virtual int		AllelePresent (int, int, int){return 0;}							
virtual long		Alleles (int, int){return 0;}
virtual int		AlleleValue (int, int, int){return 0;}
virtual void		AssignMutations(int){}
virtual void		CalcAlleleFrequencies(int){}
virtual void		CalcExpHeterozygosity(int){}
virtual void		CalcObsHeterozygosity(int){}
virtual void 		CoordsOfNthLocus(int, int, int&, int&){}
virtual long		CountAlleles (int, int, int){return 0;}							
virtual long		CountHets (int, int, int){return 0;}							
virtual long		CountHets (int, int){return 0;}							
virtual double	Disequilibrium(int){return (-2);}
virtual double	Disequilibrium(int, int, int){return (-2);}							
virtual double	Fis(int){return (-2);}							
virtual double	Fis(int, int){return (-2);}							
virtual double	Fis(int, int, int){return (-2);}							
virtual int		loci(int){return 0;}
virtual int		NthLocus(int, int, int){return (-1);}
virtual int		ploidy(int, int){return 0;}
virtual void		PrintAlleleFreqs(ostream&, int){}					
virtual void		PrintAlleleFreqs(ostream&, int, int){}							
virtual void		PrintAlleleFreqs(ostream&, int, int, int){}							
virtual void		PrintAlleleList(ostream&, int){}					
virtual void		PrintAlleleList(ostream&, int, int){}					
virtual void		PrintGeneStrings(ostream&, int, int, int){}						
virtual void		PrintHetExp(ostream&, int){}	
virtual void		PrintHetExp(ostream&, int, int){}				
virtual void		PrintHetObs(ostream&, int){}		
virtual void		PrintHetObs(ostream&, int, int){}							
virtual long		SampleSize (int, int){return 0;}
virtual void		UpdateAlleleList(int){}
*/
virtual void		ViabilitySelection(int species, Ran3& ran);
							
							
							
  	};/*end GridCell*/
	
#endif
