/******************************************************
*
*	BezierNode.h
*
*	Parameters for the node (control point on the curve) of a Bezier curve
*
*	The node holds it's Cartesian coordinates, and polar coordinates describing 1-2 handles that define the curve shape.
*	It does not (yet) support cusps (handles with separate angles, causing non-smooth transitions across the node)
*
******************************************************/

#ifndef BezierNodeh
#define BezierNodeh = 1


#include "Element.h"
#include "aList.h"
#include "MemoryManager.h"
#include "InlineTemplates.h"
#include "Randomness.h"
#include "CoordinateClass.h"

class CubicBezierSplineParamRec;
class CubicBezierSplineSection;
class BezierNode;


class BezierNode{
  public:
	CoordinateClass<long double> coords_;//position on the x-y plane
	long double angle_; //of the tangent of the curve through the node, in radians: -PI()/2 < angle < Pi()/2
	long double distL_,distR_;//distances along the tangent to the handles, in the directions R & L.  If dist<0, no handle
	bool terminalL_;  // spline terminus present on left end of curve
	bool terminalR_;  // spline terminus present on right end of curve
	CubicBezierSplineSection *section0_, *section1_;
	
	
  public:
  	BezierNode(void);
  	BezierNode(CoordinateClass<long double>& coords);

  	BezierNode(CoordinateClass<long double>& coords, long double angle, long double distL, long double distR,
  								bool terminalL, bool terminalR);
  								
  	BezierNode(BezierNode& bn);
  	virtual ~BezierNode(void);
  	virtual void			DetachPointers(void);//detaches but doesn't delete section pointers
  			void			DetachSections(void);//detaches but doesn't delete section pointers
  				
	virtual BezierNode* 	Duplicate(void);
	void 					operator=(BezierNode& bn);
	void					CopyDataFieldsOnly(BezierNode& bn);//omits sections 	

	virtual long double		Angle(void);//in radians
	virtual bool			ChangeNodeParam(int paramAtNode, Ran3& ran, BezierNode& devNode, BezierNode& lowNode, BezierNode& highNode);
	virtual long double		HandleDistanceLeft(void);
	virtual long double		HandleDistanceRight(void);
	virtual void			HandleCartesianLeft(CoordinateClass<long double>& newCoords);
	virtual void			HandleCartesianRight(CoordinateClass<long double>& newCoords);
	
	virtual void			MultiplyParam(int paramToChange, long double factor);
									//x=0,y=1,angle=2,distL=3,distR=4
									//includes error-checking
	virtual void			NodeCartesian(CoordinateClass<long double>& newCoords);//changes value of newCoords 

	virtual void 			NotifySplineOfChange(int paramChanged);//x=0,y=1,angle=2,distL=3,distR=4
	virtual inline int		NumParams(void){return 5;}
	virtual void			ParamName(int paramAtNode, aString& appendToThis);
	virtual aString*		ParamName(int paramAtNode);
	virtual long double		ParamValue(int paramAtNode);//x=0,y=1,angle=2,distL=3,distR=4
	virtual long double		Position_x(void);
	virtual long double		Position_y(void);

	virtual bool			SetAngle(long double newAngle);//in radians -- returns 1=OK, 0=fail
	virtual bool			SetCoordinates(CoordinateClass<long double>& newCoords);//changes value of coords_ -- returns 1=OK, 0=fail
	virtual bool			SetHandleDistanceLeft(long double newDist);//-- returns 1=OK, 0=fail
	virtual bool			SetHandleDistanceRight(long double newDist);//-- returns 1=OK, 0=fail
	virtual void			SetMathematicaFormString(aString& stringToFill);//used in a Mathematica notebook to print a spline graphic
	virtual bool			SetParamValue(int paramNum, long double newVal);//-- returns 1=OK, 0=fail
	virtual void			SetSectionLeft(CubicBezierSplineSection *leftSection);//-- returns 1=OK, 0=fail
	virtual void			SetSectionRight(CubicBezierSplineSection *rightSection);//-- returns 1=OK, 0=fail
	virtual void			SetTerminalLeft(bool tf);
	virtual void			SetTerminalRight(bool tf);
	virtual bool			Set_x(long double new_x,int recursion=0);//-- returns 1=OK, 0=fail
	virtual bool			Set_y(long double new_y);//-- returns 1=OK, 0=fail
	virtual bool			TerminalNode(void);
	virtual bool			TerminalNodeLeft(void);
	virtual bool			TerminalNodeRight(void);

	virtual CoordinateClass<long double>&		ValueAt_t(void);

	
	friend int	operator==(BezierNode& p, BezierNode& q);
	friend int	operator!=(BezierNode& p, BezierNode& q);

	friend class CubicBezierSplineParamRec;
	friend class CubicBezierSplineSection;
//	friend class BezierVariable;

	};//end class BezierNode



#endif