/******************************************************
*
*	CubicBezierSplineSection.h
*
*	A class holding the parameters and functions required to describe a section between 2 nodes of cubic Bezier curve,
*
*	Use use with a container class that holds a separate list of nodes, such as CubicBezierSplineParamRec
*
******************************************************/

#ifndef CubicBezierSplineSectionh
#define CubicBezierSplineSectionh = 1


#include "MemoryManager.h"
#include "InlineTemplates.h"
#include "aError.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "CoordinateClass.h"
#include "BezierNode.h"
#include "Bisection_GoldenRatioFns.h"
#include "CommonMathFunctions.h"

class CubicBezierSplineParamRec;
class CubicBezierSplineSection;
class CubicBezierSpline;
class BezierNode;

inline long double	bezierValueFn(long double t, aList<long double>& abcd){
	return abcd[0]*t*t*t + abcd[1]*t*t + abcd[2]*t + abcd[3];
	}//use for maximization functions outside of this class
	

inline long double	bezierValue_TfromX_Fn(long double t, aList<long double>& abcd){
	
	return abcd[0]*t*t*t + abcd[1]*t*t + abcd[2]*t + abcd[3];
	}//use for maximization functions outside of this class
	

class CubicBezierSplineSection{
  public:
	int ID_;//section in spline list; -1 if not listed
	BezierNode *node0_, *node1_;//also held separately in a node list in CubicBezierSpline
	CubicBezierSplineSection *sectionPrev_, *sectionNext_;
	CubicBezierSpline *spline_;//
	CoordinateClass<long double> xL_,xR_;//range of the section on the x-axis --> update if node position varies!
//	aList<long double> bezierParams_;
	ObjList<CoordinateClass<long double> > cp_;//control points
	bool bezierParamsCurrent_;
	StatList<long double> splineXvals_;//holds spline coordinates for data points X, for calculating residuals
	aList<long double> splineYforXinData_;//holds spline coordinates for data points X, for calculating residuals
	bool splineYforXinDataCurrent_;
	long double toleranceFactor_, yMin_,yMax_;
	aList<long double> realizedRange_;//y range of spline given control points
	bool realizedRangeCurrent_;
//	long double lnL_section_, lnLmax_section_;
//	bool lnL_current_;
	bool deleteContentsOnDisposal_;//if true, then delete contents in ~CubicBezierSplineSection; otherwise delete elsewhere
	aList<long double> quadraticSolutions_;//solutions to find t values where dy/dt
	
	
  public:
  	CubicBezierSplineSection(int sectionID=-1, bool deleteContentsOnDisposal=false);
  	CubicBezierSplineSection(BezierNode *node0, BezierNode *node1, int sectionID, bool deleteContentsOnDisposal=false);
  	CubicBezierSplineSection(BezierNode *node0, BezierNode *node1,
  								CubicBezierSplineSection *section0, CubicBezierSplineSection *section1,
  								int sectionID, bool deleteContentsOnDisposal=false);
  	CubicBezierSplineSection(CubicBezierSplineSection& cbss, bool deleteContentsOnDisposal=true);
  	virtual ~CubicBezierSplineSection(void);
  				
  	CubicBezierSplineSection* 		Duplicate(void);
	virtual void 					operator=(CubicBezierSplineSection& cbss);
	virtual void 					Copy(CubicBezierSplineSection& cbss, bool deleteContentsOnDisposal=true);
	virtual void 					CopyDataFieldsOnly(CubicBezierSplineSection& cbss);//omits spline, nodes & adjacent sections
	virtual void					DetachPointers(void);//detaches but does not destroy nodes, spline, & adjacent sections
	virtual void					DetachSpline(void);//detaches but does not destroy spline only
	virtual void					DetachNodes(void);//detaches but does not destroy nodes only
	virtual void					DetachSections(void);//detaches but does not destroy adjacent sections only
  														

	virtual long double		bezierValue(long double x, CoordinateClass<long double>& coordsOnCurve, 
									long double toleranceFactor=(long double)0.001);
									//determines {x,y} at position t on the curve; returns vals in coordsOnCurve

	virtual long double		bezierValue(long double t, long double a, long double b, long double c, long double d);
									//determines x or y at position t on the curve, depending on values in a, b, c, d

	virtual long double		bezierValue(long double t, aList<long double>& abcd);
									//determines x or y at position t on the curve, depending on values in abcd

	virtual bool			bezierValuesInRange(void);

	virtual long double 	CalculateBezierValue(long double x, long double toleranceFactor=(long double)1.0e-8);
	virtual long double 	CalculateBezierValue(long double x, long double toleranceFactor,
										aList<long double>& abcd_x, aList<long double>& abcd_y);
	virtual void			CalculateBezierValues(long double toleranceFactor=(long double)1.0e-8);
											//fill in y values of splineYforXinData_; do when nodes change values
	virtual void			CheckToleranceFactor(long double toleranceFactorToCheck);
	virtual long double		GetTabledYfromX(long double x);
	virtual void			handleCoordsL(CoordinateClass<long double>& coordsToGet);
	virtual void			handleCoordsR(CoordinateClass<long double>& coordsToGet);
	
//	virtual void			lnLNowInvalid(void);
	
	virtual void			nodeCoordsL(CoordinateClass<long double>& coordsToGet);
	virtual void			nodeCoordsR(CoordinateClass<long double>& coordsToGet);

	virtual long double		range_x(void);//along x-axis

	virtual long double		ResidualAt_x(CoordinateClass<long double>& datum, long double toleranceFactor);
										//on x-axis scale 
		
	virtual void			SetBezierParams(void);
	virtual void			SetBezierParamsCurrent(bool tf);
	virtual void			SetControlPoints(void);	
	virtual void			SetEndpoints(void);//sets xL_ && xR_				
//	virtual void			SetLnL(long double lnLval);
			void			SetSectionID(int id);
	virtual void			SetNodeLeft(BezierNode *node);//can set to NULL
	virtual void			SetNodeRight(BezierNode *node);
	virtual void 			SetSectionLeft(CubicBezierSplineSection *section);
	virtual void 			SetSectionRight(CubicBezierSplineSection *section);
	virtual void			SetSpline(CubicBezierSpline *spline);
	virtual void			SetSplineYforXinDataCurrent(bool tf);
	virtual void			SetXvalues(aList<long double>& localBinCenters);//sets first value in splineYforXinData_
	virtual void			SetYminMax(long double ymin, long double ymax);
	
//	virtual long double		t_from_x(long double x, CoordinateClass<long double>& finalCoords, 
//										long double tolerance=(long double)0.001);
//	virtual long double		t_from_x(long double x, bool monotonic_x);
	virtual bool			xIsInSection(long double x);
	virtual long double		xL(void);//left end of range
	virtual long double		xR(void);//right end of range
	virtual long double		yL(void);//left end of range
	virtual long double		yR(void);//right end of range
	


	friend int	operator==(CubicBezierSplineSection& p, CubicBezierSplineSection& q);
	friend int	operator!=(CubicBezierSplineSection& p, CubicBezierSplineSection& q);

	friend class CubicBezierSplineParamRec;
//	friend class BezierNode;
//	friend class BezierVariable;

	};//end class CubicBezierSplineSection



#endif