/******************************************************
*
*	CubicBezierSplineBetaParamRec.h
*
*	Contains the parameters and functions required to fit a cubic Bezier spline to data,
*	assuming a Beta distribution of error constant along the spline.  Use this for 
*	bounded data.
*
*
******************************************************/

#ifndef CubicBezierSplineBetaParamRech
#define CubicBezierSplineBetaParamRech = 1


#include "MemoryManager.h"
#include "InlineTemplates.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "CoordinateClass.h"
#include "Stringclass.h"
#include "CubicBezierSpline.h"
#include "CubicBezierSplineSection.h"
#include "BezierNode.h"
#include "ParamRec.h"
//#include "BezierVariable.h"

class CubicBezierSplineBetaParamRec;
//class BezierVariable;

//variableParamLookupTable [paramNum][2] --> node, nodeparam= {x=0,y=1,angle=2,distL=3,distR=4}

class CubicBezierSplineBetaParamRec: virtual public ParamRec{
  public:
		
  public:
	long double beta_;//used to model spread
	CubicBezierSpline *spline_;
  	ObjList<aList<int> > *variableParamLookupTable_;//an external table, holding info to link to nodes and their params
  							//indexed [param], holds node (-1=sigma), paramInNode, sections affected (-1=all sections)
  	StatList<long double> lnLbySection_;
  	aList<int> sectionsToRecalculate_lnL_;
  	ObjList<aList<int> > *dataLocationsBySection_;//length of data
  	long double toleranceFactor_;
  	long double rangeLowBound_,rangeHighBound_;//extreme values that the original distribution can take;
  													//scaled to {0,1} bounds during analysis
  	long double alphaLow_,alphaHigh_;
  	
  	
	
  public:
  	CubicBezierSplineBetaParamRec(void);
  	CubicBezierSplineBetaParamRec(long double beta, CubicBezierSpline *spline, 
  							   ObjList<aList<int> > *variableParamLookupTable,
							   StatList<long double>& lnLbySection, aList<int>& sectionsToRecalculate_lnL,
							   ObjList<aList<int> > *dataLocationsBySection
							   );
	CubicBezierSplineBetaParamRec(CubicBezierSplineBetaParamRec& p);
  	virtual ~CubicBezierSplineBetaParamRec(void);

  	virtual ParamRec* 				Duplicate(void);
  	CubicBezierSplineBetaParamRec* 	_Duplicate(void);
	virtual void 					SetPointerToDerivedSelf(void);//always override in derived
	virtual long 					GetPointerToDerivedSelf(void);//always override in derived
									//handle case where OS moves pointers to new memory locations
	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

  	


			long double beta(void);//returns beta_
			void 		beta(long double b);//sets beta_	
	virtual int 		ChangeBezierParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=one);
	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=one);
	virtual int			ChangeParams(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, 
							aList<int>& paramChoices, int& paramsToChooseAmong, 
							bool sequentialParamVariation, long double weight=zero);
			int 		ChangeBeta(Ran3& ran, long double sigmaForNewBeta, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void		CheckToleranceFactor(long double newFactor);
	virtual inline int	ErrorDistParams(void){return 1;}
	virtual long double ExpectedValue(aList<long double>& coords);
	virtual long double ExpectedValue(aList<long double>& coords, long double toleranceFactor);
	virtual void		FindMinMax(int ss,CoordinateClass<long double>& coordsMinX, 
													CoordinateClass<long double>& coordsMaxX);
	virtual int 		MeetsParamCriteria(ParamRec* /*lowerLimits*/, ParamRec* /*upperLimits*/);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumDerivedParams(void);//number of parameters that are functions of varying parameters 
	virtual int 		NumParams(void);//number of parameters
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual void		SectionsAtNodes(aList<int>& nodelist, aList<int>& newSectionList);
	virtual int			SectionAt_x(long double x);
			void		SetAlphaRange(long double alphaLow, long double alphaHIgh);
			void		SetDataLocationsBySection(ObjList<aList<int> > *newLocations);//deletes old
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual void 		SetParamValueAllNodes(int paramNumAtNode, long double newVal);//use to set deviations & upper/lower limits
	virtual void		SetRangeBoundaries(long double lowbound, long double highbound);
	virtual void		SetToleranceFactor(long double newFactor);
	virtual void		SetVariableParamLookupTable(ObjList<aList<int> > *vplt);
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file


	friend int	operator==(CubicBezierSplineBetaParamRec& p, CubicBezierSplineBetaParamRec& q);
	friend int	operator!=(CubicBezierSplineBetaParamRec& p, CubicBezierSplineBetaParamRec& q);

	friend class CubicBezierSplineSection;
	friend class BezierNode;

	};//end class CubicBezierSplineBetaParamRec



#endif