/**************************************************
*
*	MetropolisHastings.h
*
*	This class optimizes a parameter estimate given
*	a likelihood function and a data set.  It is intended  
*	to be generic, and derived classes of the likelihood function,
*	data set, and parameter classes needed.
*
**************************************************/

#ifndef MetropolisHastingsh
#define MetropolisHastingsh = 1

#include "ParamRec.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "SortList.h"
#include "Stringclass.h"
#include "Randomness.h"
#include "InlineTemplates.h"
#include "MemoryManager.h"
#include <fstream.h>
#include <iostream.h>


//	20070903	move inlines to .cpp file
//	20090728	correct linking of liklihood function pointers to the class, to fit C++ standard
//	20090728	removed pointerToDerivedSelf_ from MetropolisHastingsData -- always allocate dynamically using GetPointerToDerivedSelf()
//	20090728	converted to long double


class MetropolisHastings;
class MetropolisHastingsData;//a container class, defined below


typedef long double (*LikelihoodFunc)(ParamRec&, MetropolisHastingsData*);
	/* Returns the log likelihood (lnL) of the parameters given the data*/
	/* Must specify the exact form of the likelihood function when*/
	/* before using the MetropolisHastings object */


/*default Likelihood function*/
long double EquallyLikely(ParamRec&, MetropolisHastingsData*);//returns 0.0
	//defined at end of .cpp file

class MetropolisHastings{
  public:
  	LikelihoodFunc LogLikelihood;

  	MetropolisHastingsData* data;
  	ParamRec *paramsInUse, *paramsToTry, *bestParams;
  	ParamRec *paramDeviations, *origParamDeviations;
  	ParamRec *lowerConstraints, *upperConstraints;//use pointers because paramRec is virtual base class
  	ParamRec *lower2unitLimits, *upper2unitLimits;//use pointers because paramRec is virtual base class
  	SortList<ParamRec> savedParamList;
  	StatList<long double> lower2unitDerivedParams, upper2unitDerivedParams;

	StatList<long> paramChangesAccepted, paramChangesTried;//assess efficiency
	aList<int> runOfSuccessfulChanges, runOfFailedChanges;//use to tweak sampling range
	long totalTries;
	long double temperature;//defines sampling range
	long double annealedTemperature;//narrowest sampling range when annealing
	long double annealTempSteps;//temperature to drop at each annealing step; 0.9 usually
	long repsBetweenAnnealSteps;
	int savingNow;
	int annealed;
	int supportLimitsCalculated;
	long repsToSave_;
	int repsBetweenSaves_;
	
  public:
	MetropolisHastings(void);
	
	MetropolisHastings(ParamRec& initialParams, ParamRec& initParamDeviations,
							ParamRec& lowerParamLimits, ParamRec& upperParamLimits,
							MetropolisHastingsData* theData);
	
	virtual ~MetropolisHastings(void);
	
	virtual void Initialize(ParamRec& initialParams, ParamRec& initParamDeviations,
							ParamRec& lowerParamLimits, ParamRec& upperParamLimits,
							MetropolisHastingsData* theData);
	
  public:
	void 	Anneal(Ran3& ran, long double startTemperature, long double endTemperature, bool reportProgress=true,
					long double tempStep = (long double)0.9, long repsBetweenAnnealings = 250, bool sequentialParamVariation=false);
	virtual void 	Clear(void);
	void	FitOfTestParams(aString& outFileName);
	void 	Find2unitSupportLimits(void); //call after SampleForSupportLimits()
	void 	PrintParamEstimates(aString& outFileName, long seed, bool printSupport, int tabSeparated, aString& info); //estimates & 2-unit support limits
	void 	PrintParamEstimates(ostream& outfile, long seed, bool printSupport, int tabSeparated, aString& info); 
	void 	PrintSavedParams(aString& outFileName); 
	void 	PrintSavedParams(ostream& outfile); 
	void 	SampleForSupportLimits(Ran3& ran, long double temperature, bool reportProgress=true, 
						long repsToSave=2000, int repsBetweenSaves=50, bool sequentialParamVariation=false);
	int 	SaveParam(int newBest, int rescale);//saves paramsToTry and returns 1 if list is full, 0 otherwise
	void 	SetLikelihoodFunction (LikelihoodFunc theLikelihoodFunction);

	};//end class MetropolisHastings





class MetropolisHastingsData{
	public:
	//a dummy container class used for communicating between
	//data sets and likelihood functions in the MetropolisHastings algorithms
//	long pointerToDerivedSelf_;//VIRTUAL BASE CLASS COPYING TRICK

	public:	
	//use this class as a shared base class for any data set classes.
	MetropolisHastingsData(void);
	
	MetropolisHastingsData(MetropolisHastingsData& mhd);
	
	~MetropolisHastingsData(void){}
	
	virtual MetropolisHastingsData* Duplicate(void);
	
	void Clear(void){}
	
//	virtual void SetPointerToDerivedSelf(void);//always override in derived
	virtual long GetPointerToDerivedSelf(void);//always override in derived

	};//end class MetropolisHastingsData
#endif

