/*********************************************
*
*	ParamRec.h
*
*	This base class holds a set of parameter values that
*	are used in a metropolis algorithm for numerically 
*	maximizing an estimate.
*
*	Use derived classes to specify the names of the parameters,
*	and derive functions to vary these values and check
*	their limits.
*
*********************************************/

#ifndef ParamRech
#define ParamRech = 1

#include <math.h>
#include <ostream.h>
#include "aError.h"
#include "InlineTemplates.h"
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "RandomList.h"
#include "MemoryManager.h"


//20090728	removed pointerToDerivedSelf_ and SetPOiontertoDerivedSelf() --> always find dynamically using GetPointerToDerivedSelf()
//20090728	attached MamoryManager, mostly for use in derived classes
//20090728	converted everything to long double

class ParamRec {
  public:
	long double lnL_;
	long double lnLmaxMinusLnL_;
//	ParamRec* pointerToDerivedSelf;//VIRTUAL BASE CLASS COPYING TRICK
//	long pointerToDerivedSelf;//VIRTUAL BASE CLASS COPYING TRICK; obsolete ->use dynamic referencing
	
  public:
  	ParamRec(void):lnL_(zero),lnLmaxMinusLnL_(zero)
//  	,pointerToDerivedSelf(0)
  	{}
  	ParamRec(ParamRec& p):
  				lnL_(p.lnL_),
  				lnLmaxMinusLnL_(p.lnLmaxMinusLnL_)
//  				,pointerToDerivedSelf(0)
  				{
//  			SetPointerToDerivedSelf();
  			}  	
  	ParamRec(ParamRec* p):
  				lnL_(p->lnL_),
  				lnLmaxMinusLnL_(p->lnLmaxMinusLnL_)
//  				,pointerToDerivedSelf(0)
  				{
//  			SetPointerToDerivedSelf();
			}
  	virtual ~ParamRec(void){
//  			pointerToDerivedSelf = 0;//forget this
  			}
  	
  	//OVERRIDE THESE IN ALL DERIVED CLASSES
  	virtual ParamRec* 	Duplicate(void);
//	virtual void 		SetPointerToDerivedSelf(void);
	virtual long 		GetPointerToDerivedSelf(void);
	
  	
	virtual int 	 	ChangeAnnealingParams(Ran3& ran, //returns which parameter was changed
									ParamRec* paramVarationValues, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
									//modify this function when some parameters already have known maxima

									
	virtual int 	 	ChangeAnnealingParams(Ran3& ran, //returns which parameter was changed
									ParamRec* paramVarationValues, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& paramChoices, int& paramsToChooseAmong,
									bool sequentialParamVariation=false);
									//modify this function when some parameters already have known maxima
									
	virtual int 		ChangeParam(int /*paramNum*/, Ran3& /*ran*/, 
									ParamRec* /*paramVarationValues*/, 
									ParamRec* /*lowerLimits*/, ParamRec* /*upperLimits*/, long double weight=one);
							//returns 0 for failure, 1 for success
							//shouldn't arrive here; use derived classes
	virtual int 	 	ChangeParams(Ran3& /*ran*/, //returns which parameter was changed
									ParamRec* /*paramVarationValues*/, 
									ParamRec* /*lowerLimits*/, ParamRec* /*upperLimits*/
									);
	virtual int 	 	ChangeParams(Ran3& /*ran*/, //returns which parameter was changed
									ParamRec* /*paramVarationValues*/, 
									ParamRec* /*lowerLimits*/, ParamRec* /*upperLimits*/,
									aList<int>& paramChoices,  int& paramsToChooseAmong,
									bool sequentialParamVariation=false, long double weight= one
									);
	virtual long double ExpectedValue(aList<long double>& /*coords*/);
			long double lnL(void);
			void 		lnL(long double lnLval);
			long double lnLmaxMinusLnL(void);//on log scale
			long double LtoLcurrentRatio(long double Lcurrent);//not on log scale
			long double LtoLmaxRatio(void);//not on log scale
	virtual int 		MeetsParamCriteria (ParamRec* /*lowerLimits*/, ParamRec* /*upperLimits*/);
									//shouldn't arrive here; use derived classes
	virtual void 		MultiplyParam(int /*paramNum*/, long double /*factor*/);//use to rescale a param deviation
	virtual int 		NumParams(void);//number of parameters that could vary
	virtual int 		NumDerivedParams(void);//number of parameters that are functions of varying parameters 
			int 		Outside2unitSupportLimits(void);
	virtual aString* 	ParamName(int /*paramNum*/);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int /*paramNum*/);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep=0);
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual int 		RandomParameter(Ran3& ran);
	virtual int 		RandomParameter(Ran3& ran, aList<int>& paramChoices);
	virtual int 		RandomParameterSequential(Ran3& ran, aList<int>& paramChoices, int& paramsToChooseAmong);
								//use for sequential param selection; paramsToChooseAmong decremented and paramChoices list elements swapped
			void 		SetLmaxRatio(long double Lmax);
	virtual void 		SetParamValue(int /*paramNum*/, long double /*newVal*/);//sets value of parameter
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
	virtual int 		TotalParams(void);//number of parameters that could vary, plus derived paramerters
			int 		Within2unitSupportLimits(void);
	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

			friend int	operator==(ParamRec& p, ParamRec& q);
			friend int	operator!=(ParamRec& p, ParamRec& q);
			friend int	operator>(ParamRec& p, ParamRec& q);
			friend int	operator<(ParamRec& p, ParamRec& q);
			friend int	operator>=(ParamRec& p, ParamRec& q);
			friend int	operator<=(ParamRec& p, ParamRec& q);

	};//end base class ParamRec


#endif

