/******************************************
*
*	DioeciousGeneticLandScape.cpp
*
*   This header defines the derived LandScape class
*	that forms the basis for individuals with genetic information.
*	It seeds the landscape with base-class genetic individuals,
*	and presumes constant habitat quality.  Derived
*	classes add features and/or take data.
******************************************/

#ifndef DioeciousGeneticLandScapecp
#define DioeciousGeneticLandScapecp = 1

#include "DioeciousGeneticLandScape.h"
#include "RandomList.cpp"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"



//	CONSTRUCTORS AND DESTRUCTORS
DioeciousGeneticLandScape::DioeciousGeneticLandScape(LandScapeInitializer01& init):
	/*general purpose initializer*/
	SexualGeneticLandScape(init){}



// OVERRIDDEN AND NEW FUNCTIONS
void DioeciousGeneticLandScape::DoOneCycle (int species, long /*gen*/, Ran3List& ran, double mutationRate, double sigma){
	//PROBABLY BEST TO OVERRIDE THIS IN DERIVED CLASSES
//	AdultDispersal(species, ran);
	AdultDispersal(species, ran,mutationRate,sigma);
//	MateOnGrid(species, ran);
	Reproduce(species, ran, mutationRate, sigma);//mating done here for ideal hermaphrodite reproduction
	AgeOneStep(species);
	SomeAdultsDie(species, ran);
	ReplaceTheDead(species, ran);

	//DERIVED should take data from dead individuals before calling BuryTheDead
	BuryTheDead(species);
	
	}//end DioeciousGeneticLandScape::DoOneCycle
	

void DioeciousGeneticLandScape::ILandScape (Ran3List& ran){
	//only called just after constructors. Exists to avoid code duplication
	CoordinateType coords;
	for(long row=0;row<len_;++row){
		ObjList<GridCell>*  gridList = new ObjList<GridCell>();
		grid_.Append(gridList);
		for(long col=0;col<width_;++col){
			coords.set(row,col);
			GeneticGridCell* g = new GeneticGridCell(this, coords,0);//habitat=0
			gridList->Append(g);
			for (int s=0;s<species_;++s){
				DioeciousGeneticPopulation* pop= new DioeciousGeneticPopulation(
								totalPopsCreated_++,s,g,maxCapacity_[s]);
				g->Append(pop);
				}//end for species
			}//end for cols
		}// end for rows
	CreateMigrationMatrix();	
	cout<<len_<<" x "<<width_<<" landscape created"<<endl;
	for (int s=0;s<species_;++s){
		SeedLandScape(s, initSize_[s], ran);}
	}//end DioeciousGeneticLandScape::ILandScape




void DioeciousGeneticLandScape::ILandScapeByHabitat (Ran3List& ran, ObjList<aList<int> >& habitatGrid,
					ObjList<ObjList<ObjList<Phenotype> > >& genesByFreqByHabitatBySpecies,
					ObjList<ObjList<aList<double> > >& freqsByHabitatBySpecies){
	/*only called just after constructors. Exists to avoid code duplication*/
	int row,col;
	CoordinateType coords;
	for(row=0;row<len_;++row){
		ObjList<GridCell>*  gridList = new ObjList<GridCell>();
		grid_.Append(gridList);
		for(col=0;col<width_;++col){
			coords.set(row,col);
			GeneticGridCell* g = new GeneticGridCell(this, coords,0);/*habitat=0*/
			gridList->Append(g);
			for (int s=0;s<species_;++s){
				DioeciousGeneticPopulation* pop= new DioeciousGeneticPopulation(
								totalPopsCreated_++,s,g,maxCapacity_[s]);
				g->Append(pop);
				}/*end for species*/
			}/*end for cols*/
		}/* end for rows	*/
	SetHabitatTypes(habitatGrid);
	CreateMigrationMatrix();	
	cout<<len_<<" x "<<width_<<
			" landscape created"<<endl;
	for (int s=0;s<species_;++s){
				SeedLandScapeByHabitat(s, initSize_[s], ran,
					genesByFreqByHabitatBySpecies,freqsByHabitatBySpecies);}
	}/*end DioeciousGeneticLandScape::ILandScapeByHabitat*/




void DioeciousGeneticLandScape::SeedLandScape (int species, long initSize, Ran3List& ran){
	//puts dioecious sexual genetic individuals randomly into an empty landscape
	CoordinateType loc;
	//make a weighted aList for genotypes, to be sure exact frequencies are seeded
	StatList<long> geneWeights;
	ObjList<Phenotype>& geneList = SeededGeneList(species,0);
	aList<double>& freqs = SeededGeneFreqList(species,0);
	for(long i=0;i<geneList.len();++i){
		long b= (long) (initSize * (freqs[i]));
		geneWeights.Append(b);
		}//end for i
	while (geneWeights.sum()<initSize){//in case frequencies don't divide exactly into initSize
			geneWeights.Increment(geneWeights.len()-1);//increment the last one
			}//end while
	StatList<long> sexWeights;
	long girls=(long) (initSize*intrinsicSexRatio(species));
	long boys=(long) (initSize*(1.0-intrinsicSexRatio(species)));
	if(initSize>girls+boys){++girls;}//may need to round off: if so add a female
	sexWeights.Append(girls);
	sexWeights.Append(boys);
	while (geneWeights.sum()<initSize){//in case frequencies don't divide exactly into initSize
			geneWeights.Increment(geneWeights.len()-1);//increment the last one
			}//end while
		
	//make and initialize each individual and install it in a specified (presently random) grid location	
	for (long i=0;i<initSize;++i){
		loc.RandomLoc(0,len_-1,0,width_-1,ran);
		long x=ran.weighted(geneWeights);
		geneWeights.Decrement(x);//samples without replacement
		Phenotype& newGenes= geneList[x];
		long y=ran.weighted(sexWeights);//0=girls, 1=boys
		sexWeights.Decrement(y);//samples without replacement
		genderSet sex = (y) ? M:F;//0=girls, 1=boys
//		DioeciousGeneticIndividual* indiv = new DioeciousGeneticIndividual(newGenes,++totalIndivsCreated_);
		DioeciousGeneticIndividual* indiv = new DioeciousGeneticIndividual(newGenes,sex,++totalIndivsCreated_);
		indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran);//age=1
		InstallInGrid(species, indiv, loc);}
	}//	end DioeciousGeneticLandScape::SeedLandScape 



void DioeciousGeneticLandScape::SeedLandScapeByHabitat (int species, long initSize, Ran3List& ran, 
					ObjList<ObjList<ObjList<Phenotype > > >& genesByFreqByHabitatBySpecies,
					ObjList<ObjList<aList<double> > >& freqsByHabitatBySpecies){
	//puts genetic individuals into an empty landscape
	//according to their genotype and habitat
	StatList<long> sexWeights;
	long girls=(long) (initSize*intrinsicSexRatio(species));
	long boys=(long) (initSize*(1.0-intrinsicSexRatio(species)));
	if(initSize>girls+boys){++girls;}//may need to round off: if so add a female
	sexWeights.Append(girls);
	sexWeights.Append(boys);
	//make a weighted aList for genotypes that will be seeded
	ObjList<ObjList<Phenotype> >& genesByFreqByHabitat=genesByFreqByHabitatBySpecies[species];
	ObjList<aList<double> >& freqsByHabitat=freqsByHabitatBySpecies[species];
	//make up a list of gridcells of each habitat
	ObjList<ObjList<GridCell> > gridcellsByHabitat;
	for(int hh=0;hh<freqsByHabitat.len();++hh){
		ObjList<GridCell>* cellsWithHabitat = new ObjList<GridCell>();
		gridcellsByHabitat.Append(cellsWithHabitat);}
	for (long x=0;x<len_;++x){
		for (long y=0;y<width_;++y){
			GridCell& g= grid_[x][y];			
			ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[g.habitat()];
			cellsWithHabitat.Append(&g);
			}}	

	for(long h=0;h<freqsByHabitat.len();++h){
		StatList<long> geneWeights;
		aList<double>& genomeFreqs = freqsByHabitat[h];
		ObjList<Phenotype>& genesInHabitat = genesByFreqByHabitat[h];
		ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[h];
		long indivsInHabitat	= (long)( initSize *( ((double) (cellsWithHabitat.len() )) / ((double) ( len_ * width_))  ) );
		for(int gg=0;gg<genesInHabitat.len();++gg){			
			long n= (long) (indivsInHabitat * (genomeFreqs[gg]));
			geneWeights.Append(n);}
		while (geneWeights.sum()<indivsInHabitat){//in case frequencies don't divide exactly into initSize
			geneWeights.Increment(geneWeights.len()-1);//increment the last one
			}//end while

		//make and initialize each individual and install it in a specified (presently random) grid location
		//that has the proper habitat type
		for (long i=0;i<indivsInHabitat;++i){
			int cellNum = ran.longBetween(0,cellsWithHabitat.len()-1);
			GridCell& g = cellsWithHabitat[cellNum];
			CoordinateType loc = g.coords();
			long x=ran.weighted(geneWeights);
			geneWeights.Decrement(x);//samples without replacement
			Phenotype& newGenes=genesInHabitat[x];
			long y=ran.weighted(sexWeights);//0=girls, 1=boys
			sexWeights.Decrement(y);//samples without replacement
			genderSet sex = (y) ? M:F;//0=girls, 1=boys
//			DioeciousGeneticIndividual* indiv = new DioeciousGeneticIndividual(newGenes,++totalIndivsCreated_);
			DioeciousGeneticIndividual* indiv = new DioeciousGeneticIndividual(newGenes,sex,++totalIndivsCreated_);
			indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran);//age=1
			InstallInGrid(species, indiv, loc);
			}//end for i
		
		}//end for h
	
	//clean up
	for(int h=0;h<gridcellsByHabitat.len();++h){
		ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[h];
		cellsWithHabitat.Detach();}
	gridcellsByHabitat.Clear();

	}//	end DioeciousGeneticLandScape::SeedLandScapeByHabitat 




#endif

