/******************************************************
*
*	BezierNode.cpp
*
*	parameters for the node (control point on the curve) of a Bezier curve
*
*	the node holds its Cartesian coordinates, and polar coordinates describing 1-2 handles that define the curve shape.
*
******************************************************/

#ifndef BezierNodecpp
#define BezierNodecpp = 1


#include "BezierNode.h"
#include "CubicBezierSplineSection.h"
#include "Element.cpp"
#include "aList.cpp"
#include "CoordinateClass.cpp"


	
BezierNode::BezierNode(void):angle_(zero), distR_(zero), distL_(zero), terminalL_(true), terminalR_(true),
								section0_(NULL), section1_(NULL){}


BezierNode::BezierNode(CoordinateClass<long double>& coords):
  					coords_(coords), angle_(zero), distL_(zero), distR_(zero), 
  					terminalL_(false), terminalR_(false),
								section0_(NULL), section1_(NULL){}

BezierNode::BezierNode(CoordinateClass<long double>& coords, long double angle, long double distL, long double distR,
  								bool terminalL, bool terminalR):
  					coords_(coords), angle_(angle), distL_(distL), distR_(distR), 
  					terminalL_(terminalL), terminalR_(terminalR),
								section0_(NULL), section1_(NULL){}

BezierNode::BezierNode(BezierNode& bn){
				operator=(bn);
				}

BezierNode::~BezierNode(void){}
  				

BezierNode*	BezierNode::BezierNode::Duplicate(void){
	return NEW BezierNode(*this);}


void BezierNode::operator=(BezierNode& bn){
	coords_=bn.coords_;
	angle_=bn.angle_;
	distL_=bn.distL_;
	distR_=bn.distR_;
	terminalL_=bn.terminalL_;
	terminalR_=bn.terminalR_;
	section0_=bn.section0_;
	section1_=bn.section1_;
	}

void BezierNode::CopyDataFieldsOnly(BezierNode& bn){
	coords_=bn.coords_;
	angle_=bn.angle_;
	distL_=bn.distL_;
	distR_=bn.distR_;
	terminalL_=bn.terminalL_;
	terminalR_=bn.terminalR_;
	}


void BezierNode::DetachPointers(void){
		//detaches but doesn't delete section pointers -- virtual function
  	DetachSections();
  	}


void BezierNode::DetachSections(void){
		//detaches but doesn't delete section pointers
  	section0_ = section1_ = NULL;
  	}


long double	BezierNode::Angle(void) {
	return angle_;}



bool BezierNode::ChangeNodeParam(int paramToChange, Ran3& ran, 
						BezierNode& deviations, BezierNode& lowNode, BezierNode& highNode){
	//x=0,y=1,angle=2,distL=3,distR=4; returns 0 for failure, 1 for success
	bool returnval=false;
	int countAttempts=0;
	long double oldVal, newVal, sigma, lowdev=(long double)0.0001;
	bool illegal=true;
	switch(paramToChange){
		case 0://x - won't be called if this is a terminal node
			oldVal = coords_[0];
			sigma = deviations.coords_[0];
			while(illegal){
				++countAttempts;
				newVal=ran.NormDev(oldVal,sigma);
				if(newVal>section0_->node0_->coords_[0] && newVal<section1_->node1_->coords_[0]){
					illegal=false;//must remain between adjacent nodes 
					returnval=true;
					}
				if(countAttempts>20 && illegal==true){
					countAttempts=0;
					deviations.coords_[0] /= two;
					if(deviations.coords_[0] < lowdev){
						returnval=false;
						illegal=false;
						}//if deviations
					}//if countAttempts
				}//while
			if(returnval==0){
					coords_[0]=oldVal;}
				else{
						coords_[0] = newVal;
					if(section0_ != NULL){
						section0_->SetEndpoints();}
					if(section1_ != NULL){
						section1_->SetEndpoints();}
					}
			break;
		case 1://y
/*
			oldVal = coords_[1];
			sigma = deviations.coords_[1];
//			illegal=true;
			while(illegal){
				newVal=ran.NormDev(oldVal,sigma);
				if(newVal> lowNode.coords_[1] || newVal < highNode.coords_[1]){
					illegal=false;}
				}//while
			coords_[1] = newVal;
			if(section0_ != NULL){
				section0_->SetEndpoints();}
			if(section1_ != NULL){
				section1_->SetEndpoints();}
			returnval=true;
			break;
*/
			oldVal = coords_[1];
			sigma = deviations.coords_[1];
			while(illegal){
				++countAttempts;
				newVal=ran.NormDev(oldVal,sigma);
				if(newVal>section0_->node0_->coords_[1] && newVal<section1_->node1_->coords_[1]){
					illegal=false;//must remain between adjacent nodes 
					returnval=true;
					}
				if(countAttempts>20 && illegal==true){
					countAttempts=0;
					deviations.coords_[1] /= two;
					if(deviations.coords_[1] < lowdev){
						returnval=false;
						illegal=false;
						}//if deviations
					}//if countAttempts
				}//while
			if(returnval==0){
					coords_[1]=oldVal;}
				else{
						coords_[1] = newVal;
					if(section0_ != NULL){
						section0_->SetEndpoints();}
					if(section1_ != NULL){
						section1_->SetEndpoints();}
					}
			break;
		case 2://angle
			oldVal = angle_;
			sigma = deviations.angle_;
			long double halfpi=PI()/two;
			while(illegal){
				++countAttempts;
				newVal=ran.NormDev(oldVal,sigma);
				if(newVal < highNode.angle_ && newVal > lowNode.angle_){//cannot be at or past vertical
					illegal=false;
					returnval=true;
					}
				if(countAttempts>20 && illegal==true){
					countAttempts=0;
					deviations.angle_ /= two;
					if(deviations.angle_ <= 0.0001){
						returnval=false;
						illegal=false;
						}//if deviations
					}//if countAttempts
				}//while
			if(returnval==1){
					angle_ = newVal;}
				else{
					angle_=oldVal;}
			break;
		case 3://distL
			oldVal = distL_;
			sigma = deviations.distL_;
			while(illegal){
				++countAttempts;
				newVal=ran.NormDev(oldVal,sigma);
				if(newVal >= lowNode.distL_){//no negative distances
					illegal=false;
					returnval=true;
					}
				if(countAttempts>20 && illegal==true){
					countAttempts=0;
					deviations.distL_ /= two;
					if(deviations.distL_ < lowdev){
						returnval=false;
						illegal=false;
						}//if deviations
					}//if countAttempts
				}//while
			if(returnval){
					distL_ = newVal;}
				else{
					distL_=oldVal;}
			break;
		case 4://distR
			oldVal = distR_;
			sigma = deviations.distR_;
			while(illegal){
				++countAttempts;
				newVal=ran.NormDev(oldVal,sigma);
				if(newVal >= lowNode.distR_){//no negative distances
					illegal=false;
					returnval=true;
					}
				if(countAttempts>20 && illegal==true){
					countAttempts=0;
					deviations.distL_ /= two;
					if(deviations.distR_ < lowdev){
						returnval=false;
						illegal=false;
						}//if deviations
					}//if countAttempts
				}//while
			if(returnval){
					distR_ = newVal;}
				else{
					distR_=oldVal;}
			break;
		default:
			break;
		}//switch
	if(returnval==true){
		NotifySplineOfChange(paramToChange);
		}
	return returnval;
	}//BezierNode::ChangeNodeParam()


	

long double	BezierNode::HandleDistanceLeft(void){
	return distL_;}


long double	BezierNode::HandleDistanceRight(void){
	return distR_;}


void BezierNode::HandleCartesianLeft(CoordinateClass<long double>& newCoords){
	//left handle position
	if(terminalL_) {newCoords = coords_;}
		else{
//			newCoords[0]= coords_[0] - distL_ * cos(angle_);
//			newCoords[1]= coords_[1] - distL_ * sin(angle_);
			long double c0=coords_[0],c1=coords_[1],dL=distL_,cosa=cos(angle_),sina=sin(angle_),newX,newY;
			newX = c0 - dL * cosa;
			newY = c1 - dL * sina;
			CoordinateClass<long double> tempCoords(newX,newY,zero);
			newCoords = tempCoords;
			}
	}
	
	
void BezierNode::HandleCartesianRight(CoordinateClass<long double>& newCoords){
	if(terminalR_) {newCoords = coords_;}
		else{
//			newCoords[0]= coords_[0] + distL_ * cos(angle_);
//			newCoords[1]= coords_[1] + distL_ * sin(angle_);
			long double c0=coords_[0],c1=coords_[1],dR=distR_,cosa=cos(angle_),sina=sin(angle_),newX,newY;
			newX = c0 + dR * cosa;
			newY = c1 + dR * sina;
			CoordinateClass<long double> tempCoords(newX,newY,zero);
			newCoords = tempCoords;
			}
	}


void BezierNode::MultiplyParam(int paramToChange, long double factor){
	//x=0,y=1,angle=2,distL=3,distR=4; returns 0 for failure, 1 for success
	//includes error-checking
	long double newVal;
	switch(paramToChange){
		case 0://x - won't be called if this is a terminal node
			newVal= coords_[0] * factor;
			if(newVal<= section0_->node0_->coords_[0] || newVal >= section1_->node1_->coords_[0]){
					aError("Error: BezierNode::MultiplyParam() yields a value out of bounds for coords_[0]");
					}
				else{
					coords_[0] = newVal;
					NotifySplineOfChange(0);
					}
			break;
		case 1://y -- any value acceptable
			coords_[0] *= factor;
			NotifySplineOfChange(1);
			break;
		case 2://angle
			newVal = angle_ * factor;
			long double halfpi=PI()/two;
			if(newVal > halfpi || newVal < -halfpi){//cannot be at or past vertical
					aError("Error: BezierNode::MultiplyParam() yields a value out of bounds for angle_");
					}
				else{
					angle_ = newVal;
					NotifySplineOfChange(2);
					}
			break;
		case 3://distL
			newVal = distL_ * factor;
			if(newVal < zero){//cannot be negative
					aError("Error: BezierNode::MultiplyParam() yields a value out of bounds for distL_");
					}
				else{
					distL_ = newVal;
					NotifySplineOfChange(3);
					}

			break;
		case 4://distR
			newVal = distR_ * factor;
			if(newVal < zero){//cannot be negative
					aError("Error: BezierNode::MultiplyParam() yields a value out of bounds for distR_");
					}
				else{
					distR_ = newVal;
					NotifySplineOfChange(4);
					}
			break;
		default:
			break;
		}//switch
	}//BezierNode::MultiplyParam(int,long double)	



void BezierNode::NodeCartesian(CoordinateClass<long double>& newCoords){
	newCoords = coords_;}


void BezierNode::NotifySplineOfChange(int paramChanged){//x=0,y=1,angle=2,distL=3,distR=4
	if(section0_ != NULL && paramChanged != 4){
		section0_->SetBezierParamsCurrent(false);
		section0_->SetBezierParams();
		}
	if(section1_ != NULL && paramChanged != 3){
		section1_->SetBezierParamsCurrent(false);
		section1_->SetBezierParams();
		}
//	if(section0_ != NULL && (paramChanged == 0 || paramChanged == 1)){
//		section0_->SetEndpoints();}
//	if(section1_ != NULL && (paramChanged == 0 || paramChanged == 1)){
//		section1_->SetEndpoints();}
	}



long double	BezierNode::Position_x(void){
	return coords_[0];
	}


long double	BezierNode::Position_y(void){
	return coords_[1];
	}


aString* BezierNode::ParamName(int paramAtNode){
	aString *returnStr=NEW aString();
	switch(paramAtNode){
		case 0:
			returnStr->cat("x"); break;
		case 1:
			returnStr->cat("y"); break;
		case 2:
			returnStr->cat("angle"); break;
		case 3:
			returnStr->cat("dist_L"); break;
		case 4:
			returnStr->cat("dist_R"); break;
		default:
		returnStr->clear(); break;
		}
	return returnStr;
	}//end BezierNode::ParamName(int )



void BezierNode::ParamName(int paramAtNode, aString& appendToThis){
	switch(paramAtNode){
		case 0:
			appendToThis.cat("x"); break;
		case 1:
			appendToThis.cat("y"); break;
		case 2:
			appendToThis.cat("angle"); break;
		case 3:
			appendToThis.cat("dist_L"); break;
		case 4:
			appendToThis.cat("dist_R"); break;
		default:
		break;
		}
	}//BezierNode::ParamName(int, aString&)


long double BezierNode::ParamValue(int paramAtNode){
	long double val=zero;
	switch(paramAtNode){
		case 0:
			val = Position_x(); break;
		case 1:
			val = Position_y(); break;
		case 2:
			val = Angle(); break;
		case 3:
			if(! terminalL_){
				val = HandleDistanceLeft(); }
			break;
		case 4:
			if(! terminalR_){
				val = HandleDistanceRight(); break;}
		default:
			break;
		}
	return val;
	}//end BezierNode::ParamName(int )



bool BezierNode::SetAngle(long double newAngle){//in radians
	long double halfpi = PI()/two, ang=newAngle;
	long double sgn = (long double) sign(ang);
	while(-halfpi < ang && ang > halfpi){
		ang -= sgn * PI();
		}
	angle_=ang;
	NotifySplineOfChange(2);
	return true;//never fails
	}//BezierNode::SetAngle(long double )



bool BezierNode::SetCoordinates(CoordinateClass<long double>& newCoords){
	bool testval=true;
	testval=Set_x(newCoords[0]);
	if(testval){
		testval=Set_y(newCoords[1]);}
	return testval;
	}
	
bool BezierNode::SetHandleDistanceLeft(long double newDist){
	bool testval=false;
	if(! terminalL_){
		if(newDist < zero) {
				distL_=zero;
				}
			else {
				distL_ = newDist;
				testval=true;}
		if(testval){
			NotifySplineOfChange(3);}
		}
	return testval;
	}
	
bool BezierNode::SetHandleDistanceRight(long double newDist){
	bool testval=false;
	if(! terminalR_){
		if(newDist < zero) {
				distR_=zero;}
			else {
				distR_ = newDist;
				testval=true;}
		if(testval){
			NotifySplineOfChange(4);}
		}
	return testval;
	}
	
	
void BezierNode::SetMathematicaFormString(aString& stringToFill){
	stringToFill = "{{";
	stringToFill.cat(coords_[0]);
	stringToFill.cat(", ");
	stringToFill.cat(coords_[1]);
	stringToFill.cat("}, ");
	stringToFill.cat(angle_);
	stringToFill.cat(", ");
	stringToFill.cat(distL_);
	stringToFill.cat(", ");
	stringToFill.cat(distR_);
	stringToFill.cat("}");
	}//BezierNode::SetMathematicaFormString(aString& stringToFill)
	


bool BezierNode::SetParamValue(int paramNum, long double newVal){
	bool testval=false;
	switch(paramNum){
		case 0: 
			testval=Set_x(newVal);
			break;
		case 1: 
			testval=Set_y(newVal);
			break;
		case 2: 
			testval=SetAngle(newVal);
			break;
		case 3: 
			testval=SetHandleDistanceLeft(newVal);
			break;
		case 4: 
			testval=SetHandleDistanceRight(newVal);
			break;
		default: 
			aString err("ERROR: BezierNode::SetParamValue(): param # ");
			err.cat(paramNum);
			err.cat(" is not in the range {0,4}");
			aError(err);
			break;
		}//switch
	return testval;
	}//BezierNode::SetParamValue(int, long double)


void BezierNode::SetSectionLeft(CubicBezierSplineSection *leftSection){
	section0_ = leftSection;
	}


void BezierNode::SetSectionRight(CubicBezierSplineSection *rightSection){
	section1_ = rightSection;
	}


void BezierNode::SetTerminalLeft(bool tf){
	terminalL_=tf;
	if(tf==true){
		distL_=zero;}
	}

void BezierNode::SetTerminalRight(bool tf){
	terminalR_=tf;
	if(tf==true){
		distR_=zero;}
	}

bool BezierNode::Set_x(long double new_x, int recursion){
	bool inRange=true;
	if(recursion==0){
		if(section0_!=NULL){
			if(section0_->xIsInSection(new_x)){
					inRange=true;
					section0_->node0_->Set_x(new_x,recursion+1);
					}
				else{
					inRange=false;
					}
			}
		if(inRange && section1_!=NULL){
			if(section1_->xIsInSection(new_x)){
					inRange=true;
					section1_->node1_->Set_x(new_x,recursion+1);
					}
				else{
					inRange=false;
					}
			}//section1_
		}//recursion
	if(inRange){
			coords_[0]=new_x;
			NotifySplineOfChange(0);			
			}
		else{
			aError("ERROR: BezierNode::Set_x() -- x is out of range");}
	return inRange;
	}//BezierNode::Set_x(long double)
	
	
bool BezierNode::Set_y(long double new_y){
	coords_[1]=new_y;
	NotifySplineOfChange(1);
	return true;
	}


bool BezierNode::TerminalNode(void){
	return terminalL_ || terminalR_;
	}

bool BezierNode::TerminalNodeLeft(void){
	return terminalL_;
	}

bool BezierNode::TerminalNodeRight(void){
	return terminalR_;
	}

CoordinateClass<long double>&	BezierNode::ValueAt_t(void){
	return coords_;
	}



int	operator==(BezierNode& p, BezierNode& q){
	return p.coords_==q.coords_ && p.angle_==q.angle_ && p.distL_==q.distL_ && p.distR_==q.distR_
				 && p.terminalL_==q.terminalL_  && p.terminalR_==q.terminalR_;
				 }
				 
				 
int	operator!=(BezierNode& p, BezierNode& q){
	return !(p==q);}




#endif