/*********************************************
*
*
*		FDistribution.cpp
*
*
*		Functions for calculating F-distribution statistics
*
*
**********************************************/


#ifndef FDistributioncp
#define FDistributioncp = 1

#include "FDistribution.h"
#include "aList.cpp"
#include "Element.cpp"


long double FDistribution(int df1, int df2, long double x){
	//calculates the height of F(df1,df2) at point x
	long double density = zero;
	if (x>zero){
		long double v1 = df1, v2 = df2;
		long double gamma12 = gammaln((v1+v2)/two);
		long double gamma1 = gammaln(v1/two);
		long double gamma2 = gammaln(v2/two);
		long double lnv1pow = (v1/two)*ln(v1);	
		long double lnv2pow = (v2/two)*ln(v2);	
	//	long double lnxv1 = (v1/(long double)2.0 -(long double)1.0)*ln(x);
		long double lnxv1 = (v1/two -one)*ln(x);
		long double v2v1x = v2 + v1*x;
		long double lnxv1v2 = (-(v1+v2)/two)*ln(v2v1x);
		long double lndensity = gamma12 - gamma1 - gamma2 + lnv1pow + lnv2pow + lnxv1 + lnxv1v2;
		density = exp(lndensity);
		}
	return density;
}//end FDistribution(int, int, double)


/*
double Fdistribution1TailedP(int df1, int df2, double observedF){
	//calculates the area under the F distribution from x = observedF back to x = 0
	double increment = 1.0e-5, areaUnderDistribution = 0.0;
	double x = observedF;
	double Fx = FDistribution(df1, df2, x);
	while (x>0.0){
		double xdec = x - increment;  if(xdec<0.0) {xdec=0.0;}
		double Fxdec = FDistribution(df1, df2, xdec);
		areaUnderDistribution += (Fx + Fxdec)*increment/2.0;//area under curve in increment, approx
		Fx = Fxdec;
		x=xdec;
		}//end while
	return 1.0 - areaUnderDistribution;
	}//end Fdistribution1TailedP
*/


long double Fdistribution1TailedP(int df1, int df2, long double observedF){
	//calculates the area under the F distribution from x = observedF back to x = 0
	long double areaToObsF = qrombFDistribution(df1, df2, zero, observedF, 30);
	long double areaUnderDistribution = qrombFDistribution(df1, df2, observedF, 1.0e4, 30);//1.0e4 approximates infinity here
	long double OneMinusAlphaArea = areaToObsF/(areaToObsF+areaUnderDistribution);
	return one - OneMinusAlphaArea;
	}//end Fdistribution1TailedP


long double qrombFDistribution(int df1, int df2, long double a, long double b, int n){
	//see Press et al. p. 129, a method called Romberg's integration procedure	
	//returns integral from a to b of the Fdistribution; n is a scaling constant 
	const long double eps = 1.0e-6;
	const int jmax = 23, jmaxp = jmax+1, k = 5;
	aList<long double> h, s;
	int ii, i, j;
	for(ii=0;ii<jmaxp;++ii){
		h.Append(zero);  s.Append(zero);}
	h[0]=one;
	aList<long double> c, d;
	for(ii=0;ii<n;++ii){
		c.Append(zero);  d.Append(zero);}
	long double ss=zero, dss=zero;
	int iterator_=1;
	for(j=0;j<jmax;++j){
		s[j]=trapzdFDistribution(df1, df2, a, b, s[j],j, iterator_);
		if(j>=k-1){
			for(i=0;i<k;++i){
				c[i] = h[(j+1)-k+(i+1)-1];
				d[i] = s[(j+1)-k+(i+1)-1];}
			polintFDistribution(c,d,k,n,zero,ss,dss);
			if(ABS(dss)< eps*ABS(ss)){
				break;}
			}//end if
		s[j+1] = s[j];
		h[j+1] = (long double)0.25 * h[j];
		}//end for j
	return ss;
	}//end 	qrombFDistribution

long double trapzdFDistribution(int df1, int df2, long double a, long double b, 
									long double s, int n, int& trapzdIterator){
	//see Press et al. p. 124, a rough integration procedure	
	//returns approximate integral from a to b of the Fdistribution
	//n is a scaling constant that determines the accuracy of the integral
	if (n==0){
			trapzdIterator=1;
			return (long double)0.5*(b-a)*(FDistribution(df1,df2,a) + FDistribution(df1,df2,b));
			}
		else{
			long double tnm = trapzdIterator;
			long double del = (b-a)/tnm;
			long double x = a + (long double)0.5*del;
			long double summ = zero;
			for (int j=0;j<trapzdIterator;++j){
				summ += FDistribution(df1, df2, x);
				x += del;}
			s = (long double)0.5*(s+(b-a)*summ/tnm);  
			if(s== -zero){s=zero;}
			trapzdIterator *= two;
			}
	return s;
	}//end trapzdFDistribution


void polintFDistribution(aList<long double>& xa, aList<long double>& ya, const int n, int np, 
						 long double x, long double& y, long double& dy){
	//see Press et al. p. 92, an interpolation/extrapolation procedure for polynomials
	//given x, and arrays xa and ya, it returns an approximate y and an approximate error dy
	aList<long double> c, d;	
	int i, m, ii;
	for (i=0;i<np;++i){
		c.Append(zero);
		d.Append(zero);}
	int ns = 0;
	long double dif = ABS(x - xa[0]);
	for (i=0;i<n;++i){
		long double dift  = ABS(x - xa[i]);
		if(dift<dif){
			ns=i;
			dif=dift;}
		c[i] = ya[i];
		d[i] = ya[i];
		}//end for i
	y = ya[ns];
	--ns;
	for(m=0;m<n-1;++m){
		for(ii=0;ii<n-m;++ii){
			long double ho = xa[ii]-x;
			long double hp = xa[(ii+1)+(m+1)-1]-x;
			long double w = c[ii+1]- d[ii];
			long double den = ho-hp;
			if(den==zero){
				cout<<"error in polintFDistribution"<<endl;}
			den = w/den;  
			if(den== -zero){den=zero;}
			d[ii] = hp*den;
			if(d[ii]== -zero){d[ii]=zero;}
			c[ii] = ho*den;
			if(c[ii]==-zero){c[ii]=zero;}
			}//end for ii
		if(2*(ns+1)<n-(m+1)){
				dy = c[ns+1];}
			else{
				dy = d[ns];
				--ns;}
		y += dy;
		}//end for m
	}//end polintFDistribution


#endif

