/******************************************************
*
*	CubicBezierSplineBetaParamRec.cpp
*
*	Parameters for a Beta distribution defined by alpha and beta.
*	Here, alpha is determined by the spline shape, and beta describes the error around
*	the spline.
*
*	Use this as a base class for derived classes that add parameters.
*
******************************************************/
#ifndef CubicBezierSplineBetaParamReccpp
#define CubicBezierSplineBetaParamReccpp = 1

#include "CubicBezierSplineBetaParamRec.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "CoordinateClass.cpp"


CubicBezierSplineBetaParamRec::CubicBezierSplineBetaParamRec(void):ParamRec(), beta_(one),
  					spline_(NULL), variableParamLookupTable_(NULL), //lnLbySection_(NULL),
  					//sectionsToRecalculate_lnL_(NULL),
  					dataLocationsBySection_(NULL),
  					toleranceFactor_((long double)1.0e-12), rangeLowBound_(zero), rangeHighBound_(one){
  				SetPointerToDerivedSelf();
  				}


CubicBezierSplineBetaParamRec::CubicBezierSplineBetaParamRec(long double beta, CubicBezierSpline *spline,
													 ObjList<aList<int> > *variableParamLookupTable,
													 StatList<long double>& lnLbySection,
													 aList<int>& sectionsToRecalculate_lnL,
													 ObjList<aList<int> > *dataLocationsBySection
													 ):
					ParamRec(), beta_(beta),
  					spline_(spline), variableParamLookupTable_(variableParamLookupTable),
  					lnLbySection_(lnLbySection), sectionsToRecalculate_lnL_(sectionsToRecalculate_lnL),
  					dataLocationsBySection_(dataLocationsBySection),
  					toleranceFactor_((long double)1.0e-12),rangeLowBound_(zero),rangeHighBound_(one){
  				SetPointerToDerivedSelf();
  				}


CubicBezierSplineBetaParamRec::CubicBezierSplineBetaParamRec(CubicBezierSplineBetaParamRec& p){
  				SetPointerToDerivedSelf();
				operator=(p);
				}



CubicBezierSplineBetaParamRec::~CubicBezierSplineBetaParamRec(void){
  				pointerToDerivedSelf = NULL;//forget all these
  				spline_ = NULL;
  				variableParamLookupTable_ = NULL;
  				dataLocationsBySection_ = NULL;
  				}



ParamRec* CubicBezierSplineBetaParamRec::Duplicate(void){
	CubicBezierSplineBetaParamRec* copy = NEW CubicBezierSplineBetaParamRec(*this);
	return copy;
	}//end CubicBezierSplineParamRec::Duplicate()



CubicBezierSplineBetaParamRec* CubicBezierSplineBetaParamRec::_Duplicate(void){
	return NEW CubicBezierSplineBetaParamRec(*this);
	}//end CubicBezierSplineParamRec::_Duplicate()



void CubicBezierSplineBetaParamRec::SetPointerToDerivedSelf(void){
  	pointerToDerivedSelf = (long) this;
  	}//end

long CubicBezierSplineBetaParamRec::GetPointerToDerivedSelf(void){//handle case where OS moves pointers to new memory locations
  	long pdsf = (long) this;
  	return pdsf;
  	}//end


void CubicBezierSplineBetaParamRec::operator=(ParamRec& p){
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class CubicBezierSplineParamRec
	CubicBezierSplineBetaParamRec* pp = (CubicBezierSplineBetaParamRec*) p.GetPointerToDerivedSelf();
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
	beta_ = pp->beta_;
	spline_ = pp->spline_;//copies pointer
  	variableParamLookupTable_ = pp->variableParamLookupTable_;//copies pointer
  	lnLbySection_ = pp->lnLbySection_;
  	sectionsToRecalculate_lnL_ = pp->sectionsToRecalculate_lnL_;
  	dataLocationsBySection_ = pp->dataLocationsBySection_;
  	toleranceFactor_ = pp->toleranceFactor_;
  	rangeLowBound_ = pp->rangeLowBound_;
  	rangeHighBound_ = pp->rangeHighBound_;
  	alphaLow_=pp->alphaLow_;
  	alphaHigh_=pp->alphaHigh_;
	}//end  CubicBezierSplineBetaParamRec::operator=(ParamRec&)



int CubicBezierSplineBetaParamRec::ChangeBezierParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double /*weight*/){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	CubicBezierSplineBetaParamRec* deviations = (CubicBezierSplineBetaParamRec*) paramDeviations->GetPointerToDerivedSelf();
	CubicBezierSplineBetaParamRec* lowLim = (CubicBezierSplineBetaParamRec*) lowerLimits->GetPointerToDerivedSelf();
	CubicBezierSplineBetaParamRec* highLim = (CubicBezierSplineBetaParamRec*) upperLimits->GetPointerToDerivedSelf();
	CubicBezierSpline *spline = spline_;
	int returnval,sec,s,node,paramAtNode;
	bool inRange = true;
	node =(*variableParamLookupTable_)[paramNum][0];
	paramAtNode =(*variableParamLookupTable_)[paramNum][1];
	BezierNode& primaryNode = spline_->nodeList_[node];
	
	//work with copies of node, its sections, and their nodes; transfer to spline if values are legal
	BezierNode primaryNodeCopy;
	primaryNodeCopy.CopyDataFieldsOnly(primaryNode);
	if(primaryNode.section0_ != NULL){//happens when primaryNode is the first node on the spline
		CubicBezierSplineSection *section0Copy=NEW CubicBezierSplineSection();//make left section copy
		section0Copy->CopyDataFieldsOnly(*(primaryNode.section0_));
		primaryNodeCopy.SetSectionLeft(section0Copy);
		section0Copy->node1_ = &primaryNodeCopy;//done hooking up the noed and left section
		BezierNode *nodeLeftCopy = NEW BezierNode();//make left section's far node
		nodeLeftCopy->CopyDataFieldsOnly(*(primaryNode.section0_->node0_));
		section0Copy->node0_ = nodeLeftCopy;
		nodeLeftCopy->section1_ = section0Copy;//done hooking it up to the left section
		nodeLeftCopy = NULL;//forget these -- they're accessible through primaryNodeCopy
		section0Copy =NULL;
		}
	if(primaryNode.section1_ != NULL){//happens when primaryNode is the last node on the spline
		CubicBezierSplineSection *section1Copy=NEW CubicBezierSplineSection();
		section1Copy->CopyDataFieldsOnly(*(primaryNode.section1_));
		primaryNodeCopy.SetSectionRight(section1Copy);
		section1Copy->node0_ = &primaryNodeCopy;
		BezierNode *nodeRightCopy = NEW BezierNode();
		nodeRightCopy->CopyDataFieldsOnly(*(primaryNode.section1_->node1_));
		section1Copy->node1_ = nodeRightCopy;
		nodeRightCopy->section0_ = section1Copy;
		nodeRightCopy = NULL;
		section1Copy=NULL;
		}
	BezierNode& devNode = deviations->spline_->nodeList_[node];
	BezierNode& lowNode = lowLim->spline_->nodeList_[node];
	BezierNode& highNode = highLim->spline_->nodeList_[node];
	aList<int> tempSectionsToRecalcLnL;
	returnval = primaryNodeCopy.ChangeNodeParam(paramAtNode, ran, devNode, lowNode, highNode);//first check
	if(returnval==1 && primaryNodeCopy.section0_ != NULL){//check for legal change on left section
		primaryNodeCopy.section0_->SetBezierParams();
		returnval = primaryNodeCopy.section0_->bezierValuesInRange();
		if(returnval==1){
			tempSectionsToRecalcLnL.Append(primaryNodeCopy.section0_->ID_);//meanwhile record success in left section 
			}
		}
	if(returnval==1 && primaryNodeCopy.section1_ != NULL){//check for legal change on right section
		primaryNodeCopy.section1_->SetBezierParams();
		returnval = primaryNodeCopy.section1_->bezierValuesInRange();	
		if(returnval==1){
			tempSectionsToRecalcLnL.Append(primaryNodeCopy.section1_->ID_);//meanwhile record success in right section  
			}
		}
	if(returnval==1){//copy into the spline anatomy		
		primaryNode.CopyDataFieldsOnly(primaryNodeCopy);
		if(primaryNode.section0_ != NULL){
			primaryNode.section0_->CopyDataFieldsOnly(*(primaryNodeCopy.section0_));}
		if(primaryNode.section1_ != NULL){
			primaryNode.section1_->CopyDataFieldsOnly(*(primaryNodeCopy.section1_));}
		for(s=0;s<tempSectionsToRecalcLnL.len();++s){
			sec = tempSectionsToRecalcLnL[s];
			sectionsToRecalculate_lnL_[sec]=true;
			}//s
/*		for(s=2;s<(*variableParamLookupTable_)[paramNum].len();++s){
			sec = (*variableParamLookupTable_)[paramNum][s];
			CubicBezierSplineSection sectionCopy(spline_->sectionList_[sec],true);//contents deleted when destroyed
			if(spline_->sectionList_[sec].node0_ == &primaryNode){
					primaryNodeCopy.SetSectionLeft(&sectionCopy);}//looking at section to the left
				else{
					primaryNodeCopy.SetSectionRight(&sectionCopy);}//looking at section to the right
			inRange = sectionCopy.bezierValuesInRange();
			if(! inRange){
				returnval=0;
				break;}
			tempSectionsToRecalcLnL.Append(sec);//in range, so recalculate
			}//s
*/
		}//if
	//regardless of result, toss copies safely
	CubicBezierSplineSection *leftsection = primaryNodeCopy.section0_;
	CubicBezierSplineSection *rightsection = primaryNodeCopy.section1_;
	BezierNode *leftSectionsLeftNode=NULL, *rightSectionsRightNode=NULL;
	primaryNodeCopy.DetachSections();
	if(leftsection !=NULL) {//detach primaryNodeCopy
		leftSectionsLeftNode = leftsection->node0_;
		leftsection->DetachPointers();
		}
	if(rightsection !=NULL) {
		rightSectionsRightNode = rightsection->node1_;
		rightsection->DetachPointers();
		}
	if(leftSectionsLeftNode !=NULL){
		leftSectionsLeftNode->DetachSections();}
	if(	rightSectionsRightNode != NULL){
		rightSectionsRightNode->DetachSections();}
	delete leftSectionsLeftNode;
	delete rightSectionsRightNode;
	leftSectionsLeftNode = rightSectionsRightNode = NULL;
	delete leftsection;//section copies deleted
	delete rightsection;
	leftsection = rightsection = NULL;
	spline=NULL;//clears the reference only
	return returnval;
	}//end CubicBezierSplineBetaParamRec::ChangeParamChangeBezierParam(int, Ran3&, 
		//					ParamRec*, ParamRec*, ParamRec*, long double)




int CubicBezierSplineBetaParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits,long double weight){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	bool returnval;
	if(paramNum<ErrorDistParams()){
			CubicBezierSplineBetaParamRec* deviations = (CubicBezierSplineBetaParamRec*) paramDeviations->GetPointerToDerivedSelf();
		 	returnval= ChangeBeta(ran, deviations->beta()*weight,lowerLimits,upperLimits);
		 	}
		 else{
		 	returnval= ChangeBezierParam(paramNum, ran, paramDeviations, lowerLimits, upperLimits);}
	
	return (int)returnval;
	}//end CubicBezierSplineBetaParamRec::ChangeParam(int)




int CubicBezierSplineBetaParamRec::ChangeParams(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, 
							aList<int>& paramChoices, int& paramsToChooseAmong, 
							bool sequentialParamVariation, long double weight){
	int paramToChange;
	paramToChange = ParamRec::ChangeParams(ran, paramDeviations, lowerLimits, upperLimits, 
							paramChoices, paramsToChooseAmong, sequentialParamVariation, weight);
	return paramToChange;
	}//end CubicBezierSplineBetaParamRec::ChangeParams(Ran3&, ParamRec*, ParamRec*, ParamRec*, aList<int>&, long double, int&, bool)





int CubicBezierSplineBetaParamRec::ChangeBeta(Ran3& ran, long double sigmaForNewBeta, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int sec, validChange=false, i, maxTries=10;
	long double oldBeta = beta_;
	for(i=0;i<maxTries;++i){
		beta_ = ran.NormDev(oldBeta, sigmaForNewBeta);
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		if(validChange) break;
		}
	if(validChange){
			for(sec=0;sec<sectionsToRecalculate_lnL_.len();++sec){
				sectionsToRecalculate_lnL_[sec] = true;
				}
			}
		else{
			beta_=oldBeta;
			}
	return validChange;
	}//end CubicBezierSplineBetaParamRec::ChangeSigma(Ran3&, long double, ParamRec*, ParamRec*)



void CubicBezierSplineBetaParamRec::CheckToleranceFactor(long double newFactor){
	if(newFactor != toleranceFactor_){
		SetToleranceFactor(newFactor);}
	}
	


long double CubicBezierSplineBetaParamRec::ExpectedValue(aList<long double>& coords){
	return ExpectedValue(coords,toleranceFactor_);
	}//end CubicBezierSplineBetaParamRec::ExpectedValue(aList<double>&)



long double CubicBezierSplineBetaParamRec::ExpectedValue(aList<long double>& coords, long double toleranceFactor){
	CheckToleranceFactor(toleranceFactor);
	return spline_->FunctionValue(coords[0],toleranceFactor);
	}//end CubicBezierSplineBetaParamRec::ExpectedValue(aList<double>&)



void CubicBezierSplineBetaParamRec::FindMinMax(int ss,CoordinateClass<long double>& coordsMinX, 
													CoordinateClass<long double>& coordsMaxX){
	//finds minimum and maximum y values for the spline
	//sets derivative to zero -> quadratic yields x values; plug-in gives y values
	bool legal=true;
	if(spline_==NULL){
		legal=false;}
	if(legal){
		long double a,b,c, xmin,xmax, y1,y2;
		CubicBezierSplineSection& cbss = spline_->sectionList_[ss];
		c = three * (cbss.cp_[1][0] - cbss.cp_[0][0]);
		b = three * (cbss.cp_[2][0] - cbss.cp_[1][0]) - c;
		a = cbss.cp_[3][0] - cbss.cp_[0][0] - b - c;
		
		xmin = (-b - sqrt(b*b - four*a*c))/(three*a);
		xmax = (-b + sqrt(b*b - four*a*c))/(three*a);
		if(xmin>xmax){SWAP(xmin,xmax);}
		y1 = cbss.CalculateBezierValue(xmin,toleranceFactor_);
		y2 = cbss.CalculateBezierValue(xmax,toleranceFactor_);
		coordsMinX.set(xmin,y1);
		coordsMaxX.set(xmax,y2);
		}//legal
	}//end CubicBezierSplineBetaParamRec::FindMinMax(int,CoordinateClass<long double>&,CoordinateClass<long double>& )
	


int CubicBezierSplineBetaParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
//	//use type conversion that circumvents usual virtual base class restrictions
	CubicBezierSplineBetaParamRec* lower = (CubicBezierSplineBetaParamRec*) lowerLimits->GetPointerToDerivedSelf();
	CubicBezierSplineBetaParamRec* upper = (CubicBezierSplineBetaParamRec*) upperLimits->GetPointerToDerivedSelf();
	int returnval=1, ss, rr, offendingSection=-1;
	long double y, yScaled, alpha;
	if(beta_ <= lower->beta_){
		returnval = 0;}
	if(returnval!=0 && spline_ != NULL){//used cbss.realizedRange_ values to check alpha levels at range extremes
//		for(ss=0;ss<sectionsToRecalculate_lnL_.len();++ss){//error->this is not reset before this criteria check
		for(ss=0;ss<spline_->sectionList_.len();++ss){
			CubicBezierSplineSection& cbss = spline_->sectionList_[ss];
			returnval = (int) cbss.bezierValuesInRange();
			if(returnval==0) {
				offendingSection=ss;
				break;
				}
			for(rr=0;rr<cbss.realizedRange_.len();++rr){
				y=cbss.realizedRange_[rr];
				yScaled=(y-rangeLowBound_)/(rangeHighBound_-rangeLowBound_);
//				yScaled=y;
				if(yScaled>=half){
//						alpha = yScaled*beta_/(one-yScaled);
						alpha = (yScaled*(beta_-two)+one)/(one-yScaled);//using mode
						}
					else{
//						alpha = (one-yScaled)*beta_/yScaled;
						alpha = ((one-yScaled)*(beta_-two)+one)/yScaled;//using mode
						}
				if(alpha<lower->alphaLow_ || alpha>upper->alphaHigh_){
					returnval=0;
					break;
					}
				}//rr
			if (returnval==0) break;
			}//ss
		}
	return returnval;
	}//end CubicBezierSplineBetaParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)



void CubicBezierSplineBetaParamRec::MultiplyParam(int paramNum, long double factor){
	//use to rescale a param deviation
	if(paramNum==0){
	 		 if(factor>zero){
	 		 	beta_ *= factor;}//check for further constraints on  elsewhere
	 		 	}
	 	 else{
			int node =(*variableParamLookupTable_)[paramNum][0];
			int paramAtNode =(*variableParamLookupTable_)[paramNum][1];
			BezierNode& primaryNode = spline_->nodeList_[node];
			primaryNode.MultiplyParam(paramAtNode,factor);
			}
	}//CubicBezierSplineBetaParamRec::MultiplyParam(int, double)



int CubicBezierSplineBetaParamRec::NumParams(void){
	//number of parameters that vary
//	return ErrorDistParams() + variableParamLookupTable_->len();
	int num=0;
	if(variableParamLookupTable_ != NULL){
		num=variableParamLookupTable_->len();}
	return num;
	}//end CubicBezierSplineBetaParamRec::NumParams()


int CubicBezierSplineBetaParamRec::NumDerivedParams(void){
	//number of parameters that are functions of varying parameters 
	//override in derived classes
	return 0;
	}//CubicBezierSplineBetaParamRec::NumDerivedParams()



aString* CubicBezierSplineBetaParamRec::ParamName(int paramNum){
	aString *returnStr=NEW aString();
	if(paramNum==0){
		 	*returnStr = "beta";}
		 else{
			int node = (*variableParamLookupTable_)[paramNum][0];
			int paramAtNode = (*variableParamLookupTable_)[paramNum][1];
			*returnStr=spline_->ParamName(node, paramAtNode);
			}
	return returnStr;
	}//end CubicBezierSplineBetaParamRec::ParamName(int )



void CubicBezierSplineBetaParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::ParamString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat(beta_,6,10);
	int pp, node, paramAtNode;
	for(pp=ErrorDistParams();pp<variableParamLookupTable_->len();++pp){
		dataline.cat(sepchar);
		node = (*variableParamLookupTable_)[pp][0];
		paramAtNode = (*variableParamLookupTable_)[pp][1];
		dataline.cat("node_");dataline.cat(node);dataline.cat(".");
		spline_->nodeList_[node].ParamName(paramAtNode, dataline);
		}//pp
	}//end CubicBezierSplineBetaParamRec::ParamString(aString&, char)



long double CubicBezierSplineBetaParamRec::ParamValue(int paramNum){
	//returns value of parameter	
	long double val;
	if(paramNum==0){ 
			val=beta();}
		else{
			int node=(*variableParamLookupTable_)[paramNum][0];
			int paramAtNode = (*variableParamLookupTable_)[paramNum][1];
			val = spline_->nodeList_[node].ParamValue(paramAtNode);
			}
	return val;
	}//end CubicBezierSplineBetaParamRec::ParamValue(int )


void CubicBezierSplineBetaParamRec::Print(ostream& outFile, long rep, aString& message){
	ParamRec::Print(outFile, rep, message);
	aString outStr("beta: ");
	outStr.cat(beta(),5,8);
	outFile<<outStr;
	outStr.clear();
	int pp,node, paramAtNode,prevNode=(*variableParamLookupTable_)[0][0];;
	for(pp=ErrorDistParams();pp<variableParamLookupTable_->len();++pp){
		node = (*variableParamLookupTable_)[pp][0];
		paramAtNode = (*variableParamLookupTable_)[pp][1];
		if(node != prevNode){
			outFile<<outStr<<endl;
			prevNode=node;
			outStr = "node: "; outStr.cat(node);
			}
		outStr.cat("\t");
		spline_->ParamName(node,paramAtNode,outStr);
		outStr.cat(": "); 
		outStr.cat(spline_->ParamValue(node,paramAtNode)); 
		}//pp	
	outFile<<outStr<<endl;
	}//CubicBezierSplineBetaParamRec::Print



int CubicBezierSplineBetaParamRec::SectionAt_x(long double x){
	int sec=0;
	if(spline_ != NULL){
		sec = spline_->SectionAt_x(x);}
	return sec;
	}//CubicBezierSplineBetaParamRec::SectionAt_x(long double )
	
	
void CubicBezierSplineBetaParamRec::SectionsAtNodes(aList<int>& nodelist, aList<int>& newSectionList){
	if(spline_ != NULL){
		spline_->SectionsAtNodes(nodelist,newSectionList);
		}
	}//CubicBezierSplineBetaParamRec::SectionAt_x(long double )
	

void CubicBezierSplineBetaParamRec::SetAlphaRange(long double alphaLow, long double alphaHigh){
	alphaLow_=alphaLow;
	alphaHigh_=alphaHigh;
	}

void CubicBezierSplineBetaParamRec::SetDataLocationsBySection(ObjList<aList<int> > *newLocations){
	//copies contents
	if( dataLocationsBySection_ == NULL){
			dataLocationsBySection_ = NEW ObjList<aList<int> >(*newLocations);}
		else{
			*dataLocationsBySection_ = *newLocations;}
	}//CubicBezierSplineBetaParamRec::SetDataLocationsBySection(ObjList<aList<int> >*)

void CubicBezierSplineBetaParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter
	if(paramNum==0){
			beta(newVal);}
	 	else{
	 		int node, paramAtNode;
			node = (*variableParamLookupTable_)[paramNum][0];
			paramAtNode = (*variableParamLookupTable_)[paramNum][1];
			spline_->SetParamValue(node,paramAtNode,newVal);
			}//else
	}//end CubicBezierSplineBetaParamRec::SetParamValue(int,double)




void CubicBezierSplineBetaParamRec::SetParamValueAllNodes(int paramNumAtNode, long double newVal){
	//use for setting deviations & upper/lower limits
	//set value of parameter
	if(spline_ != NULL){
 			spline_->SetParamValueAllNodes(paramNumAtNode,newVal);		
		}
	}//end CubicBezierSplineBetaParamRec::SetParamValue(int,long double)


void CubicBezierSplineBetaParamRec::SetRangeBoundaries(long double lowbound, long double highbound){
	rangeLowBound_ = lowbound;
	rangeHighBound_ = highbound;
	}


void CubicBezierSplineBetaParamRec::SetToleranceFactor(long double newFactor){
	toleranceFactor_=newFactor;
	if(spline_ != NULL){
		spline_->SetToleranceFactor(newFactor);
		}
	}


void CubicBezierSplineBetaParamRec::SetVariableParamLookupTable(ObjList<aList<int> > *vplt){
	variableParamLookupTable_ = vplt;
	}//CubicBezierSplineBetaParamRec::SetVariableParamLookupTable(ObjList<aList<int> >*)



long double CubicBezierSplineBetaParamRec::beta(void){
	return beta_;}	


void CubicBezierSplineBetaParamRec::beta(long double b){
	beta_=b;}	



void CubicBezierSplineBetaParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::TitleString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat("beta");
	int pp,node,paramAtNode;
	for(pp=ErrorDistParams();pp<variableParamLookupTable_->len();++pp){
		node = (*variableParamLookupTable_)[pp][0];
		paramAtNode = (*variableParamLookupTable_)[pp][1];
		dataline.cat(sepchar);
		dataline.cat("node_");
		dataline.cat((*variableParamLookupTable_)[pp][0]);		
		dataline.cat(".");
		spline_->ParamName(node, paramAtNode, dataline);
		}//pp	
	}//end CubicBezierSplineBetaParamRec::TitleString(aString&, char)







int	operator==(CubicBezierSplineBetaParamRec& p, CubicBezierSplineBetaParamRec& q){
	ParamRec& a = p;
	ParamRec& b = q;
	return ((a == b) && (p.beta_ == q.beta_) && (p.spline_==q.spline_) 
				&& (p.variableParamLookupTable_==q.variableParamLookupTable_));
	}//end operator==(CubicBezierSplineBetaParamRec&, CubicBezierSplineBetaParamRec&)


int	operator!=(CubicBezierSplineBetaParamRec& p, CubicBezierSplineBetaParamRec& q){
	return !(p==q);
	}//end operator!=(CubicBezierSplineBetaParamRec&, CubicBezierSplineBetaParamRec&)



#endif


