/**************************************************
*
*	MetropolisHastings.cpp
*
*	This class optimizes a parameter estimate given
*	a likelihood function and a data set.  It is intended  
*	to be generic, and derived classes of the likelihood function,
*	data set, and parameter classes needed.
*
**************************************************/

#ifndef MetropolisHastingscpp
#define MetropolisHastingscpp = 1

#include "MetropolisHastings.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "SortList.cpp"

MetropolisHastings::MetropolisHastings(void):
		data(0), 
		paramsInUse(0), paramsToTry(0),bestParams(0),
		paramDeviations(0),origParamDeviations(0),
		lowerConstraints(0),upperConstraints(0),
		lower2unitLimits(0),upper2unitLimits(0),
		totalTries(0), temperature((long double)100.0), 
		annealedTemperature((long double)10.0),annealTempSteps((long double)0.9),repsBetweenAnnealSteps(250),
		savingNow(false), annealed(false), supportLimitsCalculated(false),
		repsToSave_(2000){
//	(*LogLikelihood) = EquallyLikely;//default -- need to change before using
	LogLikelihood = EquallyLikely;//default -- need to change before using
	repsBetweenSaves_=50;
	}//end MetropolisHastings::MetropolisHastings(ParamRec&, ParamRec&)
	
	
	

MetropolisHastings::MetropolisHastings(
			ParamRec& initialParams, ParamRec& initParamDeviations,
			ParamRec& lowerParamLimits, ParamRec& upperParamLimits,
							MetropolisHastingsData* theData):
		data(0), 
		paramsInUse(0), paramsToTry(0),bestParams(0),
		paramDeviations(0),origParamDeviations(0),
		lowerConstraints(0),upperConstraints(0),
		lower2unitLimits(0),upper2unitLimits(0),
		totalTries(0), temperature((long double)100.0), 
		annealedTemperature((long double)10.0),annealTempSteps((long double)0.9),repsBetweenAnnealSteps(250),
		savingNow(false), annealed(false), supportLimitsCalculated(false),
		repsToSave_(2000){
	data = theData;
//	(*LogLikelihood) = EquallyLikely;//default -- need to change before using
	LogLikelihood = EquallyLikely;//default -- need to change before using

	paramsInUse = initialParams.Duplicate();//copies derived class info properly
	paramsToTry = initialParams.Duplicate();
	bestParams = initialParams.Duplicate();
	paramDeviations = initParamDeviations.Duplicate();
	origParamDeviations = initParamDeviations.Duplicate();
	lowerConstraints =  lowerParamLimits.Duplicate();
	upperConstraints =  upperParamLimits.Duplicate();
	int p;
	for(p=0;p<initialParams.NumParams();++p){
		paramChangesAccepted.Append(0);
		paramChangesTried.Append(0);
		runOfSuccessfulChanges.Append(0);
		runOfFailedChanges.Append(0);
		}//end for p
	repsBetweenSaves_=50;
	}//end MetropolisHastings::MetropolisHastings(ParamRec&, ParamRec&)
	
	
	
MetropolisHastings::~MetropolisHastings(void){
	data = 0;//forgotten here; must destroy elsewhere
  	LogLikelihood = NULL;//forget but do not delete
	delete paramsInUse; paramsInUse=0;
	delete paramsToTry; paramsToTry=0;
	delete bestParams; bestParams=0;
	delete paramDeviations; paramDeviations=0;
	delete origParamDeviations; origParamDeviations=0;
	delete lowerConstraints; lowerConstraints=0;
	delete upperConstraints; upperConstraints=0;
	delete lower2unitLimits; lower2unitLimits=0;
	delete upper2unitLimits; upper2unitLimits=0;
	paramChangesAccepted.Clear();
	paramChangesTried.Clear();
	runOfSuccessfulChanges.Clear();
	runOfFailedChanges.Clear();
	}//end MetropolisHastings::~MetropolisHastings()


	
void MetropolisHastings::Initialize(
			ParamRec& initialParams, ParamRec& initParamDeviations,
			ParamRec& lowerParamLimits, ParamRec& upperParamLimits,
							MetropolisHastingsData* theData){
	Clear();
	temperature = (long double)100.0;
	annealedTemperature = (long double)10.0;
	annealTempSteps = (long double)0.9; 
	repsBetweenAnnealSteps = 250;
	savingNow = false;
	annealed = false;
	supportLimitsCalculated = false;
	repsToSave_ = 2000;
	repsBetweenSaves_=50;
	data = theData;
//	(*LogLikelihood) = EquallyLikely;//default -- need to change before using
	LogLikelihood = &EquallyLikely;//default -- need to change before using

	paramsInUse = initialParams.Duplicate();//copies derived class info properly
	paramsToTry = initialParams.Duplicate();
	bestParams = initialParams.Duplicate();
	paramDeviations = initParamDeviations.Duplicate();
	origParamDeviations = initParamDeviations.Duplicate();
	lowerConstraints =  lowerParamLimits.Duplicate();
	upperConstraints =  upperParamLimits.Duplicate();
	int p;
	for(p=0;p<initialParams.NumParams();++p){
		paramChangesAccepted.Append(0);
		paramChangesTried.Append(0);
		runOfSuccessfulChanges.Append(0);
		runOfFailedChanges.Append(0);
		}//end for p
	}//end MetropolisHastings::MetropolisHastings(ParamRec&, ParamRec&)



void MetropolisHastings::Anneal(Ran3& ran, 
			long double startTemperature, long double endTemperature, bool reportProgress, long double tempStep,
			long repsBetweenAnnealings, bool sequentialParamVariation){
	annealedTemperature = endTemperature;//narrowest sampling range when annealing
	annealTempSteps = tempStep;//temperature to drop at each annealing step; 0.9 usually
	repsBetweenAnnealSteps = repsBetweenAnnealings;
	long double Lcurrent = LogLikelihood(*paramsInUse, data);
	long double Lnew, Lmax = Lcurrent;
	*bestParams = *paramsInUse;
	supportLimitsCalculated=false;
	*paramDeviations = *origParamDeviations;
	long annealTries = 0;
	aString printMessage("INITIAL VALUES");
	if(reportProgress){
		bestParams->Print(cout, annealTries, printMessage);cout<<endl;}
	savingNow = false;
	long i = 0, j, k=0, p;
	int allDone = false;
	long double temperatureNow = startTemperature;
	int numParamsToChooseAmong=0;//used for sequential parameter sampling; dummy if that's turned off
	aList<int> paramChoices;//used for sequential parameter sampling; dummy if that's turned off
	for(j=0;j<paramsInUse->NumParams();++j){
		paramChoices.Append(j);}
	printMessage = "ANNEALING FOR BEST VALUES";
	do{
		if(i >= repsBetweenAnnealSteps){
			temperatureNow *= annealTempSteps;
			i = 1;
			if(reportProgress){cout<<endl;}
			*paramsInUse = *bestParams;
			for(p=0;p<paramDeviations->NumParams();++p){//narrow the sampling range
				paramDeviations->MultiplyParam(p,annealTempSteps);}
			}
		*paramsToTry = *paramsInUse;
		if(annealTries==1403){
			k+=i;
			}
		int paramChanged = paramsToTry->ChangeAnnealingParams(ran, paramDeviations, lowerConstraints, upperConstraints, 
																paramChoices,numParamsToChooseAmong, sequentialParamVariation);
		if(paramChanged==-1){//error flag
				*paramsInUse=*bestParams;//restore to known starting place
				*paramDeviations = *origParamDeviations;
				}
			else{
				annealTries++;totalTries++;
				if(annealTries==30){
					++k;}
				Lnew = LogLikelihood(*paramsToTry, data);
				if(Lnew> -(long double)30.0){
					++k;}

				paramsToTry->SetLmaxRatio(Lmax);
				int switchParams = false;//if the fit is better, then save the new parameters
				if(paramsToTry->LtoLcurrentRatio(Lcurrent) <= one){
						//the fit is worse, but a switch is still possible
						if (ran() < paramsToTry->LtoLcurrentRatio(Lcurrent)){
							switchParams = true;//save them with a probability proportional to their L ratio
							}
						}
					else{//the fit is better, so switch automatically
						switchParams = true;
						if (Lnew > Lmax){//closer to zero since these are negative numbers
							Lmax = Lnew;
							paramsToTry->SetLmaxRatio(Lmax);
							*bestParams = *paramsToTry;
							if(reportProgress){
								bestParams->Print(cout, annealTries, printMessage);cout<<endl;}
							}//end if Lnew
						}//end if LtoLcurrentRatio < 1 ... else
				if(switchParams){
						*paramsInUse = *paramsToTry;
						Lcurrent = Lnew;
						}
				if (temperatureNow < annealedTemperature){
						allDone = true;}
				}//end if paramChanged...else
		if(sequentialParamVariation){
				if(numParamsToChooseAmong==0){
					++i;}
				}
			else{
				++i;}
		}while (! allDone);
	*paramsInUse = *bestParams;//reset these
	*paramsToTry = *paramsInUse;
	*paramDeviations = *origParamDeviations;
	annealed = true;
	}//end MetropolisHastings::Anneal(double, double, double)



void MetropolisHastings::Clear(void){
	data = 0;//forgotten here; must destroy elsewhere
  	LogLikelihood = NULL;//forget but do not delete
	delete paramsInUse; paramsInUse=0;
	delete paramsToTry; paramsToTry=0;
	delete bestParams; bestParams=0;
	delete paramDeviations; paramDeviations=0;
	delete origParamDeviations; origParamDeviations=0;
	delete lowerConstraints; lowerConstraints=0;
	delete upperConstraints; upperConstraints=0;
	delete lower2unitLimits; lower2unitLimits=0;
	delete upper2unitLimits; upper2unitLimits=0;
	paramChangesAccepted.Clear();
	paramChangesTried.Clear();
	runOfSuccessfulChanges.Clear();
	runOfFailedChanges.Clear();
	}//MetropolisHastings::Clear()



void MetropolisHastings::Find2unitSupportLimits(void){
	 //call after SampleForSupportLimits()
	if(supportLimitsCalculated){return;}
  	delete lower2unitLimits;lower2unitLimits=0;
  	delete upper2unitLimits;upper2unitLimits=0;
  	lower2unitLimits = bestParams->Duplicate();
  	upper2unitLimits = bestParams->Duplicate();
  	lower2unitDerivedParams.Clear(); 
  	upper2unitDerivedParams.Clear();
  	for(int p=0;p<bestParams->TotalParams();++p){
  		long double low = lower2unitLimits->ParamValue(p);
  		long double high = upper2unitLimits->ParamValue(p);
  		if(p>=lower2unitLimits->NumParams()){
			lower2unitDerivedParams.Append(low);
			upper2unitDerivedParams.Append(high);
			}//end if p
		for(int i=savedParamList.len()-1;i>=0;--i){
			long double val = savedParamList[i].ParamValue(p);
			if(val<low){
					low = val;
					if(p<lower2unitLimits->NumParams()){//it's a sampled parameter
							lower2unitLimits->SetParamValue(p, val);}
						else{//it's a derived parameter
							lower2unitDerivedParams.Last() = val;}
					}//end if val<low
				else if(val>high){
					high = val;
					if(p<upper2unitLimits->NumParams()){//it's a sampled parameter
							upper2unitLimits->SetParamValue(p, val);}
						else{//it's a derived parameter
							upper2unitDerivedParams.Last() = val;}
					}//end else if val>high
				else{}	
			}//end for i
		}//end for p
	supportLimitsCalculated = true;
	}//end MetropolisHastings::Find2unitSupportLimits()


void MetropolisHastings::FitOfTestParams(aString& outFileName){
	long double Lcurrent = LogLikelihood(*paramsInUse, data);
	long double Lmax = Lcurrent;
	*bestParams = *paramsInUse;
	aString printMessage("LIKELIHOOD OF TESTED PARAMETER VALUES");
	*paramsToTry = *paramsInUse;
	SaveParam(1,0);
	aString testOutFileName(outFileName);
	testOutFileName.trunc(27);
	testOutFileName.cat(".test");
	PrintSavedParams(testOutFileName);
	bestParams->Print(cout, 0, printMessage);cout<<endl;
	}//end MetropolisHastings::FitOfTestParams(aString& outFileName)
	

void MetropolisHastings::PrintParamEstimates(aString& outFileName, long seed, bool printSupport, int tabSeparated, aString& info){
	 //estimates & 2-unit support limits
	fstream outFile;
	char* rawfilename = outFileName.raw();
	outFile.open(rawfilename, ios::out);
	PrintParamEstimates(outFile,seed, printSupport, tabSeparated,info);
	outFile.close();
	delete [] rawfilename; rawfilename=0;
	}//end MetropolisHastings::PrintParamEstimates(aString&, int)



void MetropolisHastings::PrintParamEstimates(ostream& outfile, long seed, bool printSupport, int tabSeparated, aString& info){
	char sepChar=' ';
	if(tabSeparated) {sepChar='\t';}
	if(printSupport) Find2unitSupportLimits();
	outfile<<info<<endl<<endl;
	aString seedline("random number seed: ");
	seedline.cat(seed);
	outfile<<seedline<<endl;
	aString outLine("max lnL: ");outLine.cat(bestParams->lnL(),6,15);
	outfile<<outLine<<endl<<endl;
	int num = bestParams->NumParams();
	aString* paramName;
	for(int p=0;p<num;++p){
		paramName = bestParams->ParamName(p);
		paramName->cat(":");paramName->cat(sepChar);
		paramName->cat(bestParams->ParamValue(p),6,15);
		if(printSupport){
			paramName->cat(sepChar);
			paramName->cat("("); if(tabSeparated){ paramName->cat(sepChar);}
			paramName->cat(lower2unitLimits->ParamValue(p),6,15);
			paramName->cat(sepChar);paramName->cat("--");paramName->cat(sepChar);
			paramName->cat(upper2unitLimits->ParamValue(p),6,15);
			if(tabSeparated){ paramName->cat(sepChar);}
			paramName->cat(")");
			paramName->cat(sepChar);
			paramName->cat("sampling efficiency (%):");
			paramName->cat(sepChar);
			long double efficiency = zero;
			if(paramChangesTried[p]>0){
				efficiency = ((long double) paramChangesAccepted[p])/((long double)paramChangesTried[p]);}
			paramName->cat(100.0 * efficiency,3,3);
			}
		outfile<<*paramName<<endl;
		delete paramName; paramName=0;
		}//end for p
	for(int p=0;p<bestParams->NumDerivedParams();++p){
		paramName = bestParams->ParamName(num+p);
		paramName->cat(":");paramName->cat(sepChar);
		paramName->cat(bestParams->ParamValue(num+p),6,15);
		if(printSupport){
			paramName->cat(sepChar);
			paramName->cat("("); if(tabSeparated){ paramName->cat(sepChar);}
			paramName->cat(lower2unitDerivedParams[p],6,15);
			paramName->cat(sepChar);paramName->cat("--");paramName->cat(sepChar);
			paramName->cat(upper2unitDerivedParams[p],6,15);
			if(tabSeparated){ paramName->cat(sepChar);}
			paramName->cat(")");
			}
		outfile<<*paramName<<endl;
		delete paramName; paramName=0;
		}//end for p
	outfile<<endl<<endl<<endl;
	}//end MetropolisHastings::PrintParamEstimates(ostream&)





void MetropolisHastings::PrintSavedParams(aString& outFileName){
	fstream outFile;
	char* rawfilename = outFileName.raw();
	outFile.open(rawfilename, ios::out);
	PrintSavedParams(outFile);
	outFile.close();
	delete [] rawfilename; rawfilename=0;
	}//end MetropolisHastings::PrintSavedParams(aString&)

 
void MetropolisHastings::PrintSavedParams(ostream& outfile){
	savedParamList.Sort();
	aString colHeadings;
	bestParams->TitleString(colHeadings,'\t');
	outfile<<colHeadings<<endl;
	for(long i=savedParamList.len()-1;i>=0;--i){
		aString dataline;
		savedParamList[i].ParamString(dataline,'\t');
		outfile<<dataline<<endl;
		}//end for i
	}//end MetropolisHastings::PrintSavedParams(aString&)



void MetropolisHastings::SampleForSupportLimits(Ran3& ran, long double startTemperature, bool reportProgress, 
					long repsToSave, int repsBetweenSaves, bool sequentialParamVariation){
	aList<int> paramChoices;
	int numParamsToChooseAmong = 0;//used in sequential parameter sampling; reset on the fly when sampling
	temperature = startTemperature;//narrowest sampling range when annealing
	long double Lcurrent = LogLikelihood(*paramsInUse, data);
	long double Lmax = Lcurrent;
	*paramsInUse = *bestParams;
	*paramDeviations = *origParamDeviations;
	repsToSave_ = repsToSave;
	repsBetweenSaves_ = repsBetweenSaves;
	long sampleTries = 0;
	aString printMessage("SAMPLING FOR SUPPORT");
	cout<<printMessage<<endl<<endl;
	printMessage = "INITIAL BEST ESTIMATE";
	if(reportProgress){
		bestParams->Print(cout, sampleTries,printMessage);cout<<endl;}
	printMessage = "SAMPLING FOR SUPPORT: NEW BEST ESTIMATE";
	savingNow = true;
	*paramsToTry = *bestParams;
	SaveParam(1,1);//saves current best
	long i = 0;
	int allDone = false, rescale = false, savable = false;
	int consecutivePassesWithoutSaving = 0, consecutivePassesWithSameParamSet=0, lastParamSetLen, skippedSaves=0;
	long double temperatureNow = startTemperature;
	for(int p=0;p<paramChangesTried.len();++p){//reset counters
		paramChangesTried[p] = 0;
		paramChangesAccepted[p] = 0;
		runOfSuccessfulChanges[p] = 0;
		runOfFailedChanges[p] = 0;
		}//end for p
	do{
		savable=false;
		if(paramChoices.len()==0){
			for(int p=0;p<bestParams->NumParams();++p){
				paramChoices.Append(p);}
			lastParamSetLen = paramChoices.len();
			}
		*paramsToTry = *paramsInUse;
		int paramChanged = paramsToTry->ChangeParams(ran, paramDeviations, 
												lowerConstraints, upperConstraints,paramChoices,numParamsToChooseAmong,sequentialParamVariation);
		if(paramChanged==-1){//error flag
				*paramsInUse=*bestParams;//restore to known starting place
				*paramDeviations = *origParamDeviations;
				}
			else{
				paramChangesTried[paramChanged]++;
				sampleTries++; totalTries++;
				long double Lnew = LogLikelihood(*paramsToTry, data);
				
				paramsToTry->SetLmaxRatio(Lmax);
				int switchParams = false;//if the fit is better, save the new parameters
				if(paramsToTry->LtoLcurrentRatio(Lcurrent) <= 1.0){
						//the fit is worse, but a switch is still possible
						if (ran() < paramsToTry->LtoLcurrentRatio(Lcurrent)){
							switchParams = true;//save them with a probability proportional to their L ratio
							if(paramsToTry->Within2unitSupportLimits()){
								int paramLoc = paramChoices.Loc(paramChanged);
								paramChoices.Delete(paramLoc);//don't change this one until all others have been changed
								consecutivePassesWithSameParamSet=0;
								}
							}
						}
					else{//the fit is better, so switch automatically
						switchParams = true;
						if(paramsToTry->Within2unitSupportLimits()){
							int paramLoc = paramChoices.Loc(paramChanged);
							paramChoices.Delete(paramLoc);//don't change this one until all others have been changed
							consecutivePassesWithSameParamSet=0;
							}
						if (Lnew > Lmax){//closer to zero since these are negative numbers
							Lmax = Lnew;
							paramsToTry->SetLmaxRatio(Lmax);
							*bestParams = *paramsToTry;
							supportLimitsCalculated=false;
							rescale = true;
							aString mssg(printMessage);
							if(savingNow){//save new best no matter what
								allDone = SaveParam(1,rescale);
								rescale = false; savable=true; skippedSaves=0;
								mssg.cat(" "); mssg.cat(savedParamList.len());mssg.cat("/");mssg.cat(repsToSave_);
								paramChoices.Clear();//start over with these
								}
								if(reportProgress){
									bestParams->Print(cout, sampleTries, mssg);cout<<endl;}
							}//end if Lnew
						}//end if LtoLcurrentRatio < 1 ... else
				if(switchParams){
						*paramsInUse = *paramsToTry;
						Lcurrent = Lnew;
						paramChangesAccepted[paramChanged]++;//update counters
						runOfSuccessfulChanges[paramChanged]++;
						if(runOfSuccessfulChanges[paramChanged] > 0){
							runOfFailedChanges[paramChanged] = 0;}
						}
					else{//still need to update counters
						runOfFailedChanges[paramChanged]++;
						if(runOfFailedChanges[paramChanged] > 0){
							runOfSuccessfulChanges[paramChanged] = 0;}
						}//end if switchParams ... else
				if (runOfFailedChanges[paramChanged] > 7){//1% chance with sampling efficiency of 50%
						paramDeviations->MultiplyParam(paramChanged, annealTempSteps);//make the deviation a bit smaller
						runOfFailedChanges[paramChanged]=0;
						*paramsInUse = *bestParams;//go back to a known success
						paramChoices.Clear();//start over with these
						consecutivePassesWithoutSaving++;
						}
				if (savingNow && paramChoices.len()==0 && (! savable)){
					if(paramsToTry->Within2unitSupportLimits()){
							consecutivePassesWithoutSaving = 0;//savable, anyway
							savable=true;
							if(skippedSaves<repsBetweenSaves_){
									skippedSaves++;}
								else{
									allDone = SaveParam(0,rescale);
									skippedSaves=0;}
							}
						else{
							consecutivePassesWithoutSaving++;}
					}
				if (savable && runOfSuccessfulChanges[paramChanged] > 7){//1% chance with sampling efficiency of 50%
						paramDeviations->MultiplyParam(paramChanged, one/annealTempSteps);//make the deviation a bit bigger
						runOfSuccessfulChanges[paramChanged]=0;
						}
				if(paramChoices.len() == lastParamSetLen){
						consecutivePassesWithSameParamSet++;}
					else{
						lastParamSetLen = paramChoices.len();}
				rescale = false;
				if(consecutivePassesWithoutSaving>10 || consecutivePassesWithSameParamSet>10){
					//we're in a blind canyon
					*paramsInUse = *bestParams;//go back to a known success
					paramChoices.Clear();//start over with these
					consecutivePassesWithoutSaving = consecutivePassesWithSameParamSet = 0;
					}			
				}//end if paramChanged...else
		++i;
		}while (! allDone);
	*paramsInUse = *bestParams;//reset these
	*paramsToTry = *paramsInUse;
	}//end class MetropolisHastings::SampleForSupportLimits(double, long)


void MetropolisHastings::SetLikelihoodFunction (LikelihoodFunc theLikelihoodFunction){
//	( *LogLikelihood ) = theLikelihoodFunction;}
	LogLikelihood = theLikelihoodFunction;}


int MetropolisHastings::SaveParam(int newBest, int rescale){
	//saves paramsToTry and returns 1 if the list is full
	ParamRec* paramToSave = paramsToTry->Duplicate();
	if(newBest){//==1
			if(savedParamList.len()>0){
				savedParamList.DeleteFirst();}//replaces the best if it exists 
			savedParamList.Prepend(paramToSave);
			}
		else{
			savedParamList.Append(paramToSave);
			}
	supportLimitsCalculated=false;
	if(rescale){//cull the saved parameters that are outside the 2-unit support limits
		long double Lmax = bestParams->lnL();
		for(long i=savedParamList.len()-1;i>=0;--i){
			savedParamList[i].SetLmaxRatio(Lmax);
			if(savedParamList[i].Outside2unitSupportLimits()){
				savedParamList.Delete(i);}
			}
		}//end if rescale
	if(savedParamList.len()<repsToSave_){return 0;}
	return 1;
	}//end MetropolisHastings::SaveParam(int)
			
	 



	//use this class as a shared base class for any data set classes.
MetropolisHastingsData::MetropolisHastingsData(void){
//	SetPointerToDerivedSelf();
	}
	
MetropolisHastingsData::MetropolisHastingsData(MetropolisHastingsData& ){
//	SetPointerToDerivedSelf();
	}

//MetropolisHastingsData::~MetropolisHastingsData(void){
//					pointerToDerivedSelf_ = 0;}
	
MetropolisHastingsData* MetropolisHastingsData::Duplicate(void){
	MetropolisHastingsData *mhd = NEW MetropolisHastingsData(*this);
	return mhd;
	}

//void MetropolisHastingsData::SetPointerToDerivedSelf(void){//always override in derived
//	pointerToDerivedSelf_ = (long)this;
//	}

long MetropolisHastingsData::GetPointerToDerivedSelf(void){//always override in derived
	long pdsf = (long)this;
	return pdsf;
	}








/*default Likelihood function*/
long double EquallyLikely(ParamRec&, MetropolisHastingsData*){
	return zero;
	}//end EquallyLikely (ParamRec&, MetropolisHastingsData*)





#endif

