/*********************************************
*
*	ParamRec.cpp
*
*	This base class holds a set of parameter values that
*	are used in a metropolis algorithm for numerically 
*	maximizing an estimate.
*
*	Use derived classes to specify the names of the parameters,
*	and derive functions to vary these values and check
*	their limits.
*
*********************************************/
#ifndef ParamReccpp
#define ParamReccpp = 1

#include "ParamRec.h"
#include "Element.cpp"
#include "aList.cpp"

	
ParamRec* ParamRec::Duplicate(void){
	return NEW ParamRec(*this);
	}//end ParamRec::Duplicate()
	
//void ParamRec::SetPointerToDerivedSelf(void){
//  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in ParamRec"<<endl;
//  	}//end

	
long ParamRec::GetPointerToDerivedSelf(void){
	//handle case where OS moves pointers to new memory locations
  	long pdsf = (long) this;
  	return pdsf;
  	}//end


	

int ParamRec::ChangeAnnealingParams(Ran3& ran, //returns which parameter was changed
									ParamRec* paramVarationValues, 
									ParamRec* lowerLimits, ParamRec* upperLimits){
									//modify this function when some parameters already have known maxima
	return ChangeParams(ran,paramVarationValues,lowerLimits,upperLimits);}
									
int ParamRec::ChangeAnnealingParams(Ran3& ran, //returns which parameter was changed
					ParamRec* paramVarationValues, 
					ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& paramChoices, 
					int& paramsToChooseAmong, bool sequentialParamVariation){
					//modify this function when some parameters already have known maxima
	return ChangeParams(ran,paramVarationValues,lowerLimits,upperLimits,
													paramChoices,paramsToChooseAmong,sequentialParamVariation,one);}
									


int ParamRec::ChangeParam(int /*paramNum*/, Ran3& /*ran*/, 
							ParamRec* /*paramVarationValues*/, 
							ParamRec* /*lowerLimits*/, ParamRec* /*upperLimits*/, long double /*weight*/){
	//shouldn't arrive here; use derived classes
	aString errstr("bug: can't change a parameter in the base class");
	aError(errstr);
	return 0;//return 0 for failure, 1 for success
	}//end ParamRec::ChangeParam(int, aList<double>&)


int ParamRec::ChangeParams(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	int validchange = false, paramToChange,tries=0;
	do{
		paramToChange = RandomParameter(ran);
		validchange = ChangeParam(paramToChange, ran, 
										paramDeviations, lowerLimits, upperLimits);
		if(! validchange){
			tries++;
			if(tries>100){//bail out
				return -1;}
			double val = ParamValue(paramToChange);
			double devVal = paramDeviations->ParamValue(paramToChange);
			if(0.9*devVal >val/10000.0){//set a minimum deviation scaled to the parameter's value
			paramDeviations->MultiplyParam(paramToChange,0.9);}}
		}while(!(validchange));
	return paramToChange;
	}//end ParamRec::ChangeParams(Ran3&, ParamRec*, ParamRec*, ParamRec*)



int ParamRec::ChangeParams(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, 
							aList<int>& paramChoices, int& paramsToChooseAmong, 
							bool sequentialParamVariation, long double weight){
	int validchange = false, paramToChange, tries=0;
	do{
		if(sequentialParamVariation){
				paramToChange = RandomParameterSequential(ran,paramChoices,paramsToChooseAmong);}
			else{
				paramToChange = RandomParameter(ran,paramChoices);}
		validchange = ChangeParam(paramToChange, ran, 
										paramDeviations, lowerLimits, upperLimits, weight);
		if(! validchange){
			tries++;
			if(tries>100){//bail out
				return -1;}
			long double val = ParamValue(paramToChange);
			long double devVal = paramDeviations->ParamValue(paramToChange);
			if(0.9*devVal >val/10000.0){//set a minimum deviation scaled to the parameter's value
				paramDeviations->MultiplyParam(paramToChange,0.9);}
				}
		}while(!(validchange));
	return paramToChange;
	}//end ParamRec::ChangeParams(Ran3&, ParamRec*, ParamRec*, ParamRec*, aList<int>&, long double, int&, bool)




long double ParamRec::ExpectedValue(aList<long double>& /*coords*/){
	//shouldn't arrive here; use derived classes
	aString errstr("bug: can't calculate an expected value in the base class");
	aError(errstr);
	return (long double)0.0;
	}//end ParamRec::ExpectedValue(aList<double>&)


long double ParamRec::lnL(void){
		return lnL_;}
		
void ParamRec::lnL(long double lnLval){
		lnL_ = lnLval;}
		
long double ParamRec::lnLmaxMinusLnL(void){
		return lnLmaxMinusLnL_;}
		
long double ParamRec::LtoLcurrentRatio(long double Lcurrent){
		return exp(lnL_ - Lcurrent);}
		
long double ParamRec::LtoLmaxRatio(void){
		return exp(-lnLmaxMinusLnL_);}
		
int ParamRec::MeetsParamCriteria (ParamRec* /*lowerLimits*/, ParamRec* /*upperLimits*/){
	//shouldn't arrive here; use derived classes
	aString errstr("bug: can't check parameter criteria in the base class");
	aError(errstr);
	return 0;
	}//end ParamRec::MeetsParamCriteria(ParamRec*, paramRec*)


void ParamRec::MultiplyParam(int /*paramNum*/, long double /*factor*/){
	//use to rescale a param deviation
	aString errstr("bug: can't rescale a parameter in the base class");
	aError(errstr);
	}//ParamRec::MultiplyParam(int, double)


int ParamRec::NumDerivedParams(void){
	//number of parameters that are functions of varying parameters 
	//override in derived classes
	return 0;
	}//ParamRec::NumDerivedParams()


int ParamRec::NumParams(void){
	//number of parameters that could vary
	aString errstr("bug: asking for number of parameters in the base class");
	aError(errstr);
	return 0;
	}//ParamRec::NumParams()


int ParamRec::Outside2unitSupportLimits(void){
	return (! Within2unitSupportLimits());
	}//end ParamRec::Outside2unitSupportLimits()


aString* ParamRec::ParamName(int /*paramNum*/){
	//returns value of parameter
	//shouldn't arrive here; use derived classes
	aString errstr("bug: can't return a parameter's value from the base class");
	aError(errstr);
	return NEW aString();
	}//end ParamRec::ParamName(int )



void ParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	dataline.cat(lnL_,6,15);
	dataline.cat(sepchar);
	dataline.cat(lnLmaxMinusLnL_,6,15);
	}//end ParamRec::ParamString(aString&, char)


long double ParamRec::ParamValue(int /*paramNum*/){
	//returns value of parameter
	//shouldn't arrive here; use derived classes
	aString errstr("bug: can't return a parameter's value from the base class");
	aError(errstr);
	return (long double) 0.0;
	}//end ParamRec::ParamValue(int )



void ParamRec::Print(ostream& outFile, long rep){
	aString message;
	Print(outFile,rep,message);
	}//ParamRec::Print



void ParamRec::Print(ostream& outFile, long rep, aString& message){
	aString outStr;
	if(rep>0){outStr.cat(rep);}
	outStr.PrependCharToLen(' ', 7);
	outStr.cat ("  lnL: ");
	outStr.cat(lnL_,6,15);
	if(message.len()>0){
		outStr.cat('\t');
		outStr.cat(message);
		}
	outFile<<outStr<<endl;
	}//ParamRec::Print



int ParamRec::RandomParameter(Ran3& ran){
	int ranparam = ran.longBetween(0,NumParams()-1);
	return ranparam;
	}//	ParamRec::RandomParameter()

int ParamRec::RandomParameter(Ran3& ran, aList<int>& paramChoices){
	int ranparam = ran.longBetween(0,paramChoices.len()-1);
	return paramChoices[ranparam];
	}//	ParamRec::RandomParameter()


int ParamRec::RandomParameterSequential(Ran3& ran, aList<int>& paramChoices, int& paramsToChooseAmong){
	//use for sequential param selection; paramsToChooseAmong decremented and paramChoices list elements swapped
	int ranParam, selectedParam,lastParam;
	if(paramsToChooseAmong == 0){
		paramsToChooseAmong = paramChoices.len();
		}
	ranParam = ran.longBetween(0,paramsToChooseAmong-1);
	selectedParam = paramChoices[ranParam];
	--paramsToChooseAmong;
	lastParam=  paramChoices[paramsToChooseAmong];
	paramChoices[paramsToChooseAmong] = selectedParam;//send to end of list
	paramChoices[ranParam] = lastParam;//put end param into the randomly-chosen slot
	return selectedParam;
	}//	ParamRec::RandomParameterSequential()


void ParamRec::SetLmaxRatio(long double Lmax){
	lnLmaxMinusLnL_ = Lmax - lnL_;
	}//end ParamRec::SetLmaxRatio(double)


void ParamRec::SetParamValue(int /*paramNum*/, long double /*newVal*/){
	//sets value of parameter
	//shouldn't arrive here; use derived classes
	aString errstr("bug: can't set a parameter's value from the base class");
	aError(errstr);
	}//end ParamRec::SetParamValue(int,double)



void ParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the column headings that will be printed to a big file
	dataline.cat("lnL");
	dataline.cat(sepchar);
	dataline.cat("lnLmax - LnL");
	}//end ParamRec::TitleString(aString&, char)


int ParamRec::TotalParams(void){
	return NumParams() + NumDerivedParams();}


int ParamRec::Within2unitSupportLimits(void){
	if(lnLmaxMinusLnL_ <= 2.0){return 1;}
	return 0;
	}//end ParamRec::Within2unitSupportLimits()

void ParamRec::operator=(ParamRec& p){
	lnL_= p.lnL_;
	lnLmaxMinusLnL_ = p.lnLmaxMinusLnL_;	
	}//end  ParamRec::operator=(ParamRec&)


int	operator==(ParamRec& p, ParamRec& q){
	return (p.lnL_ == q.lnL_);
	}//end operator==(ParamRec&, ParamRec&)


int	operator!=(ParamRec& p, ParamRec& q){
	return !(p==q);
	}//end operator!=(ParamRec&, ParamRec&)


int	operator>(ParamRec& p, ParamRec& q){
	return (p.lnL_ > q.lnL_);
	}//end operator>(ParamRec&, ParamRec&)


int	operator<(ParamRec& p, ParamRec& q){
	return (p.lnL_ < q.lnL_);
	}//end operator<(ParamRec&, ParamRec&)


int	operator>=(ParamRec& p, ParamRec& q){
	return (p.lnL_ >= q.lnL_);
	}//end operator>=(ParamRec&, ParamRec&)


int	operator<=(ParamRec& p, ParamRec& q){
	return (p.lnL_ <= q.lnL_);
	}//end operator<=(ParamRec&, ParamRec&)




#endif


