/******************************************************
*
*	CubicBezierSpline.h
*
*	Contains the parameters and functions required to fit a cubic Bezier spline to data,
*	assuming a normal distribution of error constant along the spline.
*	The spline is made up of one or more cubic Bezier sections, between nodes that are connected
*	to handles using a polar coordinate system.
*	The error is captured by sigma_; 
*
*	Use this as a base class for derived classes that define the function.
*
******************************************************/

#ifndef CubicBezierSplineh
#define CubicBezierSplineh = 1


#include "MemoryManager.h"
#include "InlineTemplates.h"
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "CoordinateClass.h"
#include "BezierNode.h"
#include "CubicBezierSplineSection.h"
#include "dataColumnRec.h"
#include "dataColumnRecNum.h"


class CubicBezierSpline{
		
  public:
	ObjList<CubicBezierSplineSection> sectionList_;//ordered sequentially, bounded by the nodes
	ObjList<BezierNode> nodeList_;//ordered sequentially
  	CoordinateClass<long double> xL_, xR_;	//starting and ending points for the spline
  	bool usingStored_x_;
//  	curveAtKnown_x_Current;//use this if there are multiple data points for a given x value
//  	ObjList<ObjList<CoordinateClass<long double> > > curveAtKnown_x;
  			//indexed [section][order of x]-->holding x,y
  			//x-coordinate held constant for a data set; y coords change with params
  			//handles this in a way that minimizes recalculation of known values
	bool deleteContentsOnDisposal_;//if true, then delete contents in ~CubicBezierSplineSection; otherwise delete elsewhere
  						
  	
  	
	
  public:
  	CubicBezierSpline(bool vdeleteContentsOnDisposal=false);
  	CubicBezierSpline(int numNodes, bool deleteContentsOnDisposal=false);
  	CubicBezierSpline(ObjList<BezierNode>& sortedNodes, bool deleteContentsOnDisposal=false);
	CubicBezierSpline(ObjList<BezierNode>& sortedNodes, aList<long double>& x_values, bool deleteContentsOnDisposal=false);
						//x_values--> indexed [ordered datum] ->holds all x 
	CubicBezierSpline(CubicBezierSpline& p, bool deleteContentsOnDisposal=true);
  	virtual ~CubicBezierSpline(void);

  	virtual CubicBezierSpline* 	Duplicate(void);
	virtual void 		operator=(CubicBezierSpline& p);
	virtual void 		Copy(CubicBezierSpline& p, bool deleteContentsOnDisposal=true);

  

//	virtual int 		ChangeBezierParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
//									ParamRec* paramDeviations,  
//									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=(long double)1.0);
	virtual void 		ChangeSplineParam(int node, int paramNum, long double newVal);
	virtual long double ExpectedValue(aList<long double>& coords,long double toleranceFactor);
	virtual long double FunctionValue(long double x, long double toleranceFactor/*=(long double)0.001*/);
	virtual void		GetCoords(int section,long double x, CoordinateClass<long double>& pointOnCurve,
									long double toleranceFactor = (long double)0.001);//sets pointOnCurve
	virtual void 		MultiplyParam(int node, int paramAtNode, long double factor);
	virtual int 		NumParams(void);//number of parameters
	virtual aString* 	ParamName(int node, int paramAtNode);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void	 	ParamName(int node, int paramAtNode, aString& appendToThis);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int node, int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, aString& message, char* sepchar = "\t");
								//output in polar format; one line per node
	virtual void		SectionsAtNodes(aList<int>& nodePositions, aList<int>& newSectionList);
	virtual void		RecalculateYvalues(int section);
	virtual void		RecalculateYvalues(aList<int>& sectionList);
	virtual int			SectionAt_x(long double x);
	virtual void		SetMathematicaFormString(aString& stringToFill);//used in a Mathematica notebook to print a spline graphic
	virtual void		SetMathematicaFormStringList(StringList& stringListToFill);//used in a Mathematica notebook to print a spline graphic
	virtual void 		SetParamValue(int node, int paramAtNode, long double newVal);//sets value of parameter
	virtual void 		SetParamValueAllNodes(int paramNumAtNode, long double newVal);
	virtual void		SetSplineConnections(void);
	virtual void 		SetStoredValues(void);//when parameters change
	virtual void 		SetStoredValues(int node);//when parameters change at that node
	virtual void 		SetStoredValues(aList<int>& sectionsAffected);//when parameters change in these sections
	virtual void 		SetStoredValues(BezierNode& node);//when parameters change in this node
//	virtual void 		SetStoredValues(ObjList<CubicBezierSplineSection>& sectionsAffected);//when parameters change in these sections
	virtual void 		SetStoredValuesLists(aList<long double>& xValues);
	virtual void 		SetStoredValuesLists(dataColumnRecNum<long double>& xValues);
	virtual void		SetYminMax(long double ymin, long double ymax);
	virtual void		SetYminMax(aList<long double>& ymin, aList<long double>& ymax);
	virtual void		SetToleranceFactor(long double newFactor);
	virtual inline long double xL(void) {return xL_[0];}
	virtual inline long double xR(void) {return xR_[0];}


	friend int	operator==(CubicBezierSpline& p, CubicBezierSpline& q);
	friend int	operator!=(CubicBezierSpline& p, CubicBezierSpline& q);

	friend class CubicBezierSplineSection;
	friend class BezierNode;
	friend class CubicBezierSplineParamRec;

	};//end class CubicBezierSpline



#endif