/******************************************************
*
*	CubicBezierSplineParamRec.h
*
*	Contains the parameters and functions required to fit a cubic Bezier spline to data,
*	assuming a normal distribution of error constant along the spline.
*
*
******************************************************/

#ifndef CubicBezierSplineParamRech
#define CubicBezierSplineParamRech = 1


#include "MemoryManager.h"
#include "InlineTemplates.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "Stringclass.h"
#include "CoordinateClass.h"
#include "CubicBezierSpline.h"
#include "CubicBezierSplineSection.h"
#include "BezierNode.h"
#include "ParamRec.h"
//#include "BezierVariable.h"

class CubicBezierSplineParamRec;
//class BezierVariable;

//variableParamLookupTable [paramNum][2] --> node, nodeparam= {x=0,y=1,angle=2,distL=3,distR=4}

class CubicBezierSplineParamRec: virtual public ParamRec{
  public:
		
  public:
	long double sigma_;//sampling variance
	CubicBezierSpline *spline_;
  	ObjList<aList<int> > *variableParamLookupTable_;//an external table, holding info to link to nodes and their params
  							//indexed [param], holds node (-1=sigma), paramInNode, sections affected (-1=all sections)
  	StatList<long double> lnLbySection_;
  	aList<int> sectionsToRecalculate_lnL_;
  	ObjList<aList<int> > *dataLocationsBySection_;//length of data
  	long double toleranceFactor_;
  						
  	
  	
	
  public:
  	CubicBezierSplineParamRec(void);
  	CubicBezierSplineParamRec(long double sig, CubicBezierSpline *spline, 
  							   ObjList<aList<int> > *variableParamLookupTable,
							   StatList<long double>& lnLbySection, aList<int>& sectionsToRecalculate_lnL,
							   ObjList<aList<int> > *dataLocationsBySection
							   );
	CubicBezierSplineParamRec(CubicBezierSplineParamRec& p);
  	virtual ~CubicBezierSplineParamRec(void);

  	virtual ParamRec* 				Duplicate(void);
  	CubicBezierSplineParamRec* 		_Duplicate(void);
	virtual void 					SetPointerToDerivedSelf(void);//always override in derived
	virtual long 					GetPointerToDerivedSelf(void);//always override in derived
									//handle case where OS moves pointers to new memory locations
	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

  	


	virtual int 		ChangeBezierParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=(long double)1.0);
	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=(long double)1.0);
	virtual int			ChangeParams(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, 
							aList<int>& paramChoices, int& paramsToChooseAmong, 
							bool sequentialParamVariation, long double weight=0);
			int 		ChangeSigma(Ran3& ran, long double sigmaForNewSigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void		CheckToleranceFactor(long double newFactor);
	virtual inline int	ErrorDistParams(void){return 1;}
	virtual long double ExpectedValue(aList<long double>& coords);
	virtual long double ExpectedValue(aList<long double>& coords, long double toleranceFactor);
	virtual int 		MeetsParamCriteria(ParamRec* /*lowerLimits*/, ParamRec* /*upperLimits*/);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumDerivedParams(void);//number of parameters that are functions of varying parameters 
	virtual int 		NumParams(void);//number of parameters
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual void		SectionsAtNodes(aList<int>& nodelist, aList<int>& newSectionList);
	virtual int			SectionAt_x(long double x);
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual void 		SetParamValueAllNodes(int paramNumAtNode, long double newVal);//use to set deviations & upper/lower limits
	virtual void		SetToleranceFactor(long double newFactor);
	virtual void		SetVariableParamLookupTable(ObjList<aList<int> > *vplt);
			long double sigma(void);//returns sigma_
			void 		sigma(long double s);//sets sigma_	
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file


	friend int	operator==(CubicBezierSplineParamRec& p, CubicBezierSplineParamRec& q);
	friend int	operator!=(CubicBezierSplineParamRec& p, CubicBezierSplineParamRec& q);

	friend class CubicBezierSplineSection;
	friend class BezierNode;
//	friend class BezierVariable;

	};//end class CubicBezierSplineParamRec



#endif